/******************************************************************************

File Name:           GUIInterface_EnergyMeter.c
Dependencies:    Microchip Peripheral Library
Processor:           PIC18F65J90
Company:             Microchip Technology, Inc.

Copyright 2010 Microchip Technology Inc.  All rights reserved.

Microchip licenses to you the right to use, modify, copy and distribute
Software only when embedded on a Microchip microcontroller or digital signal
controller that is integrated into your product or third party product
(pursuant to the sublicense terms in the accompanying license agreement).

You should refer to the license agreement accompanying this Software for
additional information regarding your rights and obligations.

SOFTWARE AND DOCUMENTATION ARE PROVIDED ?AS IS? WITHOUT WARRANTY OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE.
IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER
CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR
OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR
CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF
SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
(INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.

Author                                     Date                                                Revision History / Comments
------------------------------------------------------------------------------------------------------------------------
Harsha Jagadish              1 - Jan - 2011                                                    Initial Version v1.0
Craig King					15 - Feb - 2011													   Port to 65J90 v1.1
*******************************************************************************/
#include "HardwareProfile_EnergyMeter.h"
#include "RegDef_EnergyMeter.h"
#include "eeprom_EnergyMeter.h"

//#define USB_GUI
//#define On_Board_USB

#define CALIBRATION_BANK	0x200

COMM_FLAGS	CommFlags;

static UINT16 read_adr;

/******************* USART Communication Buffer *********************************************/
#pragma udata   COMM_UART=0x310
UINT8  *CommsBuffer=0x0310;
static UINT8	Protect_Data1[48];		//This array is used as protection from compiler assigning variables in this location for optimization
/******************* Temporary Data for internal computations *********************************************/
#pragma udata	TEMP_DATA=0x340
UINT8	*temp_ptr=0x0340;
static UINT8	Protect_Data2[48];		//This array is used as protection from compiler assigning variables in this location for optimization
UINT8*   temp_init = 0x310,*scr_pad=0x340;


UINT8	ASCII_2_Hex ( UINT8 data );
UINT8	PC_ASCII_To_Byte(UINT8 *in_data,UINT8 *out_data );

/************************************************************************************
 Function    : void Init_USART(void)

 Description : The function initializes the USART module on the device with defined baud rate
	
 Arguments   : None
 
 Remarks     : The function configures USART that is used in communication with CALIBRATION GUI interface
		  Code uses Peripheral Library support of PIC18 available with MCC18
**************************************************************************************/
void Init_USART(void)
{
UINT8 ClearBuffer;

//************** USART1 Configuration ***************************	
//		* High Baud rate selection
//		* Continuos reception mode
//		* Transmission and reception mode Enabled
//		* SPBRG = 0x81
//**************************************************************	
#if defined (USB_GUI)
	Open1USART( USART_BRGH_HIGH | USART_CONT_RX | USART_RX_INT_ON | USART_TX_INT_OFF , 0x0081);
#else	
	Open1USART( USART_BRGH_LOW | USART_CONT_RX | USART_RX_INT_ON | USART_TX_INT_OFF , 25);
#endif

#if defined (On_Board_USB)
	BAUDCON1bits.RXDTP=1;
	BAUDCON1bits.TXCKP=1;
#endif
	
	IPR1bits.TXIP = 0;			// Low priority for both transmit and receive
	IPR1bits.RCIP = 0;			// Low priority for both transmit and receive
	ClearBuffer = RCREG1;				//Read USART RCREG1 register to clear residual value to dummy varaible to avoid Buffer overflow condition
	PIR1bits.TXIF=0;				//Clear Transmission interrupt initially before loading any new value of data to be transmitted to GUI

	//Initialize all GUI Communication associated variables to default state of zero
	memset(&Protect_Data1,0,sizeof(Protect_Data1));
	memset(&Protect_Data2,0,sizeof(Protect_Data2));
	read_adr = 0;
}


/************************************************************************************
 Function    : void ProcessGUICommand(void)

 Description : The function implements firmware establishing a protocol of communication with 
		   CALIBRATION GUI using USART communication module. The function services the 
		   Transmission request frm GUI to update the registers to GUI. Depending upon command 
		   recieved from GUI, the function does the following:
			*	Calibrate registers to default value
			*	Enable transmission
			*	Load calibration values & Energy stored in EEPORM
			*	Store Calibration values & Energy to EEPROM
			*	Recieve calibration data to be loaded into registers in RAM
			*	Write the values of calibration registers in RAM to update GUI
			*	Echo back the command sent from GUI for the purpose of debugging
	
 Arguments   : None
 
 Remarks     : The function is called in main function to update the calibration registers from/to GUI
		  Code uses Peripheral Library support of PIC18 available with MCC18
**************************************************************************************/
void ProcessGUICommand(void)
{
//******************************* Define & Initialize local pointers *************************************
//		*	ptr1 - Initialized to point to a calibration register
//		*	USART_Buffer - pointer initialized to buffer area to store intially recieved/to be transmitted data before/after conversion to HEX/ASCII respectively
//*************************************************************************************************
UINT8 *ptr1, *USART_Buffer=0x310;
UINT8  num_bytes=0,temp_char=0;


// The below condition checks for completion of Reception of command from GUI and no transmission to be progress.
	if( (CommFlags.bits.TXInProgress==0) && (CommFlags.bits.RCDone==1) )
	{	
		PIE1bits.RCIE = 0;								//Disable all RC Interrupts temporarily to clear Common flag that holds the status of previous reception
		CommFlags.bits.TXInProgress = 1;				//Set Transmission progress flag inidcating that transmission process has begun		
		CommFlags.byte_val = 0;							//Clear All Common Flags
		PIE1bits.RCIE = 1;								//Enable all RC Interrupts back

//Check for any errors in transmission/reception process		
		if( (CommFlags.byte_val & 0xFC ) )
		{
			*USART_Buffer++ = 'F';						//Load buffer with Error indication code - 'FxxX' for transmission to GUI
			
			//Indicate the exact kind of error by checking for the errors and load appropriate error response code
			if(CommFlags.bits.BufOverflow)				
				*USART_Buffer++ = '2';					//'2' error response code indicates the trans buffer overflow 
			if(CommFlags.bits.RCOverrun)
				*USART_Buffer++ = '1';					//'1' error response code indicates the reception buffer overflow 
			if(CommFlags.bits.BufOverrun)
				*USART_Buffer++ = '3';					//'3' error response code indicates the buffer overrun
			*USART_Buffer++ = 'X';						//Load treminating charected 'X' for transmission code protocol
			USART_Buffer = temp_init;					//Initialize transmission pointer to the begining from which transmission pointer keeps incrementing till end of 'x' terminating chrecter on enabling TXIE for next transmission
			PIE1bits.TXIE = 1;							//Enable Tx Interrupts to send command to GUI
			return;				
		}
		else
		{
		//execute the below section if there were no errors that occured in previous transmission/reception
				switch(*USART_Buffer)				//check for the type of command sent from GUI and act accordingly by switching to that routine
				{
			        case    'C':					// (C) indicates to load the calibration registers to defalut value
							if(*++USART_Buffer=='X')			//Check for termination chrecter to complete the protocol integrity check
							{
								ConfigCalibrationRegsToDefault();			//Call function that loads the calibration registers to default value
								temp_ptr = scr_pad;					//initialize pointer to free area in RAM for any intermidiate reception from GUI
								USART_Buffer = temp_init;		//Initialize transmission pointer to the begining from which transmission pointer keeps incrementing till end of 'x' terminating chrecter on enabling TXIE for next transmission
								PIE1bits.TXIE = 1;				//Enable Tx Interrupts to send command to GUI
								return;
							}
							else
								USART_Buffer--;					//decrement buffer pointer to original location
							break;
							
					case    'E':					// (E) indicates to enable transmission
							*USART_Buffer = 'Q';
							PIE1bits.TXIE = 1;					//Enable Tx Interrupts to send command to GUI
							return;
							
					case    'L':			   		// (L) indicates to load calibration resgisters with calibration values stored in EEPROM
							if(*++USART_Buffer=='X')			//Check for termination chrecter to complete the protocol integrity check
							{
								/**********NOTE: Below function will efectively load calibration parameters from EEPROM only if the device is previously calibrated
								else the calibration registers will retain the default value that were loaded from previously called function ************************/
								EEPROM_Load_Error != LoadFromEEPROM(ADDRESS_OF_BYTES_CAL_REGS_STORED_IN_EEPROM,
								                                   (UINT8*)&PHASE_COMPENSATION,
								                                   NUMBER_OF_BYTES_CAL_REGS_STORED_IN_EEPROM);		//Load Calibration values from EEPROM to RAM
								EEPROM_Load_Error != LoadFromEEPROM(ADDRESS_OF_BYTES_ENERGY_STORED_IN_EEPROM,
								                                   (UINT8*)&ENERGY_ACT,
								                                   NUMBER_OF_BYTES_ENERGY_STORED_IN_EEPROM);		//Load Calibration values from EEPROM to RAM

								USART_Buffer = temp_init;		//Initialize transmission pointer to the begining from which transmission pointer keeps incrementing till end of 'x' terminating chrecter on enabling TXIE for next transmission						
								METER_CONSTANT_BY_100 = METER_CONSTANT/100;
								PIE1bits.TXIE = 1;				//Enable Tx Interrupts to send command to GUI
								return;
							}
							else
								USART_Buffer--;					//decrement buffer pointer to original location
							break;
							
					case    'R':			   		// (R) indicates to Read the RAM varialbels onto GUI. Processor upon this command copies contens of the RAM vraiables to trans buffer location
							if( PC_ASCII_To_Byte( ++USART_Buffer, temp_ptr) == 0xFF ) //check for tremination conditions and errors flags
								break;									//if error then break from the execution and wait later for another command from GUI
							//PC_ASCII_To_Byte function converts the ASCII values sent from the GUI to hex values that are compatible in the procesor
							read_adr = ((UINT16)(*temp_ptr++)<<8);		//get the initial read address from whihc varaibels in that location has to be read back to the GUI					
							read_adr |= (UINT16)(*temp_ptr++);			//get the initial read address from whihc varaibels in that location has to be read back to the GUI					
							read_adr += CALIBRATION_BANK;					//Add the offset address from which the varaiables are located in RAM
							num_bytes = *temp_ptr;						//get the number of bytes of data to be read back to the GUI
							if( num_bytes > 0x1D )						//if the number of bytes to be read back to GUI is greater than 0x1D, then protocol error occured
								break;
							USART_Buffer++;								//increment the pointer to point to next location
							USART_Buffer++;
							{
							ptr1 = (UINT8*)read_adr;			//initialize the pointer to read address from which varaibles are to be read back to GUI
							//convert bytes to be read back to GUI into ASCII values
								while(num_bytes--)
								{
									temp_char = ((*ptr1 & 0xF0)>>4);
									((temp_char) < 0x0A) ? (*++USART_Buffer = temp_char + '0'):(*++USART_Buffer = temp_char + 'A' - 0x0A);
									temp_char = (*ptr1++) & 0x0F;
									(temp_char < 0x0A) ? (*++USART_Buffer = temp_char + '0'):(*++USART_Buffer = temp_char + 'A' - 0x0A);
								}								
							}
							*++USART_Buffer = 'X';				//End the transmission data to be sent to GUI with a termination charecter to complete protocol
							temp_ptr = scr_pad;					//initialize pointer to free area in RAM for any intermidiate reception from GUI
							USART_Buffer = temp_init;			//initialize pointer to staring location from where transmission for GUI has to be started
							PIE1bits.TXIE = 1;					//Enable Tx Interrupts to send command to GUI
							return;

							
					case    'S':			   		// (S) inidcates to store the recent calibrated values in calibration registers to EEPROM for future power on retreival
							if(*++USART_Buffer=='X')			//Check for termination chrecter to complete the protocol integrity check
							{	
								ENERGY_ACT = 0;			//clear all energy accumulation value on calibration & storage
								RAW_ENERGY_ACT.dword.LD = 0;	//clear all energy accumulation value on calibration & storage
								RAW_ENERGY_ACT.dword.HD = 0;	//clear all energy accumulation value on calibration & storage
								ENERGY_REACT =0;
								RAW_ENERGY_REACT.dword.LD = 0;
								RAW_ENERGY_REACT.dword.HD = 0;

						    	StoreToEEPROM(ADDRESS_OF_BYTES_ENERGY_STORED_IN_EEPROM,
								              (UINT8*)&ENERGY_ACT,
								              NUMBER_OF_BYTES_ENERGY_STORED_IN_EEPROM);
								              
								Delay10KTCYx(100);
								              
						    	StoreToEEPROM(ADDRESS_OF_BYTES_CAL_REGS_STORED_IN_EEPROM,
								              (UINT8*)&PHASE_COMPENSATION,
								              NUMBER_OF_BYTES_CAL_REGS_STORED_IN_EEPROM);
								USART_Buffer = temp_init;		//initialize pointer to staring location from where transmission for GUI has to be started						
								METER_CONSTANT_BY_100 = METER_CONSTANT/100;
								EEPROM_Load_Error = FALSE;
								PIE1bits.TXIE = 1;				//Enable Tx Interrupts to send command to GUI
								return;
							}
							else
								USART_Buffer--;					//decrement buffer pointer to original location
							break;
							
					case    'W':			   		// (W) indicates "WRITE Operation" to write values from GUI to variables in RAM
							num_bytes = PC_ASCII_To_Byte(++USART_Buffer, temp_ptr); //get the number of bytes of data to be written to RAM after ASCII to hex conversion
							if(num_bytes == 0xFF)	//check for any error conidtions/ protocol violations
								break;
							else	
								num_bytes -= 2;		//if no error reduce number of bytes by 2 to remove the byte count of 2 bytes of address that is sent at begining
							
							read_adr = ((UINT16)(*temp_ptr++)<<8);	//get initial address in RAM from which values are to be written in RAM from GUI
							read_adr |= (UINT16)(*temp_ptr++);		//get initial address in RAM from which values are to be written in RAM from GUI
							read_adr += CALIBRATION_BANK;				//add offset value of address in RAM wheer variables are located
								
							USART_Buffer++;						//increment pointer to get the values from buffer to be written to RAM
							ptr1 = (UINT8*)read_adr;			//initialize another pointer to location in RAM
							while(num_bytes--)
							{
								*ptr1++ = *temp_ptr++;			//Write all the bytes of data to be written to RAM sequentially
							}
#ifndef USB_GUI							
							++USART_Buffer;
#endif
							*++USART_Buffer = 'X';				//End with termination charecter to send ACK to GUI
							temp_ptr = scr_pad;					//initialize pointer to free area in RAM for any intermidiate reception from GUI
							USART_Buffer = temp_init;			//initialize pointer to staring location from where transmission for GUI has to be started
							PIE1bits.TXIE = 1;					//Enable Tx Interrupts to send command to GUI
							return;

							
			       	default:
							*USART_Buffer = 'F';				//Send default fault code with error condition "10" back to GUI if the command recieved is invalid
							*USART_Buffer++ = '1';				//Send default fault code with error condition "10" back to GUI if the command recieved is invalid
							*USART_Buffer++ = '0';				//Send default fault code with error condition "10" back to GUI if the command recieved is invalid
							*USART_Buffer++ = 'X';				//End with termination charecter to send ACK to GUI
							USART_Buffer = temp_init;			//initialize pointer to staring location from where transmission for GUI has to be started
							PIE1bits.TXIE = 1;					//Enable Tx Interrupts to send command to GUI
							return;
							
				}
				*USART_Buffer = 'F';							//Send default fault code with error condition "11" back to GUI if the command recieved is invalid
				*USART_Buffer++ = '1';							//Send default fault code with error condition "11" back to GUI if the command recieved is invalid
				*USART_Buffer++ = '1';							//Send default fault code with error condition "11" back to GUI if the command recieved is invalid
				*USART_Buffer++ = 'X';							//End with termination charecter to send ACK to GUI
				USART_Buffer =temp_init;						//initialize pointer to staring location from where transmission for GUI has to be started
				PIE1bits.TXIE = 1;								//Enable Tx Interrupts to send command to GUI
				return;
								
		}		
	}


	return;
}


/************************************************************************************
 Function    : UINT8	PC_ASCII_To_Byte(UINT8 *in_data,UINT8 *out_data )

 Description : The Function converts  ASCII to HEX of values located in reception buffer
	
 Arguments   : in_data - Byte pointer to the data in ASCII from where need to be converted to HEX
		   out_data - Byte pointer where the HEX converted data from where has be stored back
		   
Returns:	The functions returns df=ifferent values depending upon the condition. If the successful 
		conversion occurs in without protocol violation and termination char is found after address
		and data bytes, then number of successful conversion bytes is returned.
		Else in caseof protocol violation like no address bytes & immediate termination char, then
		0xFF value is returned.
 
 Remarks     : The function converts number of bytes of data in reception buffer area till termination cahrecter
		  is found. 
**************************************************************************************/
#if defined (USB_GUI)
UINT8	PC_ASCII_To_Byte(UINT8 *in_data,UINT8 *out_data )
{
UINT8 num_bytes1=0;

	if( *in_data == 'X')		//check if the inital charecter itself is termination char. if so send error code
		return(0xFF);

	*out_data = ASCII_2_Hex(*in_data);	//Call function that subtracts 0x30 from ASCII value to convert to HEX
	
	if( *out_data == 0xFF )				//check if any error value returned incase of non-aplhabet & numeric value encounter
		return(0xFF);					//if so return error code
	else	
		num_bytes1++;						//else increment number of bytes converted

	do
	{
		if( *++in_data == 'X')			//check for termination char
			break;

		*++out_data = ASCII_2_Hex(*in_data);	//Call function that subtracts 0x30 from ASCII value to convert to HEX
		
		if( *out_data == 0xFF )			//check if any error value returned incase of non-aplhabet & numeric value encounter
			return(0xFF);				//if so return error code
			
		*out_data <<= 4 ;				//shift to convert the lower nibble now

		if( *++in_data == 'X')			//check for termination char
			break;

		*out_data |= ASCII_2_Hex(*in_data);	//check if any error value returned incase of non-aplhabet & numeric value encounter
		
		if( *out_data == 0xFF )			//check if any error value returned incase of non-aplhabet & numeric value encounter
			return(0xFF);				//if so return error code

		num_bytes1++;					//else increment number of bytes converted
	}
	while(1);							//continue conversion till the termination char or eror occurs
	
	//check for validity in number of bytes that were recieved and converted and if they dont confine with in protocol specifications, then return error code
	if( ( num_bytes1 < 0x03) || ( num_bytes1 > 0x1F ) )
		return(0xFF);
	else
		return(num_bytes1);
		
	
}
#else
UINT8	PC_ASCII_To_Byte(UINT8 *in_data,UINT8 *out_data )
{
UINT8 num_bytes1=0;

	if( *in_data == 'X')		//check if the inital charecter itself is termination char. if so send error code
		return(0xFF);

	if( *in_data == 0xFF )				//check if any error value returned incase of non-aplhabet & numeric value encounter
		return(0xFF);					//if so return error code

	*out_data = ASCII_2_Hex(*in_data);	//Call function that subtracts 0x30 from ASCII value to convert to HEX
	
		num_bytes1++;						//else increment number of bytes converted

	do
	{
		if( *++in_data == 'X')			//check for termination char
			break;

	if( *in_data == 0xFF )				//check if any error value returned incase of non-aplhabet & numeric value encounter
		return(0xFF);					//if so return error code


		*++out_data = ASCII_2_Hex(*in_data);	//Call function that subtracts 0x30 from ASCII value to convert to HEX
			
		*out_data <<= 4 ;				//shift to convert the lower nibble now

		if( *++in_data == 'X')			//check for termination char
			break;

	if( *in_data == 0xFF )				//check if any error value returned incase of non-aplhabet & numeric value encounter
		return(0xFF);					//if so return error code

		*out_data |= ASCII_2_Hex(*in_data);	//check if any error value returned incase of non-aplhabet & numeric value encounter
		
		num_bytes1++;					//else increment number of bytes converted
	}
	while(1);							//continue conversion till the termination char or eror occurs
	
	//check for validity in number of bytes that were recieved and converted and if they dont confine with in protocol specifications, then return error code
	if( ( num_bytes1 < 0x03) || ( num_bytes1 > 0x1F ) )
		return(0xFF);
	else
		return(num_bytes1);
		
	
}
#endif

/************************************************************************************
 Function    : UINT8	ASCII_2_Hex ( UINT8 data )

 Description : The Function converts the input value in ASCII to HEX.
	
 Arguments   : data - Byte of data in ASCII that need to be converted to HEX
		   
Returns:	HEX value equivalent of ASCII input
 
 Remarks     : Function returns error code of "0xFF" if the HEX value does not confine within the range "0-9 & A-F"
**************************************************************************************/
UINT8	ASCII_2_Hex ( UINT8 data )
{
	if( (data <= '9') && ( data >= '0' ) )
		data -= '0' ;
	else if ( (data <= 'F' ) && ( data >= 'A' ) )
		data = data - 'A' + 0x0A ;	
	else
		return(0xFF);	

	return(data);
}


