/******************************************************************************

File Name:           HardwareProfile_EnergyMeter.h
Dependencies:    Microchip Peripheral Library
Processor:           PIC18F65J90
Company:             Microchip Technology, Inc.

Copyright 2010 Microchip Technology Inc.  All rights reserved.

Microchip licenses to you the right to use, modify, copy and distribute
Software only when embedded on a Microchip microcontroller or digital signal
controller that is integrated into your product or third party product
(pursuant to the sublicense terms in the accompanying license agreement).

You should refer to the license agreement accompanying this Software for
additional information regarding your rights and obligations.

SOFTWARE AND DOCUMENTATION ARE PROVIDED ?AS IS? WITHOUT WARRANTY OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE.
IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER
CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR
OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR
CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF
SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
(INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.

Author                                     Date                                                Revision History / Comments
------------------------------------------------------------------------------------------------------------------------
Harsha Jagadish              1 - Jan - 2011                                                    Initial Version v1.0
*******************************************************************************/
#ifndef __HardwareProfile_EnergyMeter
#define __HardwareProfile_EnergyMeter

#define USE_OR_MASKS

#include <p18cxxx.h>
#include "math.h"
#include "GenericTypeDefs.h"

//**************** Peripheral Library files from MCC18 ******************************
#include "spi.h"
#include "timers.h"
#include "usart.h"
#include "pwm.h"
#include "delays.h"
#include "portb.h"
#include "lcd.h"
//*******************************************************************************


//******************************************************************************
// Clock Information
//******************************************************************************

//System Clock = 40MHz =  10MHz crystal x 4 PLL
//System Clock = 10MHz =  4MHz crystal x 4 PLL - FOR PIC18F65J90

//#define GetSystemClock()                    40000000UL
#define GetSystemClock()                    16000000UL
#define GetPeripheralClock()                (GetSystemClock() / 4)
#define GetInstructionClock()               (GetSystemClock() / 4)


//Signal used for DebuggingPurpose - direction setting
#define EM_DEBUG_SIG_DIR                             TRISDbits.TRISD6

//Chip select signal for "Sigma Delta ADC"  port direction selection 
#define EM_SDADC_CS_DIR                              TRISAbits.TRISA5

//Chip select signal for"Sigma Delta ADC"  port
#define EM_SDADC_CS                                  LATAbits.LATA5

//Chip select signal for EEPROM  port direction selection 
#define EM_EEPROM_CS_DIR                             TRISAbits.TRISA1

//Chip select signal for EEPROM  port
#define EM_EEPROM_CS                                 LATAbits.LATA1

// MCLR for"Sigma Delta ADC".
#define EM_SDADC_MCLR                                LATAbits.LATA0

// MCLR for"Sigma Delta ADC" direction selection
#define EM_SDADC_MCLR_DIR                            TRISAbits.TRISA0

//CF Pulse port used in IN-LINE ASSEMBLY code for Active Energy
#define EM_ASM_CF_PULSE_ACT                          LATA,3

//CF Pulse port used in IN-LINE ASSEMBLY code for Reactive Energy
#define EM_ASM_CF_PULSE_REACT                        LATA,2

//Chip select signal for ADC"Sigma Delta ADC"  port used in IN-LINE ASSEMBLY code
#define EM_ASM_SDADC_CS                              LATA,5

//CCP1 - PWM output pin direction selection
#define EM_PWM_CCP1_DIR                              TRISCbits.TRISC2

//CCP1 - PWM output pin
#define EM_PWM_CCP1                                  LATCbits.LATC2

// CF Pulse for Active Energy
#define	EM_CF_PULSE_ACT                              LATAbits.LATA3

//CF Pulse sginal direction selection for active energy
#define	EM_CF_PULSE_ACT_DIR                          TRISAbits.TRISA3

// CF Pulse for Reactive Energy
#define	EM_CF_PULSE_REACT                            LATAbits.LATA2

//CF Pulse sginal direction selection for Reactive energy
#define	EM_CF_PULSE_REACT_DIR                        TRISAbits.TRISA2

//Swicth to change the LCD display state to show various meter output
#define	LCD_SWITCH                                   PORTBbits.RB4

// CF pulse width is approximately 90 ms for which the CF Pulse should be on (81ms at 60Hz, 98 ms at 50 Hz) @ 10MHz crystal
#define CF_PULSE_WIDTH_VAL                           0x0A

//UART recption & Transmission buffers
#define COMM_RECEPTION_BUFFER						RCREG1
#define COMM_TRANSMISSION_BUFFER					TXREG1

//Clear Timer to zero
#define RESET_TIMER_COUNT							0x0000

//MCP SD-ADC Current channel Gain configuration
#define NOMINAL_CURR_CHANNEL_GAIN					8

//Set TIMER0 counter value such that roll over interrupt occurs every 1 seconds
#define SET_TIMER0_COUNTER_1SEC						((0x10000UL)-(GetPeripheralClock()/256))

#define PWM_CLOCK									(2000000UL)

#define PWM_COUNT									((GetPeripheralClock()/PWM_CLOCK)-1)
#define PWM_DUTY_CYCLE								4

#if ( ( GetPeripheralClock() / (PWM_COUNT+1)) != PWM_CLOCK )
#error "Expected PWM Clock cannot be derived from this System Clock"
#endif

//Timer prescale used in line frequency calculation = 1:4
#define FREQUENCY_TIMER_PRESCALE					4

#define MCP_OSR_REG				0x20							//OSR configured to 128

/****************************************************************************
  Macro:   EnableInterruptPriorityMode()

 Description:   Macro Enables the interrupt in priority mode

  Precondition:  None

  Parameters:   None

  Remarks: None
  ***************************************************************************/ 
 #define  EnableInterruptPriorityMode()				RCONbits.IPEN = 1
  
/****************************************************************************
  Macro:   EnableSystemPLL()

 Description:   Macro enables the controller PLL circuit for multiplying the clock frequency by 4 times

  Precondition:   Appropriate clock configuration must be done

  Parameters:   None

  Remarks: None
  ***************************************************************************/
#define  EnableSystemPLL()		OSCTUNEbits.PLLEN = 1


/****************************************************************************
  Macro:   ClearWDT()

 Description:   Macro clears the WDT timer

  Precondition:   WDT must be enabled prior to this

  Parameters:   None

  Remarks: Macro is called to avoid WDT time out causing system reset
  ***************************************************************************/
 #define  ClearWDT()			_asm clrwdt _endasm

 
/****************************************************************************
  Macro:   ConfigPortsForDigitalMode()

 Description:   Macro configures all ports to be digital in mode

  Precondition:   ADC Config register ADCON1 will be configured for digital mode

  Parameters:   None

  Remarks: None
  ***************************************************************************/ 
 #define  ConfigPortsForDigitalMode()		ADCON1 = 0x0F
  
/****************************************************************************
  Macro:   SDADCChipSelectConfigure()

 Description:   Macro configures SD-ADC Chip select pins to output & disabled

  Precondition:   Ports must be configured as digital

  Parameters:   None

  Remarks: None
  ***************************************************************************/ 
 #define SDADCChipSelectConfigure()			EM_SDADC_CS_DIR = 0;	\
											EM_SDADC_CS	= 1

/****************************************************************************
  Macro:   SDADCResetPinConfigure()

 Description:   Macro configures SD-ADC reset pins to output & disabled

  Precondition:   Ports must be configured as digital

  Parameters:   None

  Remarks: None
  ***************************************************************************/ 
 #define SDADCResetPinConfigure()				EM_SDADC_MCLR_DIR = 0;	\
												EM_SDADC_MCLR = 0
											

/****************************************************************************
  Macro:   EEPROMChipSelectConfigure()

 Description:   Macro configures EEPROM Chip select pins to output & disabled

  Precondition:   Ports must be configured as digital

  Parameters:   None

  Remarks: None
  ***************************************************************************/ 
 #define EEPROMChipSelectConfigure()		EM_EEPROM_CS_DIR = 0;	\
											EM_EEPROM_CS = 1
  
/****************************************************************************
  Macro:   ACTIVECFPulseConfigure()

 Description:   Macro configures Active energy CF Pulse pin to output & clear CF Pulse

  Precondition:   Ports must be configured as digital

  Parameters:   None

  Remarks: None
  ***************************************************************************/ 
 #define ACTIVECFPulseConfigure()				EM_CF_PULSE_ACT_DIR = 0;	\
												EM_CF_PULSE_ACT = 0
 
/****************************************************************************
  Macro:   REACTIVECFPulseConfigure()

 Description:   Macro configures Active energy CF Pulse pin to output & clear CF Pulse

  Precondition:   Ports must be configured as digital

  Parameters:   None

  Remarks: None
  ***************************************************************************/ 
 #define REACTIVECFPulseConfigure()				EM_CF_PULSE_REACT_DIR = 0;	\
												EM_CF_PULSE_REACT = 0

/****************************************************************************
  Macro:   UnusedPortsConfigure()

 Description:   Macro configures unused port pins to output & driven to low

  Precondition:   Ports must be configured as digital

  Parameters:   None

  Remarks: Unused port pin configuration to output & drive low reduces the power consumption
  ***************************************************************************/ 
 #define UnusedPortsConfigure()					TRISGbits.TRISG1 = 0;	\
												LATGbits.LATG1 = 0;		\
												//TRISAbits.TRISA3 = 0;	\
												//LATAbits.LATA3 = 0;		\
												//TRISAbits.TRISA2 = 0;	\
												//LATAbits.LATA2 = 0;		\
												//TRISAbits.TRISA0 = 0;	\
												//LATAbits.LATA0 = 0;		\
												//EM_DEBUG_SIG_DIR = 0
 
/****************************************************************************
  Macro:   DisableAllGlobalInterrupts()

 Description:   Macro disables all global interrupts

  Precondition:   None

  Parameters:   None

  Remarks: Disables High & Low priority interrupts
  ***************************************************************************/ 
 #define  DisableAllGlobalInterrupts()			INTCONbits.GIEH = 0;	\
												INTCONbits.GIEL = 0

/****************************************************************************
  Macro:   EnableAllGlobalInterrupts()

 Description:   Macro enables all global interrupts

  Precondition:   None

  Parameters:   None

  Remarks: Enables High & Low priority interrupts
  ***************************************************************************/ 
 #define  EnableAllGlobalInterrupts()			INTCONbits.GIEH = 1;	\
												INTCONbits.GIEL = 1


/****************************************************************************
  Macro:   CCPPWMPinConfig()

 Description:   Macro configures PWM pins to output and brief pulse clears the 
                        PLL lock detect when no signal is present and Again configure back 
                        the PWM pin to output state

  Precondition:   None

  Parameters:   None

  Remarks: None
  ***************************************************************************/ 
 #define  CCPPWMPinConfig()							EM_PWM_CCP1_DIR = 0;	\
													EM_PWM_CCP1 = 1;		\
													EM_PWM_CCP1 = 0

/****************************************************************************
  Macro:   ConfigSWTimerInterruptToLowPriority()

 Description:   Macro configures Timer 0 interrupt to low priority

  Precondition:   Interrupt priority must be enabled and Timer0 interrupt must be enabled

  Parameters:   None

  Remarks: None
  ***************************************************************************/ 
 #define  ConfigSWTimerInterruptToLowPriority()		INTCON2bits.TMR0IP = 0

 
 /****************************************************************************
  Macro:   PowerDownDetectConfig()

 Description:   Macro configures Power Down detect using LVD with high priority interrupt

  Precondition:   Interrupt priority must be enabled and set to high

  Parameters:   None

  Remarks: None
  ***************************************************************************/ 
 #define  PowerDownDetectConfig()					IPR2bits.LVDIP = 1;	\
													PIE2bits.LVDIE = 1;	\
													PIR2bits.LVDIF = 0
  
  
  
#endif	//__HardwareProfile_EnergyMeter

