/******************************************************************************

File Name:          SDADCConfig_EnergyMeter.c
Dependencies:    Microchip Peripheral Library
Processor:           PIC18F65J90
Company:             Microchip Technology, Inc.

Copyright 2010 Microchip Technology Inc.  All rights reserved.

Microchip licenses to you the right to use, modify, copy and distribute
Software only when embedded on a Microchip microcontroller or digital signal
controller that is integrated into your product or third party product
(pursuant to the sublicense terms in the accompanying license agreement).

You should refer to the license agreement accompanying this Software for
additional information regarding your rights and obligations.

SOFTWARE AND DOCUMENTATION ARE PROVIDED ?AS IS? WITHOUT WARRANTY OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE.
IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER
CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR
OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR
CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF
SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
(INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.

Author                                     Date                                                Revision History / Comments
------------------------------------------------------------------------------------------------------------------------
Harsha Jagadish              1 - Jan - 2011                                                    Initial Version v1.0
Craig King					15 - Feb - 2011													   Port to 65J90 v1.1
*******************************************************************************/
#include "HardwareProfile_EnergyMeter.h"
#include "RegDef_EnergyMeter.h"
#include "sdadc_EnergyMeter.h"
#include "eeprom_EnergyMeter.h"

#define NOMINAL_CURR_GAIN	16

static UINT8  MCP3901_data1;
static UINT8  MCP3901_data2;
static UINT8  MCP3901_Cmd;

void MCP3901_ByteWriteCmd(void);
void MCP3901_ByteRead(void);

/************************************************************************************
 Function    : UINT8 Init_MCP_SD_ADC(void)

 Description : The function initializes "Sigma-Delta ADC" for defined rate of sampling & Gain
	
 Arguments   : None
 
 Remarks     : This function is called from "EM_Initialization" function after SPI initialization 
		  for "Sigma Delta ADC" communication
**************************************************************************************/
UINT8 Init_MCP_SD_ADC(UINT8 Gain)
{
	DisableEEPROM();			//Deselect the EEPROM by setting the CS
	DisableSDADC();
	
	if(Gain==32)
		Gain = 0x05;
	else if (Gain == 16)
		Gain = 0x04;
	else if (Gain == 8)
		Gain = 0x03;
	else if (Gain == 4)
		Gain = 0x02;
	else if (Gain == 2)
		Gain = 0x01;
	else if (Gain == 1)
		Gain = 0x00;
	else
		Gain = 0x00;
	
	// Disabling all the interrupts
	DisableAllGlobalInterrupts();
	 
	Init_ADC_SPI();				// check whether it supports 11 mode or not
	
	EM_SDADC_MCLR = 0;			//Put the "Sigma Delta ADC" in reset
	Nop();
	Nop();
	Nop();
	Nop();
	Nop();
	Nop();
	EM_SDADC_MCLR = 1;			//Release the MCP from reset - this allows it to run.

	MCP_Config.config_val.mcp_configl = 0x01;
	MCP_Config.config_val.mcp_configh = MCP_OSR_REG;
	MCP_Config.config_val.mcp_gain = Gain;
	MCP_Config.config_val.mcp_status = 0xA0|0x03;

//*********************************************************
//CONFIGH : 
//	to default values with OSR=64(@ 10MHz) & 32 (@ 4MHz), 
//	No dither, 
//	Both channel in 16bit mode, 
//	No modulation enabled
//	Both channels in running mode with no reset hold & Shut down
//CONFIGL:
//	Intrenal reference voltage
//	External Clock
//**********************************************************
	MCP3901_data2 = MCP_Config.config_val.mcp_configl;
	MCP3901_data1 = MCP_Config.config_val.mcp_configh;
	
	EnableSDADC();
	MCP3901_Cmd = 0x14;			// A6-A5--->00;A4-A0---->0x0A;R/W---1
	MCP3901_ByteWriteCmd();		// send write cmd and address of config H & L;
	DisableSDADC();
	EnableSDADC();

	MCP3901_data2 = 0x00;
	MCP3901_data1 = 0x00;	
	MCP3901_Cmd = 0x15;
	MCP3901_ByteRead();			// Send Read cmd and read data from config 1 & 2;
	DisableSDADC();
	if( (MCP3901_data1!=MCP_Config.config_val.mcp_configh) ||
    	(MCP3901_data2!=MCP_Config.config_val.mcp_configl) )
		return(0xFF);

//*********************************************************
//GAIN : 
//	Channel 1 gain -- 16
//	Channel 1 gain -- 1
//STATUS AND COMMUNICATION:
//	Read in Type mode with 8 bytes at a time before CS cleared
//	No Latency Conversion, DR pulses after 3 DRCLK periods. (Default)
//**********************************************************		
	MCP3901_data1 = MCP_Config.config_val.mcp_gain;		//GAIN register
	MCP3901_data2 = MCP_Config.config_val.mcp_status;		//STATUS AND COMMUNICATION:
	EnableSDADC();
	MCP3901_Cmd = 0x10;

	MCP3901_ByteWriteCmd(); 	// Send write cmd and address of Gain;
	DisableSDADC();
	EnableSDADC();
	Nop();
	MCP3901_data2 = 0x00;
	MCP3901_data1 = 0x00;	
	MCP3901_Cmd = 0x11;
	MCP3901_ByteRead();			// Send Read cmd and read data from Gain
	DisableSDADC();
	if ((MCP3901_data1!=MCP_Config.config_val.mcp_gain)||
	    (MCP3901_data2!=(MCP_Config.config_val.mcp_status)))
		return(0xFF);	

	// Enabling all the interrupts
	EnableAllGlobalInterrupts();		
	EnableInterruptSDADC();
	EnableSDADC();				//Enable chip select for "Sigma Delta ADC"
	return(0);

}

/************************************************************************************
 Function    : void Init_ADC_SPI(void)
 
 Description : The function initializes SPI module in mode-01 for communication with "Sigma Delta ADC"
	
 Arguments   : None
 
 Remarks     : This function is called prior to"Sigma Delta ADC" initalization and "Sigma Delta ADC"9 data read process
		  Code uses Peripheral Library support of PIC18 available with MCC18
**************************************************************************************/
void Init_ADC_SPI(void)
{
	UINT8	ClearBuffer;
//*********** Configure SPI for EEPROM Communication *******************
//		* SPI Master mode, clock = FOSC/64
//		* MODE 01 with CKP=0 & CKE = 0 -- Idle state Clock low & Transmit occurs on transition from Idle to active clock state
//		* Smaple Data at middle of transmission
//********************************************************************
	OpenSPI(SPI_FOSC_4,MODE_11,SMPMID);
	ClearBuffer = SSPBUF;				//Clear SSPBUF to avoid buffer overflow condition on next write to buffer

}


/************************************************************************************
 Function    : void Check_SD_ADC_Config(void)

 Description : The function re-initializes SD-ADC if any corruption is reported while checking periodically in SD-ADC process
	
 Arguments   : None
 
 Remarks     : This function is called after "Sigma Delta ADC" initalization 
**************************************************************************************/
void Check_SD_ADC_Config(void)
{
	if(Mcp_Con_Check.bits.mcp_con_err==1)							//In case of any loss in SD MCP3901 config values then re-initialize
	{
		while(Init_MCP_SD_ADC(NOMINAL_CURR_GAIN));				// Call function to initialize the "Sigma Delta ADC"
		Mcp_Con_Check.bits.mcp_con_err = 0;
	}	
}


/************************************************************************************
 Function    : void MCP3901_ByteWriteCmd(void)

 Description : The function writes data into SD-ADC through SPI
	
 Arguments   : Command is passed through "MCP3901_Cmd" global variable
                        Data1 is passed thorugh "MCP3901_data1" global variable
		   Data2 is passed thorugh "MCP3901_data2" global variable
 
 Remarks     : This function is called from "Sigma Delta ADC" initalization 
**************************************************************************************/
void MCP3901_ByteWriteCmd(void)
{
	EnableSDADC();				//select the "Sigma Delta ADC"
	Nop();
	Nop();
	Nop();
	Nop();
	Nop();
	Nop();
	Output(MCP3901_Cmd);		

	Nop();
	Nop();
	Nop();

	Output(MCP3901_data1);

	Output(MCP3901_data2);

	DisableSDADC();				//deselect the "Sigma Delta ADC"
}


/************************************************************************************
 Function    : void MCP3901_ByteRead(void)

 Description : The function reads data from SD-ADC through SPI
	
 Arguments   : Command is passed through "MCP3901_Cmd" global variable
 
 Returns        : Data1 is passed thorugh "MCP3901_data1" global variable
		   Data2 is passed thorugh "MCP3901_data2" global variable
 
 Remarks     : This function is called from "Sigma Delta ADC" initalization 
**************************************************************************************/
void MCP3901_ByteRead(void)
{
UINT8 delay_cnt;

	EnableSDADC();				//select the "Sigma Delta ADC"
	
	Output(MCP3901_Cmd);
	for(delay_cnt=0;delay_cnt<20;delay_cnt++);

	MCP3901_data1 = ReadData();
	for(delay_cnt=0;delay_cnt<20;delay_cnt++);
	MCP3901_data2 = ReadData();
	
	DisableSDADC();				//deselect the "Sigma Delta ADC"
}	



