/******************************************************************************

File Name:           eeprom_EnergyMeter.c
Dependencies:    Microchip Peripheral Library
Processor:           PIC18F65J90
Company:             Microchip Technology, Inc.

Copyright 2010 Microchip Technology Inc.  All rights reserved.

Microchip licenses to you the right to use, modify, copy and distribute
Software only when embedded on a Microchip microcontroller or digital signal
controller that is integrated into your product or third party product
(pursuant to the sublicense terms in the accompanying license agreement).

You should refer to the license agreement accompanying this Software for
additional information regarding your rights and obligations.

SOFTWARE AND DOCUMENTATION ARE PROVIDED ?AS IS? WITHOUT WARRANTY OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE.
IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER
CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR
OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR
CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF
SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
(INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.

Author                                     Date                                                Revision History / Comments
------------------------------------------------------------------------------------------------------------------------
Harsha Jagadish              1 - Jan - 2011                                                    Initial Version v1.0
Craig King					15 - Feb - 2011													   Port to 65J90 v1.1
*******************************************************************************/
#include "eeprom_EnergyMeter.h"


UINT8  SaveEnergyStatus=0;
UINT32 Tmr_Seconds_since_EEP_Save=0;
BOOL   EEPROM_Load_Error = FALSE;

static UINT8  WriteStatus=0;

/************************************************************************************
 Function    : void Init_EEPROM_SPI(void)

 Description : The function initializes SPI module in mode-11 for communication with External EEPROM
	
 Arguments   : None
 
 Remarks     : This function is called prior to EEPROM data write/read process
		  Code uses Peripheral Library support of PIC18 available with MCC18
**************************************************************************************/
void Init_EEPROM_SPI(void)
{
UINT8 ClearSPIBuffer=0;
//*********** Configure SPI for EEPROM Communication *******************
//		* SPI Master mode, clock = FOSC/64
//		* MODE 11 with CKP=1 & CKE = 0 -- Idle state Clock High & Transmit occurs on transition from Idle to active clock state
//		* Smaple Data at end of transmission
//********************************************************************
		OpenSPI(SPI_FOSC_64,MODE_11,SMPEND);
		ClearSPIBuffer = ReadSPI();						//Clear SSPBUF for any residual data
}


/***********************************************************************************************************
 Function    :   void SaveEnergyOnPowerDown(void)

 Description : The function saves all energy accumulator registers to EEPROM on power down condition 
	
 Arguments   : None
 
 Remarks     : 1. Function must be executed in very limited time of power down during which microcontroller is running on
                          remaining charge stored. Hence it must be optimized for speed
		The function disables all the interrupts till the EEPROM data transfer. The state of Interrupts
		  will be restored back on return from function.
***********************************************************************************************************/
void SaveEnergyOnPowerDown(void)
{
	
	/*check if it is valid to save energy on following conditions:
	1. Energy load initialization was successful & complete configuration of meter was successul 
	2. It is not power down condition before energy load initialzation, in which case RAM may have garbage data
	3. Energy meter is in valid calibrated state
	If above conditions satisfy, then save energy data on power down condition
	*/
	if(SaveEnergyStatus == SAVE_ENERGY_TO_EEPROM_ENABLED)
	{
		//call EEPROM driver that stores energy into EEPROM 
    	StoreToEEPROM(ADDRESS_OF_BYTES_ENERGY_STORED_IN_EEPROM,
		              (UINT8*)&ENERGY_ACT,
		              NUMBER_OF_BYTES_ENERGY_STORED_IN_EEPROM);
    }
	
	//clear the power down detect (LVD) interrupt status flag
	ClearInterruptStatusPowerDownDetect();
	
	/* Carry out system reset to:
	1. avoid accumulation of energy in unstable power down state
	2. avoid unexpected behavioral output from SD-ADC which may send not valid data during this period
	3. Reset will carry put system through complete initialization, which will take care of any configuration corruption correction
	    which is a possibility in transition state of power down to power restore state, if the power down condition was a glitch & momentory
	*/
	SystemReset();
}

/***********************************************************************************************************
 Function    :   void SaveEnergyEvery24Hours(void)

 Description : The function saves all energy accumulator registers to EEPROM every 24 hours since last data saving
	
 Arguments   : None
 
 Remarks     : The timing calculation is based on TIMER0 Software Clock emulation with 1s interrupt. Using Timer as clock source
                        24 hours is counted as 24 x 60 x 60 seconds
		The function disables all the interrupts till the EEPROM data transfer. The state of Interrupts
		  will be restored back on return from function.
***********************************************************************************************************/
void SaveEnergyEvery24Hours(void)
{
	//Save energy once a day i.e, 86400 seconds
	if(Tmr_Seconds_since_EEP_Save >= SAVE_ENERGY_24_HOURS)
	{
		Tmr_Seconds_since_EEP_Save=0;

		//call EEPROM driver that stores energy into EEPROM 
    	StoreToEEPROM(ADDRESS_OF_BYTES_ENERGY_STORED_IN_EEPROM,
		              (UINT8*)&ENERGY_ACT,
		              NUMBER_OF_BYTES_ENERGY_STORED_IN_EEPROM);	
	}
}


/************************************************************************************
 Function    : void Store_EN_To_EEP_On_WDT_Reset(void)

 Description : The function stores the number of bytes passed as parameter to EEPROM. The RAM
		  to the location from where the data has to be read to EEPROM is passed as parameter.
		   Emulated EEPROM address to where the data has to be written is also passed as parameter.
		   Function checks for reset source and if it is WDT reset then save routine is called
		   
 Arguments   : None
 
 Remarks     : The function disables all the interrupts till the EEPROM data transfer. The state of Interrupts
		  will be restored back on return from function.
		  Function will be called first on system initialization. Hence necessary to initialize few ports
		  asociated with EEPROM as reset will have set them to default state
		  The  Code uses Peripheral Library support of PIC18 available with MCC18
 NOTE:	The makes use of the RAM pointer that will be initialized to point to desired location before
		this function is called.		  
**************************************************************************************/
void Store_EN_To_EEP_On_WDT_Reset(void)
{

	if( (RCONbits.RI==0) || (RCONbits.PD==0) || (RCONbits.POR==0) || (RCONbits.BOR==0) )
	{
		RCON |= 0x17;		//clear all above checked sources of reset logic
	}
	else if( ( (RCONbits.TO==0) || ((RCON&0x20)==0) ) && 
	         (SaveEnergyStatus == SAVE_ENERGY_TO_EEPROM_ENABLED) )
	{
		RCON |= 0x37;		//clear all above checked sources of reset logic

		//Set all analog multiplexed ports as Digital
		ConfigPortsForDigitalMode();
		
		//"Sigma Delta ADC" Chip select port in configured as output port & disabled
		SDADCChipSelectConfigure();
		
		//SD-ADC reset line is set to output & disabled
		SDADCResetPinConfigure();
		
		//EEPROM Chip select line is set to disable chip select of EEPROM
		EEPROMChipSelectConfigure();
	
		//call EEPROM driver that stores energy into EEPROM 
    	StoreToEEPROM(ADDRESS_OF_BYTES_ENERGY_STORED_IN_EEPROM,
		              (UINT8*)&ENERGY_ACT,
		              NUMBER_OF_BYTES_ENERGY_STORED_IN_EEPROM);

	}
	else
	{
		ClearWDT();
		RCON |= 0x37;		//clear all above checked sources of reset logic
	}	
	
	SaveEnergyStatus == SAVE_ENERGY_TO_EEPROM_DISABLED;
}


/************************************************************************************
 Function    : BOOL LoadFromEEPROM(UINT16 EEPROM_Adr,UINT16 RAM_Adr,UINT8 num_bytes)

 Description : The function load the number of bytes passed as parameter from EEPROM. The RAM
		  to the location to where the data has to be written from EEPROM is passed as parameter.
		   EEPROM address from where the data has to be read is also passed as parameter.
		   
 Arguments   : EEPROM_Adr - Address of the EEPROM location from which data has to be read
		   RAM_Adr - The location of the RAM to where the data has to be loaded
		   num_bytes - number of bytes of data to be loaded from EEPROM to RAM
 
 Returns        : Returns the status of load operation
                        TRUE - If load was not successful
		   FALSE - If load wa successful
  
 Remarks     : The function disables all the interrupts till the EEPROM data transfer. The state of Interrupts
		  will be restored back on return from function.
		  The  Code uses Peripheral Library support of PIC18 available with MCC18
 NOTE:	The makes use of the RAM pointer that will be initialized to point to desired location before
		this function is called.
**************************************************************************************/
BOOL LoadFromEEPROM(UINT16 EEPROM_Adr,UINT8* RAM_Adr,UINT8 num_bytes)
{
UINT8 int_state,CheckSum=0,ClearBuffer,WriteStatus=1,MulFactorOf64=0,CheckPage=0;
WORD_VAL EEP_Address;
BOOL ReturnStatus = FALSE;

	int_state = (INTCON&0xC0);				//Save the Global Interrupt enable Status 
	
	DisableSDADC();							//Disabling ADC Chip Select
	
	// Disabling all the interrupts
	DisableAllGlobalInterrupts();				
	
	Init_EEPROM_SPI();						//Call function that initializes the SPI i MODE-11 for EEPROM communication
	
	EEP_Address.Val = EEPROM_Adr;

	EnableEEPROM();						//Enabling the EEPROM Chip Select.

	//Checking the status of EEPROM.This would be required to make sure that EEPROM is free to execute the read opeartion	
	//This function sends the Command to EEPROM to read its status
	//Wait untill the EEPROM is busy
	while((ReadStatus()&0x01));

	// Check for The Identifier1	
	if(ByteRead(&EEP_Address)==EEP_ID1)
	{
		EEPROMAddrIncrement(&EEP_Address,&WriteStatus,&MulFactorOf64,&CheckPage);// Increment the EEPROM address to read check for the identifier2	
		if(ByteRead(&EEP_Address) == EEP_ID2)					// Check for The Identifier2 to check the validity of the data being read after this	
		{
			// Then Read the data from EEPROM till number of bytes become zero and copy the contents into RAM location.
			EEPROMAddrIncrement(&EEP_Address,&WriteStatus,&MulFactorOf64,&CheckPage);// Increment the EEPROM address to read further data till number of bytes specified are completed
			if(num_bytes)
			{
				while(num_bytes--)
				{
					*RAM_Adr = ByteRead(&EEP_Address);
					CheckSum = CheckSum + *RAM_Adr;
					EEPROMAddrIncrement(&EEP_Address,&WriteStatus,&MulFactorOf64,&CheckPage);// Increment the EEPROM address to read further data till number of bytes specified are completed
					RAM_Adr++;					//Increment Pointer that points
				}
				if(CheckSum != ByteRead(&EEP_Address))
				{
					ReturnStatus = TRUE;
				}
			}
		}
		else
		{
			ReturnStatus = TRUE;
		}
	}
	else
	{
		ReturnStatus = TRUE;
	}
	
	DisableEEPROM();				//Disable EEPROM by deselecting Chip select
	
	// After Completion of Reading, Initialize the SPI in mode-01 for communication with"Sigma Delta ADC"
	Init_ADC_SPI();
	EnableSDADC();							//Enabling ADC Chip Select
	INTCON |= int_state;					//Restore back the state of Global Interrupt Enable bits	
	return(ReturnStatus);
}


/************************************************************************************
 Function    : void StoreToEEPROM(UINT16 EEPROM_Adr,UINT16 RAM_Adr,UINT8 num_bytes)

 Description : The function stores the number of bytes passed as parameter to EEPROM. The RAM
		  to the location from where the data has to be read to EEPROM is passed as parameter.
		   EEPROM address to where the data has to be written is also passed as parameter.
		   
 Arguments   : EEPROM_Adr - Address of the EEPROM location to which data has to be written
		   RAM_Adr - The location of the RAM from where the data has to be read
		   num_bytes - number of bytes of data to be stored to EEPROM from RAM
 
 Remarks     : The function disables all the interrupts till the EEPROM data transfer. The state of Interrupts
		  will be restored back on return from function.
		  The  Code uses Peripheral Library support of PIC18 available with MCC18
 NOTE:	The makes use of the RAM pointer that will be initialized to point to desired location before
		this function is called.		  
**************************************************************************************/
void StoreToEEPROM(UINT16 EEPROM_Adr,UINT8* RAM_Adr,UINT8 num_bytes)
{
UINT8 int_state,CheckSum=0,ClearBuffer,WriteStatus=1,MulFactorOf64=0,CheckPage=0;
WORD_VAL EEP_Address;

	EEP_Address.Val = EEPROM_Adr;
	
	int_state = (INTCON&0xC0);				//Save the Global Interrupt enable Status 
	
	DisableSDADC();							//Disabling ADC Chip Select
	
	// Disabling all the interrupts
	DisableAllGlobalInterrupts();	
	
	Init_EEPROM_SPI();						//Call function that initializes the SPI i MODE-11 for EEPROM communication
	
	EnableEEPROM();
	
	ClearBuffer = SSPBUF;

	//Checking the status of EEPROM.This would be required to make sure that EEPROM is free to execute the read opeartion	
	//This function sends the Command to EEPROM to read its status
	//Wait untill the EEPROM is busy
	while((ReadStatus()&0x01));

	Nop();
	Nop();
	WriteEnable();							//Call function to WRTIE ENABLE the EEPROM so that data can be written into it

	//Call function that abides by protocol to check the status of EEPROM
	//Check if busy flag bit is set. If so, be in the loop till the EEPROM is relieved
	while(!(ReadStatus()&0x02));

	ByteWriteEnableandWrite(&EEP_Address,EEP_ID1);
	EEPROMAddrIncrement(&EEP_Address,&WriteStatus,&MulFactorOf64,&CheckPage);
	
	ByteWrite(EEP_ID2);
	EEPROMAddrIncrement(&EEP_Address,&WriteStatus,&MulFactorOf64,&CheckPage);

	if(num_bytes)
	{
		CheckPage = 0;
		//This routine writes the number bytes of data specified to EEPROM sequentially
		while(num_bytes)
		{
			CheckSum = CheckSum + *RAM_Adr; 
			if(CheckPage)
			{
				CheckPage = 0;
				ByteWriteEnableandWrite(&EEP_Address,*RAM_Adr);
			}
			else
			{
				ByteWrite(*RAM_Adr);
			}
			RAM_Adr++;
			EEPROMAddrIncrement(&EEP_Address,&WriteStatus,&MulFactorOf64,&CheckPage);
			num_bytes--;
		}
		ByteWrite(CheckSum);
	}

	DisableEEPROM();

	WriteStatus = 0;

	// After Completion of Reading, Initialize the SPI in mode-01 for communication with "Sigma Delta ADC"
	Init_ADC_SPI();
	EnableSDADC();							//Enabling ADC Chip Select
	INTCON |= int_state;					//Restore back the state of Global Interrupt Enable bits
	return; 
}


/************************************************************************************
 Function    : void EEPROMAddrIncrement(WORD_VAL* Addr, UINT8* StatusWrite, UINT8* Page, UINT8* PageStatus)

 Description : The function increments the value of EEPROM address. It also checks if the updated
		EEPROM address is in match with boundary address that is multiple of 64 bytes. 
		If so Then EEPROM buffer is flushed by write enabling
		   
 Arguments   : Addr - pointer to EEPROM address variable to be incremented
		  StatusWrite -  pointer to write status variable indicating if it is write or read eeprom oprations
		  Page - pointer to page of eeprom
		  PageStatus -  pointer to page of eeprom indicating the status if page completion is reached or not
 
 Returns	: None. But Updates the eeprom address & page status
 
 Remarks     : 1. This function is called from LoadFromEEPROM & StoreToEEPROM
		 2. The  Code uses Peripheral Library support of PIC18 available with MCC18  
**************************************************************************************/
void EEPROMAddrIncrement(WORD_VAL* Addr, UINT8* StatusWrite, UINT8* Page, UINT8* PageStatus)
{	
	UINT16 delay;
	if(Addr->byte.LB == 0xFF)						//Increment the EEPROM address value
	{
		Addr->byte.HB=Addr->byte.HB+1;
		Addr->byte.LB += 1;
	}
	else
	{
		(*Page) +=1;
		if(*Page == 0x40)					//check if the address matches any boundary line in multiple of 64 bytes
		{
			*Page = 0;
			Addr->byte.LB += 1;

			//Checking the status of EEPROM.This would be required to make sure that EEPROM is free to execute the read opeartion	
			//This function sends the Command to EEPROM to read its status
			//Wait untill the EEPROM is busy
			while((ReadStatus()&0x01));
	
			if(*StatusWrite)
			{
				DisableEEPROM();
				for(delay=0;delay<45000;delay++);
				WriteEnable();						//If so, Write Enable to flush the bufer in EEPROM
				*PageStatus = 1;
			}
		}
		else
		{
			Addr->byte.LB += 1;
		}
	}
	//return the incremented EEPROM address
}	



/**************************************************************************
Function       : void WriteEnable(void)
 
Description : Writes into EEPROM Control register to enable writing sequence
 
Arguments   : None
 
Remarks     : This Function Writes into EEPROM Control register to enable writing sequence
***************************************************************************/ 
void WriteEnable(void)
{
	DisableEEPROM();
	Nop();
	Nop();
	Nop();
	EnableEEPROM();

	Output(EEPROM_EN_WRITE);

	DisableEEPROM();
	Nop();
	Nop();
	return;
}


/**************************************************************************
Function       :UINT8 ReadStatus(void)
 
Description : Returns the status of EEPROM
 
Arguments   : None

Returnd       : Returns the Status of EEPROM status being requested
 
Remarks     : This Function checks and returns the status of EEPROM if it is busy in writing operation or not
***************************************************************************/ 
UINT8 ReadStatus(void)
{
UINT8 Data_Return;

	DisableEEPROM();
	Nop();
	Nop();
	Nop();
	EnableEEPROM();
	
	Output(EEPROM_READ_STATUS_REG);
	
	Data_Return = ReadData();
	
	DisableEEPROM();
	
	return(Data_Return);
}


/**************************************************************************
Function       : UINT8 ByteRead(WORD_VAL* EEPROM_Addr)
 
Description : Reads from EEPROM
 
Arguments   : EEPROM_Addr - pointer to eeprom address from where data need to be read

Returns       : Data read from EEPROM addressed location
 
Remarks     : Function should be called only after making sure that EEPROM is not busy
***************************************************************************/ 
UINT8 ByteRead(WORD_VAL* EEPROM_Addr)
{	
UINT8 Data_Return;

	EnableEEPROM();
	
	Output(EEPROM_READ);

	Output(EEPROM_Addr->byte.HB);

	Output(EEPROM_Addr->byte.LB);

	Data_Return = ReadData();

	DisableEEPROM();
	
	return(Data_Return);
}	


/**************************************************************************
Function       : void Output(UINT8 Data)
 
Description : Writes into EEPROM by outputting data to SPI
 
Arguments   : Data - Data to be written to SPI to wrtie to EEPROM
 
Remarks     : Function should be called only after making sure that EEPROM is not busy
***************************************************************************/ 
void Output(UINT8 Data)
{
unsigned char ClearBuffer;

	while(SSPSTATbits.BF ==1)
		ClearBuffer = SSPBUF;
	
	SSPBUF = Data;
	PIR1bits.SSPIF = 0;
	Nop();
	Nop();
	Nop();
	Nop();
	Nop();
	Nop();
	Nop();
	Nop();
	Nop();
	Nop();
	while(!PIR1bits.SSPIF);
	Nop();
	Nop();
	ClearBuffer = SSPBUF;
	
	return;
}	


/**************************************************************************
Function       : UINT8 ReadData(void)
 
Description : Reads from EEPROM by reading data from SPI
 
Arguments   : None

Returns       : Data from EEPROM read through SPI
 
Remarks     : Function should be called only after making sure that EEPROM is not busy
***************************************************************************/ 
UINT8 ReadData(void)
{
unsigned char ClearBuffer;

	while(SSPSTATbits.BF ==1)
		ClearBuffer = SSPBUF;
	
	SSPBUF = 0x00;
	PIR1bits.SSPIF = 0;
	Nop();
	Nop();
	Nop();
	Nop();
	Nop();
	Nop();
	Nop();
	Nop();
	Nop();
	while(!PIR1bits.SSPIF);
	Nop();
	Nop();
	return(SSPBUF);
}


/**************************************************************************
Function       : void ByteWriteEnableandWrite(WORD_VAL* EEPROM_Addr, UINT8 DataWrite)
 
Description : Writes into EEPROM after enabling the Data write
 
Arguments   : EEPROM_Addr - pointer to address of eeprom variable to which data need to be written
                      DataWrite - Data to be written  
 
Remarks     : Function should be called only after making sure that EEPROM is not busy
***************************************************************************/ 
void ByteWriteEnableandWrite(WORD_VAL* EEPROM_Addr, UINT8 DataWrite)
{
	EnableEEPROM();

	Output(EEPROM_WRITE);

	Output(EEPROM_Addr->byte.HB);

	Output(EEPROM_Addr->byte.LB);

	Output(DataWrite);

	return;
}

/**************************************************************************
Function       :void ByteWrite(UINT8 DataWrite)
 
Description : Writes into EEPROM
 
Arguments   : DataWrite - Data to be written  
 
Remarks     : Function should be called only after making sure that EEPROM is not busy
***************************************************************************/
void ByteWrite(UINT8 DataWrite)
{
	EnableEEPROM();	//  ; clear the chip select line

	Output(DataWrite);  //; store in RAM location outbyte
}





