/******************************************************************************

File Name:           Main_EnergyMeter.h
Dependencies:    Microchip Peripheral Library
Processor:           PIC18F65J90
Company:             Microchip Technology, Inc.

Copyright 2010 Microchip Technology Inc.  All rights reserved.

Microchip licenses to you the right to use, modify, copy and distribute
Software only when embedded on a Microchip microcontroller or digital signal
controller that is integrated into your product or third party product
(pursuant to the sublicense terms in the accompanying license agreement).

You should refer to the license agreement accompanying this Software for
additional information regarding your rights and obligations.

SOFTWARE AND DOCUMENTATION ARE PROVIDED ?AS IS? WITHOUT WARRANTY OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE.
IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER
CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR
OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR
CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF
SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
(INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.

Author                                     Date                                                Revision History / Comments
------------------------------------------------------------------------------------------------------------------------
Harsha Jagadish              1 - Jan - 2011                                                    Initial Version v1.0
Craig King					15 - Feb - 2011													   Port to 65J90 v1.1

*******************************************************************************/
#include "RegDef_EnergyMeter.h"
#include "HardwareProfile_EnergyMeter.h"
#include "MeteringEngine_EnergyMeter.h"
#include "Config_EnergyMeter.h"

//NOTE: It is very important to use linker file associated with project. This linker is configured to use zero initialization of all global variables which otherwise may affect the functionality
/************ DEVICE CONFIGURATION PARAMETERS ************************************************
	*	Extended Instruction mode is disabled
	*	Reset on Stack overflow is disabled
	*	Watch Dog Timer Enabled
	*	Watch Dog timer set for 2s time out
	*	Program memory is not Code Protected
	*	Internal to External switch over in socillator is disabled
	*	Fail Safe monitor is enabled
	*	Oscillator is configured in High Speed HS mode
	*	Hardware Debugger disabled
	*	CCP2 output is configured on defalut pin
***************************************************************************************************/
#pragma	config  XINST=OFF, STVREN=OFF, WDTEN=OFF, WDTPS = 512
#pragma	config  CP0=OFF
#pragma	config  IESO=OFF, FCMEN=ON, FOSC=HSPLL
#pragma	config CCP2MX=DEFAULT

//NOTE: Excercise caution before the addition/deletion of variables or relocating them as few are used in assembly which are bank specific
//NOTE: Donot add or delete any register from below upto comment line (------------------------) ending
//---------------------------------------------------------------------------------------------------------------------------------------------
//********************* GLOBAL ENERGY COMPUTATION Variable Definitions ***************************************************


/******************** GUI UPDATABLE REGISTERS *********************************************/
#pragma udata	CALIBRATION_BANK=0x200
MODE1_REG		MODE1;

STATUS1_REG		STATUS1;

CALB_CONTROL	CAL_CONTROL;

UINT16          LINE_CYC;
UINT16          LINE_CYC_CNT;

QWORD_VAL       RAW2_I_RMS;
UINT16	        RAW_I_RMS;
UINT16          I_RMS;

QWORD_VAL       RAW2_V_RMS;
UINT16	        RAW_V_RMS;
UINT16          V_RMS;

UINT16	        FREQUENCY;
UINT32          PERIOD;

QWORD_VAL       RAW_POWER_ACT;
UINT32          POWER_ACT;

UINT32          POWER_APP;

QWORD_VAL       RAW_POWER_REACT;
UINT32          POWER_REACT;

QWORD_VAL       RAW_ENERGY_ACT;

QWORD_VAL       RAW_ENERGY_APP;

QWORD_VAL       RAW_ENERGY_REACT;

UINT8           I_ABS_MAX;
UINT8           V_ABS_MAX;

//********************* CALIBRATION REGISTERS ************************************************************
INT8	        PHASE_COMPENSATION;
INT8            PHASE_COMPENSATION_90;

INT16	        OFFSET_I_RMS;
UINT16	        GAIN_I_RMS;

INT16           OFFSET_V_RMS;
UINT16          GAIN_V_RMS;

INT32	        OFFSET_POWER_ACT;
UINT16	        GAIN_POWER_ACT;
UINT16          GAIN_ENERGY_ACT;
UINT16          GAIN_NUMR_ENERGY_ACT;
UINT8           GAIN_DENR_ENERGY_ACT;

INT32	        OFFSET_POWER_REACT;
UINT16	        GAIN_POWER_REACT;
UINT16          GAIN_ENERGY_REACT;
UINT16          GAIN_NUMR_ENERGY_REACT;
UINT8           GAIN_DENR_ENERGY_REACT;

UINT16          GAIN_ENERGY_APP;

UINT8	        CREEP_THRESHOLD_MINUTE;
UINT8	        CREEP_THRESHOLD_SECOND;

UINT16          METER_CONSTANT;

UINT8	        CF_PULSE_WIDTH;

UINT16	        MODE1_DEF;
UINT16          CAL_STATUS;

UINT16	        MAXIMUM_CURRENT;
UINT16          CALIBRATION_VOLTAGE;
UINT16          CALIBRATION_CURRENT;
UINT16          CALIBRATION_FREQUENCY;
UINT16          CALIBRATION_LINE_CYCLE;

#pragma udata   ENG_REGS=0x300
UINT32       ENERGY_ACT;
UINT32       ENERGY_REACT;
UINT32       ENERGY_APP;

//*************************************************************************************************************



/************************************************************************************
 Function    : void main(void)

 Description : The function calls various functions for complete calculation of energy.
		   The flow of execution follows:
		   * Energymeter hardware initialization
		   * Calibration and Firmware register initialization
		   * initiate begining of Cycle function to calculate and accumulate energy for each cycle of input
		   * Update the GUI through USART
		   * At the end of cycle, update energy accumulation and update LCD
		   * Save energy periodically to EEPROM
	
 Arguments   : None
 
 Remarks     : None
**************************************************************************************/

/****************************************************************************************
***************************** CONVENTIONS IN NAMING *********************************
		* COMPUTATION CYCLE - Completion of User specified cycles before accumulation of energy
		* LINE CYCLE - Collection of every 128 samples across one input cycle of voltage & current
*****************************************************************************************/

void main(void)
{
	Store_EN_To_EEP_On_WDT_Reset();			//Store energy in case of WDT reset	
	//NOTE: Donot clear WDT prior to this statement which otherwise will clear WDT reset indication in RCONbits.TO
	ClearWDT();
	Init_Variables();					//Initialize all variables to their respective values
	ClearWDT();
	Init_Delay();						//Added delay to stabilize MCP3901 voltage	
	ClearWDT();
		
// This function will initialize all the variables to default values and initialize all peripherals
	Init_EnergyMeter();

//********************* Main Functionality ***************************************************
//*	Begin initialization fo all accumulation registers to zero in Every_COMPUTATION_CYC_Start()
//*	At Every sine cycle, get an intermediatery update
//*	After 128 sine cycles (Computation Cycle) calculate all parameter
//*	Update LCD after one computation cycle
//*	Repeat the above process
//*********************************************************************************************
	while(1)
	{
	// This function Every_COMPUTATION_CYC_Start() will initialize all the variables which are used in process of acquistion of "N" no of samples and in caliculation.
		Computation_Cycle_Start();
		do
		{	
			Signal_Cycle_Begin();				// This bit is used for 1 single sine cycle completion. 
			
			/* Clear WDT timer at interval of 4.96mS (128 / (43* 60) mS)
			     Call Macro that updates the GUI through USART in a loop till the "CycleComplete" flag gets set
			*/
			Process_GUI_Command_Loop();			

			//Check if the execution is in the process of calibration or normal energy calculation. If its the calibration, execute else section
			if( (!IsCalibrationModeEnabled()) || UpdateParametersForCalibration() )
		 	{
			 	
				/*
				Compute Signal Frequency & Period.
				Period is required in energy calculation.
				Frequency caluclation is required in Gain compensation across frequency for reactive component
				NOTE: This is a MUST Function
				*/
			 	ComputeSignalFrequency();
			 	
				/*
				Process the Instanteneous V * I accumulation
				This function forms the base for Active Power & Energy calculation
				NOTE: This is a MUST function for Active Power & Energy Calculation
				*/
			 	ProcessInstanteneousActivePower();
			 	
				/*
				Process the Instanteneous V90 * I accumulation
				This function forms the base for Reactive Power & Energy calculation
				NOTE: This is a MUST function for Reactive Power & Energy Calculation
				*/
			 	ProcessInstanteneousReactivePower();
			 	
				/*
				Function computes Active energy & updates the CF Pulse output
				NOTE: This function should be called after 
				              1. ComputeSignalFrequency
					    2. ProcessInstanteneousActivePower
				*/
				ComputeActiveEnergy();
				
				/*
				Function computes Reactive energy & updates the CF Pulse output
				NOTE: This function should be called after 
				              1. ComputeSignalFrequency
					    2. ProcessInstanteneousReactivePower
				*/
				ComputeReactiveEnergy();
				
				/*
				Function computes Active Power
				NOTE: This function should be called after 
				              1. ComputeSignalFrequency
					    2. ProcessInstanteneousActivePower
				*/
				ComputeActivePower();
				
				/*
				Function computes Reactive Power
				NOTE: This function should be called after 
				              1. ComputeSignalFrequency
					    2. ProcessInstanteneousReactivePower
				*/
				ComputeReactivePower();
				
				/*
				Function computes RMS Voltage
				*/
				ComputeRMSVoltage();
				
				/*
				Function computes RMS Current
				*/
				ComputeRMSCurrent();
				
				/*
				Function computes Apparent Power
				NOTE: This function should be called after 
				              1. ComputeSignalFrequency
					   2. ProcessInstanteneousActivePower
					   3. ProcessInstanteneousReactivePower
					   4. ComputeActivePower & ComputeReactivePower if traingular method is used
					   5. ComputeRMSVoltage &  ComputeRMSCurrent if V * I product method is used
				*/
				ComputeApparentPower();
				
				// Function enables checking of SD-ADC configuration for corruption
				Check_SD_ADC_Config();

				//Save energy every 24 hours since last data save
				SaveEnergyEvery24Hours();
			 	
				//Indicates completion of One signal cycle
				Signal_Cycle_Complete();

				LINE_CYC_CNT++;				//Increment the number of cycles of input signal completed till the user specified "N" is achieved
				
				if(IsComputationCycleCompleted())
			 	{
					break;								//If the "N" cycles are completed, then break from the loop
			 	}
		 	}
			else
			{
				break;
			}
		}while(LineCyclesRem--);					//Loop till the "N" cycles are completed before intiating new cycle
		
		Computation_Cycle_Complete();				//Clears Calibration update cycle

		LCD_Update();								//Update LCD
	}	
}







