/******************************************************************************

File Name:           Math_EnergyMeter.c
Dependencies:    Microchip Peripheral Library
Processor:           PIC18F65J90
Company:             Microchip Technology, Inc.

Copyright 2010 Microchip Technology Inc.  All rights reserved.

Microchip licenses to you the right to use, modify, copy and distribute
Software only when embedded on a Microchip microcontroller or digital signal
controller that is integrated into your product or third party product
(pursuant to the sublicense terms in the accompanying license agreement).

You should refer to the license agreement accompanying this Software for
additional information regarding your rights and obligations.

SOFTWARE AND DOCUMENTATION ARE PROVIDED ?AS IS? WITHOUT WARRANTY OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE.
IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER
CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR
OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR
CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF
SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
(INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.

Author                                     Date                                                Revision History / Comments
------------------------------------------------------------------------------------------------------------------------
Harsha Jagadish              1 - Jan - 2011                                                    Initial Version v1.0
Craig King					15 - Feb - 2011													   Port to 65J90 v1.1

*******************************************************************************/
#include "p18cxxx.h"
#include "GenericTypeDefs.h"
#include "RegDef_EnergyMeter.h"

UINT8   Dummy_Data2;

#pragma udata MATH_ASSEMBLY_DATA=0x1A9
UINT32   SquaredVal;
UINT32   FinalSum;
UINT8   temp1_var;
UINT8   temp2_var;
UINT8   temp3_var;

/************************************************************************************
 Function    : void ADD_64(QWORD_VAL* Data1, QWORD_VAL* Data2, QWORD_VAL* Res)

 Include     : Energymeter.h
 
 Description : The function carries out addition of two 64 bit values that are passed as parameter
		       and stores the result of addition in 64 bit varaible that is also passed as parameter
	
 Arguments   : Data1 - Pointer to 64-bit variable that need to be added
		       Data2 - Pointer to 64-bit variable that need to be added
		       Res - Pointer to 64 - bit variable in which result of addition is stored
 
 Remarks     : 1.  MCC18 innately handles addition of variables upto 32 - bit. To accomodate 64 - bit addition 
		           this custom addition function is defined.
		       2.  The result of addition in this particular application will never exceed 64 - bit. Hence 65ht bit
		           is not taken care. In other words, the carry bit for addition of two 64 bit values in this application
		           is always zero
		       3. 64-64 bit addition is acrried interms of two 32-32 bit addition
---------------------------------------------------------------------------------------------------------------------------------
           Res (64)  =                  Data1 (64)                          +                                Data2 (64)
---------------------------------------------------------------------------------------------------------------------------------
                                           Data1(upper 32)                                      Data1(Lower 32)		   
                                           Data2(upper 32)                                      Data2(Lower 32)
                                                                         (Carry of addition)
---------------------------------------------------------------------------------------------------------------------------------
          Res        =                Res ( Upper 32)                                          Res ( Lower 32)
---------------------------------------------------------------------------------------------------------------------------------
**************************************************************************************/
void ADD_64(QWORD_VAL* Data1, QWORD_VAL* Data2, QWORD_VAL* Res)
{
UINT8 temp=0;																//temporary varaiable defined to store the status of carry
	Res->dword.LD = Data1->dword.LD + Data2->dword.LD;					//Add lower 32 bits of input values	
	
	//check if the 32-32 bit addition resulted in carry
	if(STATUSbits.C)
		temp=1;
	else
		temp = 0;
	
	Res->dword.HD = Data1->dword.HD + Data2->dword.HD + (UINT32) temp;	//add upper 32 bits along with carry genereted from lower 32 bit addition
		
}



void SUB_64(QWORD_VAL* Data1, QWORD_VAL* Data2, QWORD_VAL* Res)
{
UINT8 temp=0;																//temporary varaiable defined to store the status of carry
	Res->dword.HD = Data1->dword.HD - Data2->dword.HD;					//Add lower 32 bits of input values	
	
	//check if the 32-32 bit addition resulted in carry
	if(STATUSbits.C)
		temp=1;
	else
		temp = 0;
	
	Res->dword.LD = Data1->dword.LD - Data2->dword.LD - (UINT32) temp;	//add upper 32 bits along with carry genereted from lower 32 bit addition
		
}
/************************************************************************************
 Function    : void MUL_S_32(INT32 arg1, INT32 arg2, INT16* ptr)

 Include     : Energymeter.h
 
 Description : The function implements signed multiplication of two 32 bit values that are passed as parameter
		       yeilding 64 bit result and stores the result of multiplication in 64 bit varaible 
		       that is also passed as parameter
	
 Arguments   : arg1 - Pointer to 32-bit variable that is one of the multiplicand
		       arg2 - Pointer to 32-bit variable that is one of the multiplicand
		       Res - Pointer to 64 - bit variable implemented in terms of 16-bit pointer in which result of multiplication is stored
 
 Remarks     : 1.  MCC18 innately handles multiplication of variables 16 - bit successfully. To accomodate 32*32 - bit multiplication 
		           this custom multiplication function is defined.
		       2.  The 32-32 multiplication is carried out interms of basic 16-16 multiplication
		       3.  Result Storing variable need to be cleared before the function is called as function adds multiplication result to existing value
//*************************** (ptr = arg1 * arg2)  ******************************************************
// ptr	=			[arg1+1] [arg1]             		*	                         [arg2+1] [arg2]	
//----------------------------------------------------------------------------------------------------------------------------------------------------
//										   [arg1+1] [arg2]		[arg1] [arg2]
//										------------ + ----------------			
//					    [arg1+1] [ arg2+1]	               [arg1] [arg2+1]
//	              Carry_of_Add      
//----------------------------------------------------------------------------------------------------------------------------------------------------
// ptr           =     ptr3                                ptr2                                        ptr1                                                  ptr0
//----------------------------------------------------------------------------------------------------------------------------------------------------
//		    - [arg1+1]		           - [arg1]						NOTE:*"If and only if arg1 is negetive "*
//		    - [arg2+1]		            - [arg2]					NOTE:*"If and only if arg2 is negetive"*
//----------------------------------------------------------------------------------------------------------------------------------------------------
// ptr           =     ptr3                                ptr2                                        ptr1                                                  ptr0
//----------------------------------------------------------------------------------------------------------------------------------------------------		
**************************************************************************************/
void MUL_S_32(INT32 arg1, INT32 arg2, INT16* ptr)
{
//32 bit variable with prototype defined in header file in defined for internal multilication operation in terms of 16-16 bit multiplication
DWORD_VAL res,temp1,temp2;
	
	temp1.Val = (UINT32)arg1;		//copy arguments of multiplication to temporary 32 bit variable so that they can be accessed interms of 16 bits individually with out need for masking
	temp2.Val = (UINT32)arg2;		//copy arguments of multiplication to temporary 32 bit variable so that they can be accessed interms of 16 bits individually with out need for masking
	
	//Multiply the lower 16 bits of both arguments and get 32 bit result
	res.Val = (UINT32)temp1.word.LW * temp2.word.LW;	
	*ptr++ = (UINT16)res.word.LW;	//Store the result of inetrmediate multiplication into final variable along with addition of intermedite values if any
	*ptr   = (UINT16)res.word.HW;	//Store the result of inetrmediate multiplication into final variable along with addition of intermedite values if any
	
	//Multiply the lower 16 bit of of one and upper 16 bit value of other
	res.Val = (UINT32)temp1.word.HW * temp2.word.LW;
	*ptr++ += (UINT16)res.word.LW;		//Store the result of inetrmediate multiplication into final variable along with addition of intermedite values if any
	*ptr++ += (UINT16)(res.word.HW+((UINT16)STATUSbits.C)); //Store the result of inetrmediate multiplication into final variable along with addition of intermedite values if any
	*ptr-- += ((UINT16)STATUSbits.C);	//Store the result of inetrmediate multiplication into final variable along with addition of intermedite values if any
	
	ptr--;				//decrement result pointer so that it points to previous value needed for addition of next mul results

	//Multiply the upper 16 bit of of one and lower 16 bit value of other
	res.Val = (UINT32)temp1.word.LW * temp2.word.HW;
	*ptr++ += (UINT16)res.word.LW;		//Store the result of inetrmediate multiplication into final variable along with addition of intermedite values if any
	*ptr++ += (UINT16)(res.word.HW+((UINT16)STATUSbits.C));	//Store the result of inetrmediate multiplication into final variable along with addition of intermedite values if any
	*ptr-- += ((UINT16)STATUSbits.C);	//Store the result of inetrmediate multiplication into final variable along with addition of intermedite values if any
	
	//Multiply the upper 16 bit of of one and upper 16 bit value of other
	res.Val = (UINT32)temp1.word.HW * temp2.word.HW;
	*ptr++ += (UINT16)res.word.LW;		//Store the result of inetrmediate multiplication into final variable along with addition of intermedite values if any
	*ptr-- += (UINT16)(res.word.HW+((UINT16)STATUSbits.C));	//Store the result of inetrmediate multiplication into final variable along with addition of intermedite values if any

	//load intermediate result varaiable with upper 32 bits of temporary result for sign correction
	res.word.LW = (UINT16)(*ptr++);
	res.word.HW = (UINT16)(*ptr--);
	
	//check if either of the argument is negetive. If so sign correct by subtracting the argument with final upper 32 bits of result
	if(temp1.bits.b31)
	{
		res.Val -= temp2.Val;
		*ptr++ = (UINT16)res.word.LW;
		*ptr-- = (UINT16)(res.word.HW+((UINT16)STATUSbits.C));
	}	
	if(temp2.bits.b31)
	{
		res.Val -= temp1.Val;
		*ptr++ = (UINT16)res.word.LW;
		*ptr   = (UINT16)(res.word.HW+((UINT16)STATUSbits.C));
	}
	
	return;
} 


/************************************************************************************
 Function    : void MUL_U_32(INT32 arg1, INT32 arg2, INT16* ptr)

 Include     : Energymeter.h
 
 Description : The function implements unsigned multiplication of two 32 bit values that are passed as parameter
		       yeilding 64 bit result and stores the result of multiplication in 64 bit varaible 
		       that is also passed as parameter
	
 Arguments   : arg1 - Pointer to 32-bit variable that is one of the multiplicand
		       arg2 - Pointer to 32-bit variable that is one of the multiplicand
		       Res - Pointer to 64 - bit variable implemented in terms of 16-bit pointer in which result of multiplication is stored
 
 Remarks     : 1.  MCC18 innately handles multiplication of variables 16 - bit successfully. To accomodate 32*32 - bit multiplication 
		           this custom multiplication function is defined.
		       2.  The 32-32 multiplication is carried out interms of basic 16-16 multiplication
		       3.  Result Storing variable need to be cleared before the function is called as function adds multiplication result to existing value
//*************************** (ptr = arg1 * arg2)  ******************************************************
// ptr	=			[arg1+1] [arg1]             		*	                         [arg2+1] [arg2]	
//----------------------------------------------------------------------------------------------------------------------------------------------------
//										   [arg1+1] [arg2]		[arg1] [arg2]
//										------------ + ----------------			
//					    [arg1+1] [ arg2+1]	               [arg1] [arg2+1]
//	              Carry_of_Add      
//----------------------------------------------------------------------------------------------------------------------------------------------------
// ptr           =     ptr3                                ptr2                                        ptr1                                                  ptr0
//----------------------------------------------------------------------------------------------------------------------------------------------------		
**************************************************************************************/
void MUL_U_32(UINT32 arg1, UINT32 arg2, UINT16* ptr)
{
//32 bit variable with prototype defined in header file in defined for internal multilication operation in terms of 16-16 bit multiplication
DWORD_VAL res,temp1,temp2;
	
	temp1.Val = arg1;			//copy arguments of multiplication to temporary 32 bit variable so that they can be accessed interms of 16 bits individually with out need for masking
	temp2.Val = arg2;			//copy arguments of multiplication to temporary 32 bit variable so that they can be accessed interms of 16 bits individually with out need for masking

	//Multiply the lower 16 bits of both arguments and get 32 bit result
	res.Val = (UINT32)temp1.word.LW * temp2.word.LW;
	*ptr++ = res.word.LW;		//Store the result of inetrmediate multiplication into final variable along with addition of intermedite values if any
	*ptr   = res.word.HW;		//Store the result of inetrmediate multiplication into final variable along with addition of intermedite values if any
	
	//Multiply the upper 16 bit of of one and lower 16 bit value of other
	res.Val = (UINT32)temp1.word.HW * temp2.word.LW;	
	*ptr++ += res.word.LW;		//Store the result of inetrmediate multiplication into final variable along with addition of intermedite values if any
	*ptr++ += res.word.HW+((UINT16)STATUSbits.C);	//Store the result of inetrmediate multiplication into final variable along with addition of intermedite values if any
	*ptr-- += ((UINT16)STATUSbits.C);		//Store the result of inetrmediate multiplication into final variable along with addition of intermedite values if any
	
	ptr--;				//decrement result pointer so that it points to previous value needed for addition of next mul results

	//Multiply the lower 16 bit of of one and upper 16 bit value of other
	res.Val = (UINT32)temp1.word.LW * temp2.word.HW;
	*ptr++ += res.word.LW;		//Store the result of inetrmediate multiplication into final variable along with addition of intermedite values if any
	*ptr++ += res.word.HW+((UINT16)STATUSbits.C);	//Store the result of inetrmediate multiplication into final variable along with addition of intermedite values if any
	*ptr-- += ((UINT16)STATUSbits.C);		//Store the result of inetrmediate multiplication into final variable along with addition of intermedite values if any

	//Multiply the upper 16 bit of of one and upper 16 bit value of other
	res.Val = (UINT32)temp1.word.HW * temp2.word.HW;	
	*ptr++ += res.word.LW;		//Store the result of inetrmediate multiplication into final variable along with addition of intermedite values if any
	*ptr   += res.word.HW+((UINT16)STATUSbits.C);	//Store the result of inetrmediate multiplication into final variable along with addition of intermedite values if any
	
	return;
} 	


/************************************************************************************
 Function    : void MUL_S_64(QWORD_VAL* arg1, UINT16 arg2, QWORD_VAL* ptr)

 Include     : Energymeter.h
 
 Description : The function implements signed multiplication of  64-16 bit values that are passed as parameter
		       yeilding 64 bit result and stores the result of multiplication in 64 bit varaible 
		       that is also passed as parameter
	
 Arguments   : arg1 - Pointer to 64-bit variable that is one of the multiplicand
		       arg2 - 16-bit variable that is one of the multiplicand
		       Res - Pointer to 64 - bit variable implemented in terms of 16-bit pointer in which result of multiplication is stored
 
 Remarks     : 1.  MCC18 innately handles multiplication of variables 16 - bit successfully. To accomodate 64*16 - bit multiplication 
		           this custom multiplication function is defined.
			   2.  The 64-16 multiplication is carried out interms of basic 16-16 multiplication
		       3.  In this application 64 - 16bit multiplication will always result in value that will never exceed 64 - bit. Hence no need to care
		           about the sign extended higher 16 - bit data which is redundent
		       4.  Result Storing variable need to be cleared before the function is called as function adds multiplication result to existing value
//*************************** (ptr = arg1 * arg2)  ******************************************************
// ptr	=			[arg1+1] [arg1+1] [arg1+1] [arg1]             		*	                        [arg2]	
//----------------------------------------------------------------------------------------------------------------------------------------------------
// ptr           =     ptr3                                ptr2                                        ptr1                                                  ptr0
//----------------------------------------------------------------------------------------------------------------------------------------------------		
**************************************************************************************/
void MUL_S_64(QWORD_VAL* arg1, UINT16 arg2, QWORD_VAL* ptr)
{
DWORD_VAL temp=0;

	ptr->dword.LD = (UINT32)arg1->word.LW * arg2;
	
	temp.Val = (UINT32)arg1->word.HW * arg2;
	
	ptr->word.HW += temp.word.LW;
	
	ptr->word.UW += temp.word.HW+(UINT16)STATUSbits.C;
	
	ptr->word.MW += STATUSbits.C;
	
	temp.Val = (UINT32)arg1->word.UW * arg2;
	
	ptr->word.UW += temp.word.LW;
	
	ptr->word.MW += temp.word.HW+STATUSbits.C;

	return;

} 


/************************************************************************************
 Function    :  UINT16 	Compute_RMS(void)

 Include     : Energymeter.h
 
 Description : The function implements square root of 32 - bit unsigned value, returning the posisitve root of square root
		       which will be 16 bit value. The function implements the NEWTON-RAPHSON's Method for computing the
		       square root.
	
 Arguments   : The function operates on variable "SquaredVal" that is global in definition. This variable is not passed as 
		       argument for function, however the function operates on it.
 
 Returns      :  Returns 16 bit positive square root value of 32 - bit positive value
 
 Remarks     : 1. MCC18 innately provides the square root function in math library. However for performance and application 
		          specific needs, square root is implemented in IN-LINE ASSEMBLY using NEWTON-RAPHSON's Method
		       2. The 64-16 multiplication is carried out interms of basic 16-16 multiplication
		       3. In this application 64 - 16bit multiplication will always result in value that will never exceed 64 - bit. Hence no need to care
		          about the sign extended higher 16 - bit data which is redundent
NOTE: Function is dependent on few variable allocation in BANK 1 of RAM. Hence below variables must be allocated 
            in BANK1 of RAM
	static UINT32   SquaredVal;
	static UINT32   FinalSum;
	static UINT8   temp1_var;
	static UINT8   temp2_var;
	static UINT8   temp3_var;
**************************************************************************************/
UINT16 Compute_RMS(void)
{

_asm
	movff	BSR,Dummy_Data2			//// Save BSR
	movlb	1						// BSR to scratchpad area
	movf	square_val+2,0,1		// Add 32-bit value of FSR0
	addwf	sq_offset,0,1			//  to sign extended 16-bit
	movwf	FinalSum,1				//  value of FSR1
	movf	square_val+3,0,1
	addwfc	sq_offset+1,0,1
	movwf	FinalSum + 1,1
	clrf	WREG,0
	btfsc	sq_offset+1,7,1
	comf	WREG,1,0
	addwfc	square_val+4,0,1
	movwf	FinalSum + 2,1
	clrf	WREG,0
	btfsc	sq_offset+1,7,1
	comf	WREG,1,0
	addwfc	square_val+5,0,1
	movwf	FinalSum + 3,1

	clrf	SqrRoot,1				// Clear square root value
	clrf	SqrRoot + 1,1

	btfsc	FinalSum + 3,7,1		// If FinalSum is negative, done
	bra	AOSR_Exit

// Compute bits 14-8 of the square root
	movlw	0x07					// Testing bits 14-8 (7 bits total)
	movwf	temp1_var,1
	movlw	0x40					// Set bit 14 initially
	movwf	temp2_var,1
	comf	temp2_var,0,1				// AND Mask for clearing bit
	movwf	temp3_var,1
AOSR_MS_Loop:
	movf	temp2_var,0,1				// Set bit
	iorwf	SqrRoot + 1,1,1
	movf	SqrRoot + 1,0,1			// Square top 8-bits of square root
	mulwf	SqrRoot + 1,1
	movf	PRODH,0,0				// Check top 8-bits of 16-bit result to
	subwf	FinalSum + 3,0,1		//  see if equal to final sum
	bnz	AOSR_MS_15_8_NotEqual		// If not equal, check top 8-bits
AOSR_MS_15_8_Equal:
	movf	PRODL,0,0				// Check bottom 8-bits of 16-bit result,
	cpfslt	FinalSum + 2,1			//  bit is OK if FinalSum + 3 >= PRODL
	bra	AOSR_MS_End
	bra	AOSR_MS_ClrBit
AOSR_MS_15_8_NotEqual:
	movf	PRODH,0,0				// Check top 8-bits of 16-bit result, bit
	cpfslt	FinalSum + 3,1			//  is OK if FinalSum + 4 >= PRODH
	bra	AOSR_MS_End
AOSR_MS_ClrBit:
	movf	temp3_var,0,1				// Result is too big, clear bit
	andwf	SqrRoot + 1,1,1
AOSR_MS_End:
	rrncf	temp2_var,1,1				// Rotate bit into next lower position
	rrncf	temp3_var,1,1				// Same for AND mask
	decfsz	temp1_var,1,1				// Done?
	bra	AOSR_MS_Loop

// Compute bits 7-0 of the square root
	movlw	0x08					// Testing bits 7-0 (8 bits total)
	movwf	temp1_var,1
	movlw	0x80					// Set bit 7 initially
	movwf	temp2_var,1
	comf	temp2_var,0,1				// AND Mask for clearing bit
	movwf	temp3_var,1
AOSR_LS_Loop:
	movf	temp2_var,0,1				// Set bit
	iorwf	SqrRoot,1,1
	call	AOSR_Square,1			// Square the square root
	call	AOSR_Compare,1			// Compare to the final sum
	btfsc	STATUS,0,0
	bra	AOSR_LS_End					// Keep the bit if carry is set
AOSR_LS_ClrBit:
	movf	temp3_var,0,1				// Result is too big, clear bit
	andwf	SqrRoot,1,1
AOSR_LS_End:
	rrncf	temp2_var,1,1				// Rotate bit into next lower position
	rrncf	temp3_var,1,1				// Same for AND mask
	decfsz	temp1_var,1,1				// Done?
	bra	AOSR_LS_Loop
	
AOSR_Exit:
	movff	Dummy_Data2,BSR
	bra		SqrComplete

// Square the square root value and place the result in SquaredVal[31:0]. Note that the multiplication
// requires that the square root value be positive.
AOSR_Square:
	movf	SqrRoot,0,1				// Arg1L * Arg1L -> PRODH:PRODL -> move to bits 15:0 of result
	mulwf	SqrRoot,1
	movf	PRODL,0,0
	movwf	SquaredVal,1
	movf	PRODH,0,0
	movwf	SquaredVal + 1,1
	movf	SqrRoot + 1,0,1			// Arg1H * Arg1H -> PRODH:PRODL -> move to bits 31:16 of result
	mulwf	SqrRoot + 1,1
	movf	PRODL,0,0
	movwf	SquaredVal + 2,1
	movf	PRODH,0,0
	movwf	SquaredVal + 3,1
	rlncf	SqrRoot + 1,0,1			// Cross products: Arg1L * (2*Arg1H) -> PRODH:PRODL ->
	mulwf	SqrRoot,1				//  add to bits 31:8 of result
	movf	PRODL,0,0				// (Note: the rotate to produce 2*Arg1H works because
	addwf	SquaredVal + 1,1,1		//  the initial value is between 0x00 and 0x7F, multiplying
	movf	PRODH,0,0				//  by two means the cross product only has to be added
	addwfc	SquaredVal + 2,1,1		//  once rather than twice.)
	clrf	WREG,0
	addwfc	SquaredVal + 3,1,1
	return 0

// Compare SquaredVal[31:0] to FinalSum[31:0]. If FinalSum is greater or equal to SquaredVal, the
// carry bit will be set. Otherwise, it will be clear.
AOSR_Compare:
	movf	SquaredVal + 3,0,1		// Check if top 8-bits are equal
	subwf	FinalSum + 3,0,1
	bz	AOSRC_Chk_23_16				// If equal, check next lower byte
	movf	SquaredVal + 3,0,1		// Compare FinalSum + 4 to SquaredVal + 4 
	cpfslt	FinalSum + 3,1
	bra	AOSRC_SetCarryRet			// FinalSum >= SquaredVal, set carry
	bra	AOSRC_ClrCarryRet			// FinalSum < SquaredVal, clear carry

AOSRC_Chk_23_16:
	movf	SquaredVal + 2,0,1		// Check if bits 23:16 are equal
	subwf	FinalSum + 2,0,1
	bz	AOSRC_Chk_15_8				// If equal, check next lower byte
	movf	SquaredVal + 2,0,1		// Compare FinalSum + 3 to SquaredVal + 3 
	cpfslt	FinalSum + 2,1
	bra	AOSRC_SetCarryRet			// FinalSum >= SquaredVal, set carry
	bra	AOSRC_ClrCarryRet			// FinalSum < SquaredVal, clear carry

AOSRC_Chk_15_8:
	movf	SquaredVal + 1,0,1		// Check if bits 15:8 are equal
	subwf	FinalSum + 1,0,1
	bz	AOSRC_Chk_7_0				// If equal, check next lower byte
	movf	SquaredVal + 1,0,1		// Compare FinalSum + 2 to SquaredVal + 2 
	cpfslt	FinalSum + 1,1
	bra	AOSRC_SetCarryRet			// FinalSum >= SquaredVal, set carry
	bra	AOSRC_ClrCarryRet			// FinalSum < SquaredVal, clear carry

AOSRC_Chk_7_0:
	movf	SquaredVal,0,1			// Check if bottom 8-bits are equal
	subwf	FinalSum,0,1
	bz	AOSRC_SetCarryRet			// If equal, set carry and return
	movf	SquaredVal,0,1			// Compare FinalSum to SquaredVal 
	cpfslt	FinalSum,1
	bra	AOSRC_SetCarryRet			// FinalSum >= SquaredVal, set carry

AOSRC_ClrCarryRet:
	bcf	STATUS,0,0					// Clear the carry and return
	movff	Dummy_Data2,BSR
	bra		SqrComplete

AOSRC_SetCarryRet:
	bsf	STATUS,0,0					// Set the carry and retur
	movff	Dummy_Data2,BSR
	bra		SqrComplete

SqrComplete:
_endasm;

	return(SqrRoot);				//Return 16 bit positive square root value
}


