/******************************************************************************

File Name:           MeteringEngine_EnergyMeter.c
Dependencies:    Microchip Peripheral Library
Processor:           PIC18F65J90
Company:             Microchip Technology, Inc.

Copyright 2010 Microchip Technology Inc.  All rights reserved.

Microchip licenses to you the right to use, modify, copy and distribute
Software only when embedded on a Microchip microcontroller or digital signal
controller that is integrated into your product or third party product
(pursuant to the sublicense terms in the accompanying license agreement).

You should refer to the license agreement accompanying this Software for
additional information regarding your rights and obligations.

SOFTWARE AND DOCUMENTATION ARE PROVIDED ?AS IS? WITHOUT WARRANTY OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE.
IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER
CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR
OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR
CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF
SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
(INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.

Author                                     Date                                                Revision History / Comments
------------------------------------------------------------------------------------------------------------------------
Harsha Jagadish              1 - Jan - 2011                                                    Initial Version v1.0
Craig King					15 - Feb - 2011													   Port to 65J90 v1.1

*******************************************************************************/
#include "MeteringEngine_EnergyMeter.h"


#if defined(ACTIVE_POWER_ENABLE_COMPUTATION) ||\
	defined(ACTIVE_ENERGY_ENABLE_COMPUTATION)

QWORD_VAL Average_Active_Power;


#endif


#if defined(REACTIVE_POWER_ENABLE_COMPUTATION) ||\
	defined(REACTIVE_ENERGY_ENABLE_COMPUTATION)

QWORD_VAL Average_Reactive_Power;

UINT16    Gain_Reactive_Energy_Frequency[22];
UINT16    Gain_Reactive_Power_Frequency[22];

BOOL		 GainChangeStatus = TRUE;

//This array contains phase compensation values for LPF output to be 90 degree phase shifted across frequencies
//The compensation values below correspond to 45Hz, 46Hz, 47Hz .. to 65Hz
UINT8     PHASE_COMPENSATION_90_FREQ[22] =  {-24,-23,-22,-21,-20,-19,-19,-18,-17,
                                                    -17,-16,-15,-15,-14,-14,-13,-13,-13,
													-12,-12,-11,-11} ;
													//360/65.1 is the angle per sample @ 60 degree;
UINT16    VAR_ENG_GAIN_FREQUENCY;
UINT16    VAR_POW_GAIN_FREQUENCY;

#endif

QWORD_VAL Accumulate_Raw_RMS_Voltage;
QWORD_VAL Accumulate_Raw_RMS_Current;

UINT8            LastLineCycCnt,LineCyclesRem;

REACT_REG Sign_Reg;

UINT32 Period_F;

UINT8    Save_BSR;

#pragma udata	MAIN_ASSEMBLY_ACCESS_SECTION=0x1BA
QWORD_VAL Active_Energy_Raw;

QWORD_VAL Reactive_Energy_Raw;

QWORD_VAL square_val;
UINT32     sq_offset;

UINT16	SqrRoot;
UINT8 temp1,temp2,temp3;
QWORD_VAL CF_Acc_Temp,CF_Acc_Temp_Reactive;
UINT8 TempyVar0;


#if defined(REACTIVE_POWER_ENABLE_COMPUTATION) ||\
	defined(REACTIVE_ENERGY_ENABLE_COMPUTATION)
UINT16 GetFrequencyFromTimerCount(UINT16 TimerCount) ;
#endif



/************************************************************************************
 Function    : void Computation_Cycle_Start(void)

 Description : The function initializes all temporary energy accumutation & associated variables to zero at the begining
		   of computation cycle so that energy calculated in this cycle can be accumulated in final result. It also initializes
		   the begining of cycle for CALIBRATION and clears all the associated calibration registers

 Arguments   : None

 Remarks     : The calling frequency of this function depends upon the USER defined LINE_CYC variable.
		  The function is called every ( 2 ^ (LINE_CYC) ) times of input signal cycle
**************************************************************************************/
void Computation_Cycle_Start(void)
{
UINT8 loop_cnt=0;
	//Check if cal mode has been turned off and the GUI is initiating a new calibration cycle. If  it is a new calibration cycle, then execution "if" section
    if( (Flags1.bits.CalMode==0) && (CAL_CONTROL.bits.CAL_MODE==1) )
    {
        Flags1.bits.CalMode=1;                	//Enable Calibration mode flag bit in mode register
    }

	if(CAL_CONTROL.bits.CAL_MODE==0)
		Flags1.bits.CalMode = 0;
		
	//check the value of LINE_CYC entered from GUI by user that specifies the number of input cycles to be comlpeted for one computation cycle.
	if(((UINT8)LINE_CYC)<=0x08)
		LastLineCycCnt = ((UINT8)LINE_CYC);		//if it is with in permissible limit of max 256 input cycles per computation cycle ( 2^8), load the same
	else
		LastLineCycCnt = 0x08;					//if exceeds the permissible range of max 256 input cycles per computation cycle ( 2^8), then load default max value
	
	LineCyclesRem = 0;                    	//Clear reg, default for 256 line cycles
	
	//if it is with in permissible limit of max 256 input cycles per computation cycle, then calculate "(LineCyclesRem= 2^LastLineCycCnt)" ;
	if((LastLineCycCnt<0x08))
	{
		loop_cnt=LastLineCycCnt;
		LineCyclesRem = 0x01;
		//calculating (2^LastLineCycCnt)
		do
		{
			LineCyclesRem <<= 1;				//implements (2 * LineCyclesRem)
		}while(--loop_cnt);
	}
	LINE_CYC_CNT = 0;                			// Clear user line cycle count

	if(Flags1.bits.CalMode)
	{//In CALIBRATION MODE
		
		Flags1.bits.PhA_Delay_SignBit = 0;		//clear phase delay flag bit
	}
	else
	{//In ENERGY COMPUTATION MODE
		if(PHASE_COMPENSATION&0x80)		//Check if phase delay is positive delay or negetive
			Flags1.bits.PhA_Delay_SignBit = 1;	//If negetive, enable sign bit for sign correction in multiplication
		else
			Flags1.bits.PhA_Delay_SignBit = 0;	//if positive, sign bit is cleared as no sign corrcetion is required in multiplication
	}
	PhaseComp_Amplitude = PHASE_COMPENSATION * 2;				//Phase delay value between V & I for phase compensation
	PhaseComp_Amplitude_90Lag = PHASE_COMPENSATION_90 * 2;			//Phase delay value between V 90 shifted & I for phase compensation

	Flags1.bits.NewLCGroupStart = 1;    		// Set flag to indicate the begining of new COMPUTATION CYCLE
	Period_F = 0;            					// Clear final period register

    CF_Pulse_Width_W = CF_PULSE_WIDTH;          // Move CF pulse width over to working register
	CF_Pulse_Width_W_Rx = CF_PULSE_WIDTH;		// Move CF pulse width over to working register
	
	Accumulate_Raw_RMS_Current.dword.LD = 0;           			// Clear active energy accumulator
	Accumulate_Raw_RMS_Current.dword.HD = 0;            		// Clear active energy accumulator
	Accumulate_Raw_RMS_Voltage.dword.LD = 0;            		// Clear active energy accumulator
	Accumulate_Raw_RMS_Voltage.dword.HD = 0;            		// Clear active energy accumulator
}


#if defined(ACTIVE_POWER_ENABLE_COMPUTATION) ||\
	defined(ACTIVE_ENERGY_ENABLE_COMPUTATION)
/************************************************************************************
 Function    : void ProcessInstanteneousActivePower(void)

 Description : The function processed the active V * I product accumulation over last signal cycle
		   Function accumulates V * I summation after absolute value computation

 Arguments   : None
 
 Return         : None

 Remarks     : Function is necessary for calculation of
		1. Active Energy
		2. Active Power
**************************************************************************************/
void ProcessInstanteneousActivePower(void)
{

// Checking for  negative active power, If it is, it converts into positive.
	if(Summation_Active_Ins_Power.bits.b39)
	{
		Summation_Active_Ins_Power.dword.HD = 
		0x000000FF- Summation_Active_Ins_Power.dword.HD;
		
		if(STATUSbits.C)
				Summation_Active_Ins_Power.dword.LD = 
				0xFFFFFFFF - Summation_Active_Ins_Power.dword.LD -1;
		else
				Summation_Active_Ins_Power.dword.LD = 
				0xFFFFFFFF - Summation_Active_Ins_Power.dword.LD;
		
		Summation_Active_Ins_Power.dword.LD = Summation_Active_Ins_Power.dword.LD+1;
		
		if(STATUSbits.C)
				Summation_Active_Ins_Power.dword.HD = Summation_Active_Ins_Power.dword.HD+1;
	}
	
	//check if it the begining of a new COMPUTATION CYCLE
	if(Flags1.bits.NewLCGroupStart)
	{
		Average_Active_Power.dword.LD = Summation_Active_Ins_Power.dword.LD;	//Move as it is begining of computation cycle, the power over one input cycle into accumulator of power over Computation Cycle.
		Average_Active_Power.dword.HD = Summation_Active_Ins_Power.dword.HD;	//Move as it is begining of computation cycle, the power over one input cycle into accumulator of power over Computation Cycle.
	}
	else	//if it is not the begining of COMPUTATION CYCLE
	{
		ADD_64(&Average_Active_Power,&Summation_Active_Ins_Power,&Average_Active_Power);	//Accumulate the active power registered over one input cycle into power accumulator over COMPUTATION CYCLE
	}
}
#endif


#if defined(REACTIVE_POWER_ENABLE_COMPUTATION) ||\
	defined(REACTIVE_ENERGY_ENABLE_COMPUTATION)
/************************************************************************************
 Function    : void ProcessInstanteneousReactivePower(void)

 Description : The function processed the reactive V90 * I product accumulation over last signal cycle
		   Function accumulates V90 * I summation after absolute value computation

 Arguments   : None
 
 Return         : None

 Remarks     : Function is necessary for calculation of
		1. Reactive Energy
		2. Reactive Power
**************************************************************************************/
void ProcessInstanteneousReactivePower(void)
{

// Checking for  negative reactive power, If it is, it converts into positive.
	if(Summation_Reactive_Ins_Power.bits.b39)
	{
		Summation_Reactive_Ins_Power.dword.HD = 
		0x000000FF- Summation_Reactive_Ins_Power.dword.HD;
		
		if(STATUSbits.C)
				Summation_Reactive_Ins_Power.dword.LD = 
				0xFFFFFFFF - Summation_Reactive_Ins_Power.dword.LD -1;
		else
				Summation_Reactive_Ins_Power.dword.LD = 
				0xFFFFFFFF - Summation_Reactive_Ins_Power.dword.LD;
		
		Summation_Reactive_Ins_Power.dword.LD = Summation_Reactive_Ins_Power.dword.LD+1;
		
		if(STATUSbits.C)
				Summation_Reactive_Ins_Power.dword.HD = Summation_Reactive_Ins_Power.dword.HD+1;		
		
		Sign_Reg.bits.Curr_Sign = 1;		
	}
	else
	{
		Sign_Reg.bits.Curr_Sign = 0;
	}
	
	//check if it the begining of a new COMPUTATION CYCLE
	if(Flags1.bits.NewLCGroupStart)
	{
		Average_Reactive_Power.dword.LD = Summation_Reactive_Ins_Power.dword.LD;
		Average_Reactive_Power.dword.HD = Summation_Reactive_Ins_Power.dword.HD;
	}
	else	//if it is not the begining of COMPUTATION CYCLE
	{
		if(Sign_Reg.bits.Curr_Sign==Sign_Reg.bits.Power_Sign)
		{
		ADD_64(&Average_Reactive_Power,&Summation_Reactive_Ins_Power,&Average_Reactive_Power);				//Accumulate the reactive power registered over one input cycle into power accumulator over COMPUTATION CYCLE
		}
		else
		{
			if( (Average_Reactive_Power.dword.HD > Summation_Reactive_Ins_Power.dword.HD) || ((Average_Reactive_Power.dword.HD == Summation_Reactive_Ins_Power.dword.HD) && ((UINT32)Average_Reactive_Power.dword.LD > (UINT32)Summation_Reactive_Ins_Power.dword.LD) ) )
			{
				SUB_64(&Average_Reactive_Power,&Summation_Reactive_Ins_Power,&Average_Reactive_Power);
			}
			else
			{
				SUB_64(&Summation_Reactive_Ins_Power,&Average_Reactive_Power,&Average_Reactive_Power);
				Sign_Reg.bits.Power_Sign=Sign_Reg.bits.Curr_Sign;
			}
		}
	}
}
#endif


#if defined(ACTIVE_POWER_ENABLE_COMPUTATION) ||\
	defined(ACTIVE_ENERGY_ENABLE_COMPUTATION) ||\
	defined(REACTIVE_POWER_ENABLE_COMPUTATION) ||\
	defined(REACTIVE_ENERGY_ENABLE_COMPUTATION)
/************************************************************************************
 Function    : void ComputeSignalFrequency(void)

 Description : The function computes signal period and frequency. Period & frequency are measured 
		using TIMER1 & TIMER3.
		Period is used in enegry calculation while frequency is used in gain compensation for
		reactive component calculation across input signal frequency (due to gain/attenuation
		variation  of LPF for 90 degree lag for reactive component calculation)

 Arguments   : None

 Remarks     : Function is necessary for calculation of
		1. Active Enegry
		2. Reactive Energy
		3. Reactive Power
**************************************************************************************/
void ComputeSignalFrequency(void)
{
	/*Line cycle period is actually 0.8 us per tick at this point. Divide by two to make it 1.6 us per tick.
	    As a side benefit this rotate limits the maximum period to 0x7FFF keeping it always a positive number
	    This above operation is equal to Period_W << 7 to get period over 128 samples and finally devide by 2 to  
	    make clock LSB value to 1.6uS */
	Period_W >>= 1;						
	FREQUENCY = GetFrequencyFromTimerCount(Signal_Frequency);
	
	Period_F =  Period_W + Period_F;	//update period that will be used by GUI for calibration parameter calculation

	if(IsComputationCycleCompleted())
	{
		PERIOD = Period_F;								//Update the period of entire input siganl cycle
	}
}
#endif



#if defined(REACTIVE_POWER_ENABLE_COMPUTATION) ||\
	defined(REACTIVE_ENERGY_ENABLE_COMPUTATION)
/************************************************************************************
 Function    : UINT16 GetFrequencyFromTimerCount(UINT16 TimerCount)

 Description : The function computes signalfrequency. Frequency are measured using TIMER1.
		TIMER1 counts are compared against precalculated boundary values.
		Timer count between successive zero crossongs and this is used in frequency calculation
		Frequency calculation is done in discrete values of 1Hz difference
		Frequency is used in gain compensation for reactive component calculation across 
		input signal frequency (due to gain/attenuation variation  of LPF for 90 degree lag 
		for reactive component calculation)

 Arguments   : TimerCount - TIMER1 count between two successive zero crossing of Voltage
 
 Return         : Frequency - Frequency value based on Timer count

 Remarks     : Function is necessary for calculation of
		1. Reactive Energy
		2. Reactive Power
**************************************************************************************/
UINT16 GetFrequencyFromTimerCount(UINT16 TimerCount) 
{
	
	if(TimerCount >= GetTimerCountForFrequency(44.5))
		return(4400);
	else if( (TimerCount < GetTimerCountForFrequency(44.5)) &&
           (TimerCount >= GetTimerCountForFrequency(45.5)))
		return(4500);
	else if( (TimerCount < GetTimerCountForFrequency(45.5)) &&
           (TimerCount >= GetTimerCountForFrequency(46.5)))
		return(4600);
	else if( (TimerCount < GetTimerCountForFrequency(46.5)) &&
           (TimerCount >= GetTimerCountForFrequency(47.5)))
		return(4700);
	else if( (TimerCount < GetTimerCountForFrequency(47.5)) &&
           (TimerCount >= GetTimerCountForFrequency(48.5)))
		return(4800);
	else if( (TimerCount < GetTimerCountForFrequency(48.5)) &&
           (TimerCount >= GetTimerCountForFrequency(49.5)))
		return(4900);
	else if( (TimerCount < GetTimerCountForFrequency(49.5)) &&
           (TimerCount >= GetTimerCountForFrequency(50.5)))
		return(5000);
	else if( (TimerCount < GetTimerCountForFrequency(50.5)) &&
           (TimerCount >= GetTimerCountForFrequency(51.5)))
		return(5100);
	else if( (TimerCount < GetTimerCountForFrequency(51.5)) &&
           (TimerCount >= GetTimerCountForFrequency(52.5)))
		return(5200);
	else if( (TimerCount < GetTimerCountForFrequency(52.5)) &&
           (TimerCount >= GetTimerCountForFrequency(53.5)))
		return(5300);
	else if( (TimerCount < GetTimerCountForFrequency(53.5)) &&
           (TimerCount >= GetTimerCountForFrequency(54.5)))
		return(5400);
	else if( (TimerCount < GetTimerCountForFrequency(54.5)) &&
           (TimerCount >= GetTimerCountForFrequency(55.5)))
		return(5500);																
	else if( (TimerCount < GetTimerCountForFrequency(55.5)) &&
           (TimerCount >= GetTimerCountForFrequency(56.5)))
		return(5600);
	else if( (TimerCount < GetTimerCountForFrequency(56.5)) &&
           (TimerCount >= GetTimerCountForFrequency(57.5)))
		return(5700);
	else if( (TimerCount < GetTimerCountForFrequency(57.5)) &&
           (TimerCount >= GetTimerCountForFrequency(58.5)))
		return(5800);
	else if( (TimerCount < GetTimerCountForFrequency(58.5)) &&
           (TimerCount >= GetTimerCountForFrequency(59.5)))
		return(5900);
	else if( (TimerCount < GetTimerCountForFrequency(59.5)) &&
           (TimerCount >= GetTimerCountForFrequency(60.5)))
		return(6000);
	else if( (TimerCount < GetTimerCountForFrequency(60.5)) &&
           (TimerCount >= GetTimerCountForFrequency(61.5)))
		return(6100);
	else if( (TimerCount < GetTimerCountForFrequency(61.5)) &&
           (TimerCount >= GetTimerCountForFrequency(62.5)))
		return(6200);
	else if( (TimerCount < GetTimerCountForFrequency(62.5)) &&
           (TimerCount >= GetTimerCountForFrequency(63.5)))
		return(6300);														
	else if( (TimerCount < GetTimerCountForFrequency(63.5)) &&
           (TimerCount >= GetTimerCountForFrequency(64.5)))
		return(6400);
	else if( (TimerCount < GetTimerCountForFrequency(64.5)) &&
           (TimerCount >= GetTimerCountForFrequency(65.5)))
		return(6500);
	else if( (TimerCount < GetTimerCountForFrequency(65.5)))
		return(6600);				
	else
	{//Return default 60Hz
		return(6000);
	}	
}
#endif


#if defined(REACTIVE_POWER_ENABLE_COMPUTATION) ||\
	defined(REACTIVE_ENERGY_ENABLE_COMPUTATION)
/************************************************************************************
 Function    : void ReactiveGainAcrossFrequency(void)

 Description : The function Calculates the compensation required for Reactive Power & Energy (reactive gain) across frequency
	
 Arguments   : None
 
 Remarks     :  * This function is called at every signal cycle interval
 		* The function will execute compensation calculation only in case of Gain change
		Function is necessary for computation of:
		1. Reactive Energy
		2. Reactive Power
**************************************************************************************/
void ReactiveGainAcrossFrequency(void)
{
static 	UINT16 Gain_Reactive_Power_Previous=0,Gain_Reactive_Energy_Previous=0;

	if(Gain_Reactive_Energy_Previous!=GAIN_NUMR_ENERGY_REACT)
	{
		Gain_Reactive_Energy_Previous = GAIN_NUMR_ENERGY_REACT;
		GainChangeStatus = TRUE;

		//45Hz
		Gain_Reactive_Energy_Frequency[0] = GAIN_NUMR_ENERGY_REACT * 0.7499;
		//46Hz
		Gain_Reactive_Energy_Frequency[1] = GAIN_NUMR_ENERGY_REACT * 0.7593;
		//47Hz
		Gain_Reactive_Energy_Frequency[2] = GAIN_NUMR_ENERGY_REACT * 0.7851;
		//48Hz
		Gain_Reactive_Energy_Frequency[3] = GAIN_NUMR_ENERGY_REACT * 0.8074;
		//49Hz
		Gain_Reactive_Energy_Frequency[4] = GAIN_NUMR_ENERGY_REACT * 0.8212;
		//50Hz
		Gain_Reactive_Energy_Frequency[5] = GAIN_NUMR_ENERGY_REACT * 0.8374;
		//51Hz
		Gain_Reactive_Energy_Frequency[6] = GAIN_NUMR_ENERGY_REACT * 0.8536;
		//52Hz
		Gain_Reactive_Energy_Frequency[7] = GAIN_NUMR_ENERGY_REACT * 0.8716;
		//53Hz
		Gain_Reactive_Energy_Frequency[8] = GAIN_NUMR_ENERGY_REACT * 0.8891;
		//54Hz
		Gain_Reactive_Energy_Frequency[9] = GAIN_NUMR_ENERGY_REACT * 0.9083;

		
		//55Hz
		Gain_Reactive_Energy_Frequency[10] = GAIN_NUMR_ENERGY_REACT * 0.9269;
		//56Hz
		Gain_Reactive_Energy_Frequency[11] = GAIN_NUMR_ENERGY_REACT * 0.9382;
		//57Hz
		Gain_Reactive_Energy_Frequency[12] = GAIN_NUMR_ENERGY_REACT * 0.9477;
		//58Hz
		Gain_Reactive_Energy_Frequency[13] = GAIN_NUMR_ENERGY_REACT * 0.9688;
		//59Hz
		Gain_Reactive_Energy_Frequency[14] = GAIN_NUMR_ENERGY_REACT * 0.9911;
		//60Hz - Calibration frequency
		Gain_Reactive_Energy_Frequency[15] = GAIN_NUMR_ENERGY_REACT;			//	GAIN_NUMR_ENERGY_REACT * 1
		//61Hz
		Gain_Reactive_Energy_Frequency[16] = GAIN_NUMR_ENERGY_REACT * 1.0155;
		//62Hz
		Gain_Reactive_Energy_Frequency[17] = GAIN_NUMR_ENERGY_REACT * 1.0399;
		//63Hz
		Gain_Reactive_Energy_Frequency[18] = GAIN_NUMR_ENERGY_REACT * 1.0495;
		//64Hz
		Gain_Reactive_Energy_Frequency[19] = GAIN_NUMR_ENERGY_REACT * 1.0663;
		//65Hz
		Gain_Reactive_Energy_Frequency[20] = GAIN_NUMR_ENERGY_REACT * 1.0931;
		//66Hz
		Gain_Reactive_Energy_Frequency[21] = GAIN_NUMR_ENERGY_REACT * 1.1185;	
	}
	
	if(Gain_Reactive_Power_Previous!=GAIN_POWER_REACT)
	{
		Gain_Reactive_Power_Previous = GAIN_POWER_REACT;
		GainChangeStatus = TRUE;

		//45Hz
		Gain_Reactive_Power_Frequency[0] = GAIN_POWER_REACT * 0.7499;
		//46Hz
		Gain_Reactive_Power_Frequency[1] = GAIN_POWER_REACT * 0.7593;
		//47Hz
		Gain_Reactive_Power_Frequency[2] = GAIN_POWER_REACT * 0.7851;
		//48Hz
		Gain_Reactive_Power_Frequency[3] = GAIN_POWER_REACT * 0.8074;
		//49Hz
		Gain_Reactive_Power_Frequency[4] = GAIN_POWER_REACT * 0.8212;
		//50Hz - Calibration frequency
		Gain_Reactive_Power_Frequency[5] = GAIN_POWER_REACT * 0.8374;
		//51Hz
		Gain_Reactive_Power_Frequency[6] = GAIN_POWER_REACT * 0.8536;
		//52Hz
		Gain_Reactive_Power_Frequency[7] = GAIN_POWER_REACT * 0.8716;
		//53Hz
		Gain_Reactive_Power_Frequency[8] = GAIN_POWER_REACT * 0.8891;
		//54Hz
		Gain_Reactive_Power_Frequency[9] = GAIN_POWER_REACT * 0.9083;
		
		//55Hz
		Gain_Reactive_Power_Frequency[10] = GAIN_POWER_REACT * 0.9269;
		//56Hz
		Gain_Reactive_Power_Frequency[11] = GAIN_POWER_REACT * 0.9382;
		//57Hz
		Gain_Reactive_Power_Frequency[12] = GAIN_POWER_REACT * 0.9477;
		//58Hz
		Gain_Reactive_Power_Frequency[13] = GAIN_POWER_REACT * 0.9688;
		//59Hz
		Gain_Reactive_Power_Frequency[14] = GAIN_POWER_REACT * 0.9911;
		//60Hz - Calibration frequency
		Gain_Reactive_Power_Frequency[15] = GAIN_POWER_REACT;			//	GAIN_POWER_REACT * 1
		//61Hz
		Gain_Reactive_Power_Frequency[16] = GAIN_POWER_REACT * 1.0155;
		//62Hz
		Gain_Reactive_Power_Frequency[17] = GAIN_POWER_REACT * 1.0399;
		//63Hz
		Gain_Reactive_Power_Frequency[18] = GAIN_POWER_REACT * 1.0495;
		//64Hz
		Gain_Reactive_Power_Frequency[19] = GAIN_POWER_REACT * 1.0663;
		//65Hz
		Gain_Reactive_Power_Frequency[20] = GAIN_POWER_REACT * 1.0931;
		//66Hz
		Gain_Reactive_Power_Frequency[21] = GAIN_POWER_REACT * 1.1185;

	}	
}

#endif


#if defined(REACTIVE_POWER_ENABLE_COMPUTATION) ||\
	defined(REACTIVE_ENERGY_ENABLE_COMPUTATION)
/************************************************************************************
 Function    : void GetReactiveGainForLineFrequency(unsigned int freq)

 Description : The function configures the reactive gain for line frequency
	
 Arguments   : freq - frequency calculated based on timer1 count from GetFrequencyFromTimerCount function
 
 Remarks     :  Function is necessary for calculation of
		1. Reactive Energy
		2. Reactive Power
**************************************************************************************/
void GetReactiveGainForLineFrequency(unsigned int freq)
{
static UINT8 Previous_Frequency=0;
UINT8 freq_w;
	
   for (freq_w = 0 ; freq >= 1000 ; freq>=1000?(freq -= 1000):freq, freq_w += 0x10 );		//Frequency is converted to BCD format for appropriate PWM configuration and gain adjustment using a switch statement
   for ( ; freq >= 100 ; freq>=100?(freq -= 100):freq, freq_w++ );

	
	if((Previous_Frequency!=freq_w) || (GainChangeStatus == TRUE))	//This check condition is used to avoid execution of the function completely on every cycle
	{																//This will be executed only in case of frequency change or gain change
		Previous_Frequency = freq_w;
		GainChangeStatus = FALSE;
		switch(freq_w)
		{
		case 0x45:		//For frequency 45Hz
		PHASE_COMPENSATION_90 = PHASE_COMPENSATION_90_FREQ[0];
		VAR_ENG_GAIN_FREQUENCY = Gain_Reactive_Energy_Frequency[0];
		VAR_POW_GAIN_FREQUENCY = Gain_Reactive_Power_Frequency[0];		
			break;
		case 0x46:		//For frequency 46Hz
		PHASE_COMPENSATION_90 = PHASE_COMPENSATION_90_FREQ[1];	
		VAR_ENG_GAIN_FREQUENCY = Gain_Reactive_Energy_Frequency[1];		
		VAR_POW_GAIN_FREQUENCY = Gain_Reactive_Power_Frequency[1];
				break;
		case 0x47:		//For frequency 47Hz
		PHASE_COMPENSATION_90 = PHASE_COMPENSATION_90_FREQ[2];
		VAR_ENG_GAIN_FREQUENCY = Gain_Reactive_Energy_Frequency[2];		
		VAR_POW_GAIN_FREQUENCY = Gain_Reactive_Power_Frequency[2];
				break;
		case 0x48:		//For frequency 48Hz
		PHASE_COMPENSATION_90 = PHASE_COMPENSATION_90_FREQ[3];
		VAR_ENG_GAIN_FREQUENCY = Gain_Reactive_Energy_Frequency[3];	
		VAR_POW_GAIN_FREQUENCY = Gain_Reactive_Power_Frequency[3];
				break;
		case 0x49:		//For frequency 49Hz
		PHASE_COMPENSATION_90 = PHASE_COMPENSATION_90_FREQ[4];
		VAR_ENG_GAIN_FREQUENCY = Gain_Reactive_Energy_Frequency[4];	
		VAR_POW_GAIN_FREQUENCY = Gain_Reactive_Power_Frequency[4];
				break;
		case 0x50:		//For frequency 50Hz
		PHASE_COMPENSATION_90 = 	PHASE_COMPENSATION_90_FREQ[5];
		VAR_ENG_GAIN_FREQUENCY = Gain_Reactive_Energy_Frequency[5];	
		VAR_POW_GAIN_FREQUENCY = Gain_Reactive_Power_Frequency[5];
				break;
		case 0x51:		//For frequency 51Hz
		PHASE_COMPENSATION_90 = PHASE_COMPENSATION_90_FREQ[6];
		VAR_ENG_GAIN_FREQUENCY = Gain_Reactive_Energy_Frequency[6];	
		VAR_POW_GAIN_FREQUENCY = Gain_Reactive_Power_Frequency[6];
				break;
		case 0x52:		//For frequency 52Hz
		PHASE_COMPENSATION_90 = PHASE_COMPENSATION_90_FREQ[7];
		VAR_ENG_GAIN_FREQUENCY = Gain_Reactive_Energy_Frequency[7];	
		VAR_POW_GAIN_FREQUENCY = Gain_Reactive_Power_Frequency[7];
				break;
		case 0x53:		//For frequency 53Hz
		PHASE_COMPENSATION_90 = PHASE_COMPENSATION_90_FREQ[8];
		VAR_ENG_GAIN_FREQUENCY = Gain_Reactive_Energy_Frequency[8];	
		VAR_POW_GAIN_FREQUENCY = Gain_Reactive_Power_Frequency[8];
				break;
		case 0x54:		//For frequency 54Hz
		PHASE_COMPENSATION_90 = PHASE_COMPENSATION_90_FREQ[9];
		VAR_ENG_GAIN_FREQUENCY = Gain_Reactive_Energy_Frequency[9];	
		VAR_POW_GAIN_FREQUENCY = Gain_Reactive_Power_Frequency[9];
				break;			
		case 0x55:		//For frequency 55Hz
		PHASE_COMPENSATION_90 = PHASE_COMPENSATION_90_FREQ[10];
		VAR_ENG_GAIN_FREQUENCY = Gain_Reactive_Energy_Frequency[10];	
		VAR_POW_GAIN_FREQUENCY = Gain_Reactive_Power_Frequency[10];
				break;			
		case 0x56:		//For frequency 56Hz
		PHASE_COMPENSATION_90 = PHASE_COMPENSATION_90_FREQ[11];
		VAR_ENG_GAIN_FREQUENCY = Gain_Reactive_Energy_Frequency[11];	
		VAR_POW_GAIN_FREQUENCY = Gain_Reactive_Power_Frequency[11];
				break;			
		case 0x57:		//For frequency 57Hz
		PHASE_COMPENSATION_90 = PHASE_COMPENSATION_90_FREQ[12];
		VAR_ENG_GAIN_FREQUENCY = Gain_Reactive_Energy_Frequency[12];	
		VAR_POW_GAIN_FREQUENCY = Gain_Reactive_Power_Frequency[12];
				break;			
		case 0x58:		//For frequency 58Hz
		PHASE_COMPENSATION_90 = PHASE_COMPENSATION_90_FREQ[13];
		VAR_ENG_GAIN_FREQUENCY = Gain_Reactive_Energy_Frequency[13];	
		VAR_POW_GAIN_FREQUENCY = Gain_Reactive_Power_Frequency[13];
				break;			
		case 0x59:		//For frequency 59Hz
		PHASE_COMPENSATION_90 = PHASE_COMPENSATION_90_FREQ[14];
		VAR_ENG_GAIN_FREQUENCY = Gain_Reactive_Energy_Frequency[14];	
		VAR_POW_GAIN_FREQUENCY = Gain_Reactive_Power_Frequency[14];
				break;			
		case 0x60:		//For frequency 60Hz
		PHASE_COMPENSATION_90 = PHASE_COMPENSATION_90_FREQ[15];
		VAR_ENG_GAIN_FREQUENCY = Gain_Reactive_Energy_Frequency[15];	
		VAR_POW_GAIN_FREQUENCY = Gain_Reactive_Power_Frequency[15];
				break;	
		case 0x61:		//For frequency 61Hz
		PHASE_COMPENSATION_90 = PHASE_COMPENSATION_90_FREQ[16];
		VAR_ENG_GAIN_FREQUENCY = Gain_Reactive_Energy_Frequency[16];	
		VAR_POW_GAIN_FREQUENCY = Gain_Reactive_Power_Frequency[16];
				break;			
		case 0x62:		//For frequency 62Hz
		PHASE_COMPENSATION_90 = PHASE_COMPENSATION_90_FREQ[17];
		VAR_ENG_GAIN_FREQUENCY = Gain_Reactive_Energy_Frequency[17];		
		VAR_POW_GAIN_FREQUENCY = Gain_Reactive_Power_Frequency[17];
				break;			
		case 0x63:		//For frequency 63Hz
		PHASE_COMPENSATION_90 = PHASE_COMPENSATION_90_FREQ[18];
		VAR_ENG_GAIN_FREQUENCY = Gain_Reactive_Energy_Frequency[18];		
		VAR_POW_GAIN_FREQUENCY = Gain_Reactive_Power_Frequency[18];
				break;			
		case 0x64:		//For frequency 64Hz
		PHASE_COMPENSATION_90 = PHASE_COMPENSATION_90_FREQ[19];
		VAR_ENG_GAIN_FREQUENCY = Gain_Reactive_Energy_Frequency[19];		
		VAR_POW_GAIN_FREQUENCY = Gain_Reactive_Power_Frequency[19];
				break;			
		case 0x65:		//For frequency 65Hz
		PHASE_COMPENSATION_90 = PHASE_COMPENSATION_90_FREQ[20];
		VAR_ENG_GAIN_FREQUENCY = Gain_Reactive_Energy_Frequency[20];	
		VAR_POW_GAIN_FREQUENCY = Gain_Reactive_Power_Frequency[20];
				break;			
		case 0x66:		//For frequency 65Hz
		PHASE_COMPENSATION_90 = PHASE_COMPENSATION_90_FREQ[21];
		VAR_ENG_GAIN_FREQUENCY = Gain_Reactive_Energy_Frequency[21];
		VAR_POW_GAIN_FREQUENCY = Gain_Reactive_Power_Frequency[21];
				break;
		default:								//This condition checks for extreme cases and if nothing is matched, then base operation frequency is used
			if(freq < 0x45)
			{
				PHASE_COMPENSATION_90 = PHASE_COMPENSATION_90_FREQ[0];
				VAR_ENG_GAIN_FREQUENCY = Gain_Reactive_Energy_Frequency[0];
				VAR_POW_GAIN_FREQUENCY = Gain_Reactive_Power_Frequency[0];	
			}
			else if (freq > 0x66)
			{
				PHASE_COMPENSATION_90 = PHASE_COMPENSATION_90_FREQ[21];
				VAR_ENG_GAIN_FREQUENCY = Gain_Reactive_Energy_Frequency[21];	
				VAR_POW_GAIN_FREQUENCY = Gain_Reactive_Power_Frequency[21];
			}
			else
			{//Set default frequency 60Hz
				PHASE_COMPENSATION_90 = PHASE_COMPENSATION_90_FREQ[15];
				VAR_ENG_GAIN_FREQUENCY = Gain_Reactive_Energy_Frequency[15];	
				VAR_POW_GAIN_FREQUENCY = Gain_Reactive_Power_Frequency[15];
			}
			break; 
		}
	}

}
#endif

