/******************************************************************************

File Name:           MeteringEngine_EnergyMeter.h
Dependencies:    Microchip Peripheral Library
Processor:           PIC18F65J90
Company:             Microchip Technology, Inc.

Copyright 2010 Microchip Technology Inc.  All rights reserved.

Microchip licenses to you the right to use, modify, copy and distribute
Software only when embedded on a Microchip microcontroller or digital signal
controller that is integrated into your product or third party product
(pursuant to the sublicense terms in the accompanying license agreement).

You should refer to the license agreement accompanying this Software for
additional information regarding your rights and obligations.

SOFTWARE AND DOCUMENTATION ARE PROVIDED ?AS IS? WITHOUT WARRANTY OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE.
IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER
CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR
OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR
CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF
SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
(INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.

Author                                     Date                                                Revision History / Comments
------------------------------------------------------------------------------------------------------------------------
Harsha Jagadish              1 - Jan - 2011                                                    Initial Version v1.0
*******************************************************************************/
#ifndef __MeteringEngine_EnergyMeter
#define __MeteringEngine_EnergyMeter

#include "GenericTypeDefs.h"
#include "HardwareProfile_EnergyMeter.h"
#include "Config_EnergyMeter.h"
#include "RegDef_EnergyMeter.h"


/****************************************************************************
  Macro:   GetTimerCountForFrequency(Frequency_Value)

 Description:   Macro gets the timer count for frequency with timer prescale & system clock usage

  Precondition:   Timer prescale value must be set properly

  Parameters:   Frequency_Value - Value of frequency for which timer count need to be calculated

  Remarks: None
  ***************************************************************************/
 #define  GetTimerCountForFrequency(Frequency_Value)  (UINT16)((GetPeripheralClock() / FREQUENCY_TIMER_PRESCALE) / Frequency_Value)
 
/****************************************************************************
  Macro:   TruncateLower2Bytes(variable)

 Description:   Macro neglects lower two bytes of data

  Precondition:   Variable must be QWORD

  Parameters:   variable- variable whose lower 2 bytes need to be neglected

  Remarks: Sign externsion will not be done
  ***************************************************************************/
  #define  TruncateLower2Bytes(variable)		variable.word.LW = variable.word.HW;	\
												variable.word.HW = variable.word.UW;	\
												variable.word.UW = variable.word.MW;	\
												variable.word.MW = 0;
												
/**************************************************************************
Macro       : Right_Shift_64(Data)
 
Description : Right shifts the data by one bit
 
Arguments   : Data- Data to be right shifted
 
Remarks     : sign extension will not be done
                      Oprerates on QWORD variables only
***************************************************************************/ 
#define Right_Shift_64(Data)				{	Data.dword.LD >>= 1;			\
												Data.bits.b31 = Data.bits.b32;	\
												Data.dword.HD >>= 1;	}
					

/**************************************************************************
Macro       : Left_Shift_64(Data)
 
Description : Left shifts the data by one bit
 
Arguments   : Data- Data to be left shifted
 
Remarks     : Oprerates on QWORD variables only
***************************************************************************/ 
#define Left_Shift_64(Data)			{	Data.dword.HD <<= 1;			\
										Data.bits.b32 = Data.bits.b31;	\
										Data.dword.LD <<= 1 ;}

										
/**************************************************************************
Macro       : IsComputationCycleCompleted()
 
Description : 128 signal cycles completion completes Computation cycle
 
Arguments   : None
 
Remarks     : chekcs LineCyclesRem value equal to 0 
***************************************************************************/
 #define  IsComputationCycleCompleted()			(LineCyclesRem==0)
 

/**************************************************************************
Macro       : Computation_Cycle_Complete()
 
Description : Indicates the completion of Computation Cycle and terminates the update cycle of multi point calibration
 
Arguments   : None
 
Remarks     : None
***************************************************************************/ 
 #define  Computation_Cycle_Complete()		CAL_CONTROL.bits.CAL_UPDATE = 0
 
 
/**************************************************************************
Macro       : Signal_Cycle_Complete()
 
Description : Indicates the completion of Signal Cycle after 128 samples
 
Arguments   : None
 
Remarks     : None
***************************************************************************/ 
 #define  Signal_Cycle_Complete()		Flags1.bits.NewLCGroupStart = 0
 
 
/**************************************************************************
Macro       : Signal_Cycle_Begin()
 
Description : Indicates the begining of Signal Cycle for 128 samples
 
Arguments   : None
 
Remarks     : None
***************************************************************************/ 
 #define  Signal_Cycle_Begin()		Flags1.bits.CycleComplete = 0

 
/**************************************************************************
Macro       : IsCalibrationModeEnabled()
 
Description : returns the status of Cal Mode bit
 
Arguments   : None
 
Remarks     : None
***************************************************************************/ 
 #define  IsCalibrationModeEnabled()		Flags1.bits.CalMode
 
/**************************************************************************
Macro       : UpdateParametersForCalibration()
 
Description : Execute main routine for once to calculate parameters that are inturn used to calculate
                       Calibration registers
 
Arguments   : None
 
Remarks     : None
***************************************************************************/ 
 #define  UpdateParametersForCalibration()		CAL_CONTROL.bits.CAL_UPDATE

 
/**************************************************************************************************************
Macro       : Process_GUI_Command_Loop()
 
Description : Calls "ProcessGUICommand" function that updates the GUI through USART in a loop till the complete of user specified 
		 number of cycles
 
Arguments   : None
 
Remarks     : The  MACROS loops waiting for "CycleComplete" flag to get set, thus updating GUI in the mean while
***************************************************************************************************************/ 
#define  Process_GUI_Command_Loop()	Flags1.bits.CycleComplete = 0;			\
											do									\
										{										\
											ProcessGUICommand();				\
											OSCCONbits.IDLEN = 1;				\
											_asm sleep _endasm;					\
											Nop();								\
										} while(!Flags1.bits.CycleComplete);	\
										ClearWDT()
										
//#define  Process_GUI_Command_Loop()		Flags1.bits.CycleComplete = 0;			\
//										do										\
//										{										\
//											ProcessGUICommand();				\
//										} while(!Flags1.bits.CycleComplete);	\
//										ClearWDT()
																				
										
#endif	//__MeteringEngine_EnergyMeter

