/******************************************************************************

File Name:           ReactiveEnergy_EnergyMeter.c
Dependencies:    Microchip Peripheral Library
Processor:           PIC18F65J90
Company:             Microchip Technology, Inc.

Copyright 2010 Microchip Technology Inc.  All rights reserved.

Microchip licenses to you the right to use, modify, copy and distribute
Software only when embedded on a Microchip microcontroller or digital signal
controller that is integrated into your product or third party product
(pursuant to the sublicense terms in the accompanying license agreement).

You should refer to the license agreement accompanying this Software for
additional information regarding your rights and obligations.

SOFTWARE AND DOCUMENTATION ARE PROVIDED ?AS IS? WITHOUT WARRANTY OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE.
IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER
CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR
OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR
CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF
SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
(INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.

Author                                     Date                                                Revision History / Comments
------------------------------------------------------------------------------------------------------------------------
Harsha Jagadish              1 - Jan - 2011                                                    Initial Version v1.0
Craig King					15 - Feb - 2011													   Port to 65J90 v1.1

*******************************************************************************/
#include "MeteringEngine_EnergyMeter.h"


#if defined(REACTIVE_ENERGY_ENABLE_COMPUTATION)

QWORD_VAL Accumulate_Reactive_Energy_Raw;

QWORD_VAL Reactive_Energy_Raw_Over_One_Cycle;

static UINT16 Gain_Energy_React_Numerator;
static INT8 Gain_Energy_React_Denominator;

void GainCompensateReactiveEnergy(void);

/************************************************************************************
 Function    : void ComputeReactiveEnergy(void)

 Description : The function computes Reactive energy & updates the CF  Pulse output

 Arguments   : None
 
 Return         : Updates Reactive energy variable

 Remarks     : Function need to be used along with "ProcessInstanteneousReactivePower" function only
**************************************************************************************/
void ComputeReactiveEnergy(void)
{
static QWORD_VAL Accumulate_Active_Energy_Raw;

	// Check if Enabled to be accumulated into the total energy registers or CF pulse output
	//Multiply power in one cycle with period to get the energy over one cycle
	MUL_S_64(&Summation_Reactive_Ins_Power,Period_W,&Reactive_Energy_Raw_Over_One_Cycle);
	
	//Neglect the lower 2 bytes of data as it is insignificant in overall energy accumulation, with out any energy accuracy loss
	TruncateLower2Bytes(Reactive_Energy_Raw_Over_One_Cycle);
	Right_Shift_64(Reactive_Energy_Raw_Over_One_Cycle);			//Right shifting to neglect lower 14 bits of data

    Reactive_Energy_Raw.dword.LD = 0;					//Clear active power sum for this line cycle
	Reactive_Energy_Raw.dword.HD = 0;					//Clear active power sum for this line cycle

	if(Sign_Reg.bits.Curr_Sign==Sign_Reg.bits.Eng_Sign)
	{
	ADD_64(&Reactive_Energy_Raw,&Reactive_Energy_Raw_Over_One_Cycle,&Reactive_Energy_Raw);			//Accumulate the phase corrected, gain multiplied reactive energy to temp energy accumulation varaiable for sign corrections in case of negetive energy record
	}
	else
	{
		if( (Reactive_Energy_Raw.dword.HD > Reactive_Energy_Raw_Over_One_Cycle.dword.HD) || ((Reactive_Energy_Raw.dword.HD == Reactive_Energy_Raw_Over_One_Cycle.dword.HD) && ((UINT32)Reactive_Energy_Raw.dword.LD > (UINT32)Reactive_Energy_Raw_Over_One_Cycle.dword.LD) ) )
		{
			SUB_64(&Reactive_Energy_Raw,&Reactive_Energy_Raw_Over_One_Cycle,&Reactive_Energy_Raw);
		}
		else
		{
			SUB_64(&Reactive_Energy_Raw_Over_One_Cycle,&Reactive_Energy_Raw,&Reactive_Energy_Raw);
			Sign_Reg.bits.Eng_Sign=Sign_Reg.bits.Curr_Sign;
		}
	}

	//check if it the begining of a new COMPUTATION CYCLE
	if(Flags1.bits.NewLCGroupStart)
	{
		Accumulate_Reactive_Energy_Raw.dword.LD = Reactive_Energy_Raw.dword.LD;
		Accumulate_Reactive_Energy_Raw.dword.HD = Reactive_Energy_Raw.dword.HD;
	}
	else
	{
		ADD_64(&Accumulate_Reactive_Energy_Raw,&Reactive_Energy_Raw,&Accumulate_Reactive_Energy_Raw);
	}

	ReactiveGainAcrossFrequency();					//Calculate Gain Compensation for Reactive energy on Gain change
	GetReactiveGainForLineFrequency(FREQUENCY);		//Compensate Gain & Phase for Reactive Energy	

	Gain_Energy_React_Numerator = VAR_ENG_GAIN_FREQUENCY;		//Gain dependency on frequency
	Gain_Energy_React_Denominator = GAIN_DENR_ENERGY_REACT;		//Gain dependency on frequency
	GainCompensateReactiveEnergy();			//Call CF Pulse accumulator increment function for active energy
	
	if(IsComputationCycleCompleted())
	{
		RAW_ENERGY_REACT.dword.LD = Accumulate_Reactive_Energy_Raw.dword.LD;
		RAW_ENERGY_REACT.dword.HD = Accumulate_Reactive_Energy_Raw.dword.HD;
	}
}


/************************************************************************************
 Function    : void GainCompensateReactiveEnergy(void)

 Description : The function computes Reactive energy by multiplying Gain to raw value and updates the
		CF Pulse output

 Arguments   : None
 
 Return         : Updates Reactive energy CF Pulse variable

 Remarks     : Function called from "ComputeReactiveEnergy" function and "ProcessInstanteneousReactivePower"
		function need to be called in main routine
NOTE:  Variables used in this function need to be allocated in BANK1 of RAM only
**************************************************************************************/
void GainCompensateReactiveEnergy(void)
{
UINT8 loop_count=0;

_asm
	movff	BSR,Save_BSR			// Save BSR
	movlb	1						// BSR to scratchpad area
	movff	FSR0L,SqrRoot
	movff	FSR0H,SqrRoot+1
	movff	FSR1L,temp2
	movff	FSR1H,temp3

	btfsc	Reactive_Energy_Raw+4,7,1		// If active power is negative, clear CF inc
	bra	ClearCFInc_Rx

	clrf	Reactive_Energy_Raw,1			// Only interested in top 32-bits of 40-bits
	clrf	Reactive_Energy_Raw+5,1			// Make sure this value is clear

	movff	Gain_Energy_React_Denominator,TempyVar0			// Put Gain_Energy_React_Denominator into temp1
	btfsc	TempyVar0,7,1
	bra	CCFI_CFDEN_Neg_Rx

// Gain_Energy_React_Denominator is positive, shift left (multiply by 2) per value in Gain_Energy_React_Denominator
	movlw	0x1F					// Limit to 31 shifts or less
	andwf	TempyVar0,1,1
	lfsr	0,Reactive_Energy_Raw+1			// Point to "first" byte (of 32-bit value)
CCFI_CFDEN_PosCoarse_Rx:
	movlw	0x07					// If value greater than 8, just adjust ptr
	cpfsgt	TempyVar0,1
	bra	CCFI_CFDEN_PosFine_Rx
	movf	POSTDEC0,1,0			// Move pointer right (shift left) by 8 bits
	movlw	0x08
	subwf	TempyVar0,1,1
	bra	CCFI_CFDEN_PosCoarse_Rx
CCFI_CFDEN_PosFine_Rx:
	movf	TempyVar0,1,1			//Test temp1, skip to end of shift if zero
	bz	CCFI_CFDEN_NegEnd_Rx
CCFI_CFDEN_PF_Loop_Rx:
	bcf	STATUS,0,0					// Clear carry for rotate
	rlcf	POSTINC0,1,0			// Rotate 32-bits
	rlcf	POSTINC0,1,0
	rlcf	POSTINC0,1,0
	rlcf	INDF0,1,0
	btfsc	POSTDEC0,7,0			// If active power overflows, set to maximum value
	bra	SetMaxCFInc_Rx
	movf	POSTDEC0,1,0
	movf	POSTDEC0,1,0
	decfsz	TempyVar0,1,1
	bra	CCFI_CFDEN_PF_Loop_Rx

	bra	CCFI_CFDEN_NegEnd_Rx

// Gain_Energy_React_Denominator is negative, shift right (divide by 2) per value in Gain_Energy_React_Denominator
CCFI_CFDEN_Neg_Rx:
	negf	TempyVar0,1				// Make positive
	movlw	0x1F					// Limit to 31 shifts or less
	andwf	temp1,1,1
	lfsr	0,Reactive_Energy_Raw+4
CCFI_CFDEN_NegCoarse_Rx:
	movlw	0x07					// If value greater than 8, just adjust ptr
	cpfsgt	TempyVar0,1
	bra	CCFI_CFDEN_NegFine_Rx
	movf	POSTINC0,1,0			// Move pointer left (shift right) by 8 bits
	movlw	0x08
	subwf	TempyVar0,1,1
	bra	CCFI_CFDEN_NegCoarse_Rx
CCFI_CFDEN_NegFine_Rx:
	movf	TempyVar0,1,1			// Test temp1, skip to end of shift if zero
	bz	CCFI_CFDEN_NegEnd_Rx
CCFI_CFDEN_NF_Loop_Rx:
	bcf	STATUS,0,0					// Clear carry for rotate
	btfsc	INDF0,7,0
	bsf	STATUS,0,0					// Keep sign bit
	rrcf	POSTDEC0,1,0			// Rotate 32-bits
	rrcf	POSTDEC0,1,0
	rrcf	POSTDEC0,1,0
	rrcf	POSTINC0,1,0
	movf	POSTINC0,1,0			// Move pointer back to start
	movf	POSTINC0,1,0
	decfsz	TempyVar0,1,1
	bra	CCFI_CFDEN_NF_Loop_Rx

	movf	POSTDEC0,1,0			// This pointer was at left end, move to right
	movf	POSTDEC0,1,0
	movf	POSTDEC0,1,0
CCFI_CFDEN_NegEnd_Rx:
	movff	POSTINC0,CF_Acc_Temp_Reactive
	movff	POSTINC0,CF_Acc_Temp_Reactive+1
	movff	POSTINC0,CF_Acc_Temp_Reactive+2
	movff	POSTINC0,CF_Acc_Temp_Reactive+3
	movff	POSTINC0,CF_Acc_Temp_Reactive+4
	movff	POSTINC0,CF_Acc_Temp_Reactive+5
	bra	CCFI_Exit_Rx

ClearCFInc_Rx:
	lfsr	1,Incremental_CF_Energy_React				// Point to 32-bit CF increment value
	bcf	INTCON,7,0					// Disable interrupts
	clrf	POSTINC1,0				// Result negative, clear increment (no CF pulse)
	clrf	POSTINC1,0
	clrf	POSTINC1,0
	clrf	POSTINC1,0
	bsf	INTCON,7,0					// Enable interrupts
	bra	CCFI_Exit_Rx

SetMaxCFInc_Rx:
	lfsr	1,Incremental_CF_Energy_React				// Point to 32-bit CF increment value
	bcf	INTCON,7,0					// Disable interrupts
	movlw	0xFF
	movwf	POSTINC1,0				// Result overflowed, set to maximum
	movwf	POSTINC1,0
	movwf	POSTINC1,0
	movlw	0x7F
	movwf	POSTINC1,0
	bsf	INTCON,7,0					//; Enable interrupts
CCFI_Exit_Rx:
	movff	SqrRoot,FSR0L
	movff	SqrRoot+1,FSR0H
	movff	temp2,FSR1L
	movff	temp3,FSR1H
	movff	Save_BSR	,BSR
_endasm;
//
//	// Multiply CFNUM
	Reactive_Energy_Raw.dword.LD=0;
	Reactive_Energy_Raw.dword.HD=0;
	MUL_S_64(&CF_Acc_Temp_Reactive,Gain_Energy_React_Numerator,&Reactive_Energy_Raw);

	if(!(Reactive_Energy_Raw.bits.b63))
	{
		loop_count=15;		//Neglect the lower 2 bytes
		while(loop_count--)
		Right_Shift_64(Reactive_Energy_Raw);

		INTCONbits.GIEH = 0;
		Incremental_CF_Energy_React = Reactive_Energy_Raw.dword.LD;
		INTCONbits.GIEH = 1;
	}
	else
	{
		INTCONbits.GIEH = 0;
		Incremental_CF_Energy_React = 0;
		INTCONbits.GIEH = 1;
	}
}
#endif


