/******************************************************************************

File Name:           sdadc_EnergyMeter.c
Dependencies:    Microchip Peripheral Library
Processor:           PIC18F65J90
Company:             Microchip Technology, Inc.

Copyright 2010 Microchip Technology Inc.  All rights reserved.

Microchip licenses to you the right to use, modify, copy and distribute
Software only when embedded on a Microchip microcontroller or digital signal
controller that is integrated into your product or third party product
(pursuant to the sublicense terms in the accompanying license agreement).

You should refer to the license agreement accompanying this Software for
additional information regarding your rights and obligations.

SOFTWARE AND DOCUMENTATION ARE PROVIDED ?AS IS? WITHOUT WARRANTY OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE.
IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER
CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR
OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR
CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF
SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
(INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.

Author                                     Date                                                Revision History / Comments
------------------------------------------------------------------------------------------------------------------------
Harsha Jagadish              1 - Jan - 2011                                                    Initial Version v1.0
Craig King					15 - Feb - 2011													   Port to 65J90 v1.1

*******************************************************************************/
#include "sdadc_EnergyMeter.h"
#include "debug_simulation_EnergyMeter.h"
#include "Config_EnergyMeter.h"

static UINT8  SaveBSRStatus;
Sw_Rtcc       Sw_Clk_Act,Sw_Clk_React;

//NOTE: Below Variables are necessarily have to be placed in BANK1 of RAM
#pragma udata  SDADC_VARIABLES_IN_BANK1=0x100
#define SDADC_ISR_VARIABLE_BANK 	1

FLAGS	Flags1;

static INT16  VoltageSample,CurrentSample;
static INT16  VoltageSample_Delayed,CurrentSample_Delayed;
static INT16  VoltageSample_PhaseComp,CurrentSample_PhaseComp;
static INT16  VoltageSample_90Lag;
static INT16  VoltageSample_90Lag_Delayed_Freq;

UINT32        Line_Freq;

UINT16        Signal_Frequency;

UINT32        Incremental_CF_Energy_Act;
UINT32        Incremental_CF_Energy_React;

UINT8         METER_CONSTANT_BY_100;
UINT8         PhaseComp_Amplitude,PhaseComp_Amplitude_90Lag;

QWORD_VAL     Summation_Active_Ins_Power,Summation_Reactive_Ins_Power;
QWORD_VAL     Summation_RMS_Square_Voltage,Summation_RMS_Square_Current;

MCP_CONFIG_CHK	Mcp_Con_Check;
SDMCP_CON     MCP_Config;

NO_LOAD_BLINK Blink_Energy;

UINT8         VoltageABSMaxAmp,CurrentABSMaxAmp;

UINT8         CF_Pulse_Width_W;
UINT8         CF_Pulse_Width_W_Rx;
UINT16        Period_W;

static UINT8  SampleCount=2;

static INT32  HPF_Intermediate_Result_1=0,HPF_Intermediate_Result_2=0;
static INT32  Voltage_Output_HPF_Delayed=0,Current_Output_HPF_Delayed=0;
static INT16  Voltage_Input_HPF_Delayed=0,Current_Input_HPF_Delayed=0;

UINT32  Energy_Act_Per_Meter_Constant;
UINT32  Energy_React_Per_Meter_Constant;

static UINT8  CF_HIGH_Time_Rem,CF_HIGH_Time_Rem_Fine;
static UINT8  CF_HIGH_Time_Rem_Rx,CF_HIGH_Time_Rem_Fine_Rx;
static UINT8  Energy_Act_Per_CF_Pulse;
static UINT8  Energy_React_Per_CF_Pulse;

static INT32  LPFAcc=0,LPFYD=0;
static INT16  LPF_X_D1=0;
static UINT8  LPI_temp1=0;

QWORD_VAL  Accumulate_Active_Ins_Power,Accumulate_Reactive_Ins_Power;

static QWORD_VAL  Accumulate_RMS_Square_Voltage,Accumulate_RMS_Square_Current;

static UINT8  LPI_Res0,LPI_Res1,LPI_Res2,LPI_Res3;
static UINT8  spi_mcp_chk;

static UINT8  CurrentABSMaxAmp_ISR;
static UINT8  VoltageABSMaxAmp_ISR;

static INT16  Phase_V_90_D1,Phase_V90;
static UINT8  Counter;

#pragma udata  SDADC_VARIABLES_FREQ_IN_BANK1=0x1FF
UINT8 FreqAddCounter;

//*******************************************************************************************************
#if defined(DEBUG_EN)
#pragma udata	DEBUG_DATA_1=0x500
INT16	DebugOnDMCIPlot1[128];
#pragma udata	DEBUG_DATA_2=0x600
INT16	DebugOnDMCIPlot2[128];
//#pragma udata	DEBUG_DATA_3=0x600
//INT16	DebugOnDMCIPlot3[128];
#endif

/***********************************************************************************************************
 Function    :   void Process_SDADC_Data(void)

 Include     :   sdadc_EnergyMeter.h
 
 Description : The Function process the SD-ADC signal samples to compute energy. This function will be called from
                        SD-ADC Data Ready interrupt. Within function, samples are processed to condition the signal by removing
		   offset, and instanteneous Active, Reactive power is calculated along with RMS square Voltage & Current values
		   The function also updates the CF Pulse blink for energy accumulation of both active & Reactive energy.
	
 Arguments   : None
 
 Remarks     : The Code is implemented in "IN LINE ASSEMBLY" for the time criticality of the execution time. As function will be called
                       every  1 / ( Data Ready Rate ) [ 1 / 3906 samples per second  =  256.4uS], it is quite necessary that function is executed with 
                       least of the possible time to reserve the processor bandwidth for other calculations & house keeping operations. 
		  
NOTE:          Caution need to be exercised while modifying the code as most part of fucntion is in In Line Assembly with "C" 
                      wrapper as macros to enhance the readability. Hence BSR value must be loaded properly as per variable location 
                      in RAM.
***********************************************************************************************************/
void Process_SDADC_Data(void)
{
	/*
	NOTE: Below part of the function is in In Line Assembly with "C" Macro wrapper to enhance the readability.
	The function uses all the variables that are used in In-Lin Assembly situated in BANK1 of RAM. Hence the BSR register
	is maintained set to 1. If any variable is placed in other banks, then it is necessary to explicitly change the BSR reg value
	by USER.
	Exercise optimization in coding to keep the function execution timing optimized to max possible. This function need to be
	executed between every Data Ready request from SD-ADC with little buffer time to execute the main rotuine of the code too.
	Hence time costly "C" functions like division are adivised to be avoided by keeping all divide operations limited to right shifts
	with sign extension in terms of 2 powers.
	*/

//************ In Line Assembly Starts *******
	//Select ADC
	SelectSDADC();		

	//Save Bank select register to maintain the integrity as to be used by compiler later
	SaveBankSelectRegister(SaveBSRStatus);
	
	//Change bank to 1-bank where all the variables that are used in this function are located
	LoadBSR(INITIATE_DATA_BUFFER_READ);	
	
	/* Frequency Calculation using zero crossing detection:
	    Determine the Zero crossing time on Voltage channel using Timer1 by comparing the present sample 
	    with previous sample for negetive to positive cross over	
	*/
	DetectZeroCrossingForFrequencyCalc(VoltageSample_90Lag);

	//Store one sample delayed current & Voltage samples to calculate phase compensation
	UpdateOneSampleDelayedRegs();
	
	//Read 16 bit data from channel 1 followed by channel 0 with MSB followed by LSB
	ReadDataFromSDADC(CurrentSample,VoltageSample);

	//Restore BSR value to transfer control to C Compiler
	RestoreBankSelectRegister(SaveBSRStatus);
	
	#if defined(ENABLE_SIMULATION)
		VoltageSample = VOLT_SAMP[128-SampleCount];
		CurrentSample = CURR_SAMP[128-(SampleCount)];
	#endif
//************* In Line Assembly Ends ***********

	//***************** The HPF for all the two channels of Current & Voltage ***************************************************************
	//	* HPF of First orer is added with cutoff 4.14Hz to filter out DC component in channel input so as to enable measurement of only AC component
	//	* HPF Filter equation: 	Y[n] = (1-(1/A0)) ( Y[n-1] + X[n] - X[n-1] ) - (Y[n-1]/A0) 		where Y[n] is present output & Y[n-1] is previous output and
	//												X[n] is present input & X[n-1] is previous input
	//	* Filter Coefficient A0 = e ^ (-2 x pi x fc / F)				where fc-> cutoff frequency & F-> sampling rate 6.4ksps
	//*******************************************************************************************************************************

	//**************** Current channel processing ********************* 
	ProcessSignalForOffsetRemoval(CurrentSample,Current_Input_HPF_Delayed,Current_Output_HPF_Delayed);

	//**************** Voltage channel processing ********************* 
	ProcessSignalForOffsetRemoval(VoltageSample,Voltage_Input_HPF_Delayed,Voltage_Output_HPF_Delayed);

//************ In Line Assembly Starts *******
	//Save Bank select register to maintain the integrity as to be used by compiler later
	SaveBankSelectRegister(SaveBSRStatus);
	
	//Change bank to 1-bank where all the variables that are used in this function are located
	LoadBSR(INITIATE_DATA_BUFFER_READ)
			
	//Compute the maximum absolute value of voltage sample value & correct any negetive most value less by "1"
	CheckSignalLimitingCondition(VoltageSample);

	//Compute the maximum absolute value of current sample value & correct any negetive most value less by "1"
	CheckSignalLimitingCondition(CurrentSample);
	
	//Updates the Max Voltage signal absolute peak value
	GetAbsMaximumVoltageAmp();
	
	//Updates the Max Voltage signal absolute peak value
	GetAbsMaximumCurrentAmp();
	
	//completes the Read cycle of SD-ADC over SPI by deselecting & reselecting Chip Select
	CompleteReadSDADC();
		
	ClearInterruptSDADC();

	//Phase compensate between voltage & current for signals to be in phase at PF = 1.0
	//Phase compensation is implemented with the interpolation method
	//Refer Reference Design Doc to understand Phase compensation angle calculation & interpolation technique
	PhaseCompensate_VI();

	//Update CF Pulse blink if the incremental active energy accumulated is more than 1 / METER CONSTANT
	//and update the Energy Accumualtion register by value 1 / METER CONSTANT
	UpdateActiveEnergy_CFPulse();
	
	#if defined(ACTIVE_POWER_ENABLE_COMPUTATION) || defined(ACTIVE_ENERGY_ENABLE_COMPUTATION)
	/*
	Calculate Active instantenesous power and accumualte over signal cycle of 128 samples
	*/
	SummateInstanteneousPower(VoltageSample_PhaseComp,CurrentSample_PhaseComp,Accumulate_Active_Ins_Power,ActivePower);
	#endif

	/*	Use Phase Compensated Voltage sample to get 90 degree phase lag to compute the reactive components.
		Phase lag of 90 degree is obtained by using an LPF in cutoff band so that output of LPF is almost phase lagged
		by 90 degree and attenuated. This LPF output is processed for Gain amplification to restore signal amplitude levels
		and Phase compensation by the technique of interpolation is applied to get exact 90 degree phase lag in voltage samples
	*/
	ProcessVoltageFor90PhaseShift();
	

	//Update CF Pulse blink if the incremental reactive energy accumulated is more than 1 / METER CONSTANT
	//and update the Energy Accumualtion register by value 1 / METER CONSTANT
	UpdateReactiveEnergy_CFPulse();
	
	#if defined(REACTIVE_POWER_ENABLE_COMPUTATION) || defined(REACTIVE_ENERGY_ENABLE_COMPUTATION)
	/*
	Calculate Reactive instantenesous power and accumualte over signal cycle of 128 samples
	*/
	SummateInstanteneousPower(VoltageSample_90Lag,CurrentSample_PhaseComp,Accumulate_Reactive_Ins_Power,ReactivePower);
	#endif
	
	#if defined(RMS_VOLTAGE_ENABLE_COMPUTATION)
	/*
	Calculate Voltage instantenesous square value for RMS and accumualte over signal cycle of 128 samples
	*/
	SummateInstanteneousSquareRMS(VoltageSample_Delayed,Accumulate_RMS_Square_Voltage,vrms);
	#endif
	
	#if defined(RMS_CURRENT_ENABLE_COMPUTATION)
	/*
	Calculate Current instantenesous square value for RMS and accumualte over signal cycle of 128 samples
	*/
	SummateInstanteneousSquareRMS(CurrentSample_Delayed,Accumulate_RMS_Square_Current,irms);
	#endif
	
	/*
	Check SD-ADC Configuration periodically to validate the integrity of the samples being recieved.
	This function checks all configuration registers for its desired configuration value to check if any external
	turbulance had caused the config reset. If so error is flagged, which enables re-configuration of SD-ADC
	in main routine
	*/
	CheckSDADCConfig(Mcp_Con_Check,MCP_Config);

	SelectSDADC();

	//Restore BSR value to transfer control to C Compiler
	RestoreBankSelectRegister(SaveBSRStatus);
//************* In Line Assembly Ends ***********

	#if defined(DEBUG_EN)
	    DebugOnDMCIPlot1[128-SampleCount] = VoltageSample_PhaseComp;	//Use this array to plot the voltage and current wave forms in DMCI tool
	 //   DebugOnDMCIPlot2[128-SampleCount] = VoltageSample_90Lag;	//Use this array to plot the voltage and current wave forms in DMCI tool
		DebugOnDMCIPlot2[128-SampleCount] = VoltageSample_Delayed;	//Use this array to plot the voltage and current wave forms in DMCI tool
	#endif

	//Decrement sample count. If zero, indicating end of 128 samples/cycle, then restore count to 128 (0x80), 
	// set cycle complete flag, and save data
	if(!--SampleCount)
	{
		
		//Read Signal period for 128 sample accumulation. Period is used later for energy calculation
		ReadSignalPeriod(Period_W);

		SampleCount = SamplesPerSignalCycle; //restore back the sample counter per cycle back to 128 samples
		SetSignalCycleComplete();			//Set Cycle of 128 samples complete flag
	
	//No Load Blink Check condition. If there is no blink within precalculated time, then clear energy accumulator	
		if( (Blink_Energy.bits.act_blink==0) &&
            ((Sw_Clk_Act.time.clk_min>=CREEP_THRESHOLD_MINUTE) && 
			(Sw_Clk_Act.time.clk_sec>=CREEP_THRESHOLD_SECOND)) )
		{
			Sw_Clk_Act.time.clk_sec = 0;	//Re initialize active no load blinking counter
			Sw_Clk_Act.time.clk_min = 0;	//Re initialize active no load blinking counter
			Energy_Act_Per_Meter_Constant = 0;						//Clear Active energy accumulator
			Incremental_CF_Energy_Act = 0;						//clear Active energy incremental accumulator
		}
		else if((Blink_Energy.bits.act_blink==1))
		{
			Sw_Clk_Act.time.clk_sec = 0;	//Re initialize active no load blinking counter
			Sw_Clk_Act.time.clk_min = 0;	//Re initialize active no load blinking counter	
			Blink_Energy.bits.act_blink = 0;	//Clear blink indicator			
		}
		
		//Reactive no load check
		if( (Blink_Energy.bits.react_blink==0) &&
		    ((Sw_Clk_React.time.clk_min>=CREEP_THRESHOLD_MINUTE) && 
		    (Sw_Clk_React.time.clk_sec>=CREEP_THRESHOLD_SECOND)) )
		{
			Sw_Clk_React.time.clk_sec = 0;	//Re initialize active no load blinking counter
			Sw_Clk_React.time.clk_min = 0;	//Re initialize active no load blinking counter
			Energy_React_Per_Meter_Constant = 0;			//Clear Reactive energy accumulator
			Incremental_CF_Energy_React = 0;			//clear Reactive energy incremental accumulator
		}	
		else if((Blink_Energy.bits.react_blink==1))
		{
			Sw_Clk_React.time.clk_sec = 0;	//Re initialize active no load blinking counter
			Sw_Clk_React.time.clk_min = 0;	//Re initialize active no load blinking counter	
			Blink_Energy.bits.react_blink = 0;	//Clear blink indicator			
		}

	//Save 40-bit power working accumulators  every line cycle		
		Summation_Active_Ins_Power.dword.LD = Accumulate_Active_Ins_Power.dword.LD;
		Summation_Active_Ins_Power.dword.HD = Accumulate_Active_Ins_Power.dword.HD;
	// Clear 40-bit power working accumulators
		Accumulate_Active_Ins_Power.dword.LD = 0;		
		Accumulate_Active_Ins_Power.dword.HD = 0;
		
	//Save 40-bit voltage and current working accumulators every line cycle
		Summation_Reactive_Ins_Power.dword.LD = Accumulate_Reactive_Ins_Power.dword.LD;
		Summation_Reactive_Ins_Power.dword.HD = Accumulate_Reactive_Ins_Power.dword.HD;
	// Clear 40-bit power working accumulators
		Accumulate_Reactive_Ins_Power.dword.LD = 0;		
		Accumulate_Reactive_Ins_Power.dword.HD = 0;
		
        if(FreqAddCounter == 4)
        {
            FreqAddCounter = 0;
            Signal_Frequency = Line_Freq >> 2;
            Line_Freq = 0;
        }
        else if(FreqAddCounter > 4)
        {
	        FreqAddCounter = 0;
	        Line_Freq = 0;
	    }
	//Save 40-bit power working accumulators  every line cycle	
		Summation_RMS_Square_Current.dword.LD = Accumulate_RMS_Square_Current.dword.LD;
		Summation_RMS_Square_Current.dword.HD = Accumulate_RMS_Square_Current.dword.HD;
	// Clear 40-bit voltage and current working accumulators
		Accumulate_RMS_Square_Current.dword.LD = 0;		
		Accumulate_RMS_Square_Current.dword.HD = 0;	
	
	//Save 40-bit power working accumulators  every line cycle	
		Summation_RMS_Square_Voltage.dword.LD = Accumulate_RMS_Square_Voltage.dword.LD;
		Summation_RMS_Square_Voltage.dword.HD = Accumulate_RMS_Square_Voltage.dword.HD;
	// Clear 40-bit voltage and current working accumulators
		Accumulate_RMS_Square_Voltage.dword.LD = 0;		
		Accumulate_RMS_Square_Voltage.dword.HD = 0;
		
	// Save absolute maximum values  every line cycle
		CurrentABSMaxAmp = CurrentABSMaxAmp_ISR;
	// Clear absolute maximum values
		CurrentABSMaxAmp_ISR = 0;
		
		VoltageABSMaxAmp = VoltageABSMaxAmp_ISR;
	// Clear absolute maximum values
		VoltageABSMaxAmp_ISR = 0;	
	}

}


