/******************************************************************************

File Name:           sdadc_EnergyMeter.h
Dependencies:    Microchip Peripheral Library
Processor:           PIC18F87J72
Company:             Microchip Technology, Inc.

Copyright 2010 Microchip Technology Inc.  All rights reserved.

Microchip licenses to you the right to use, modify, copy and distribute
Software only when embedded on a Microchip microcontroller or digital signal
controller that is integrated into your product or third party product
(pursuant to the sublicense terms in the accompanying license agreement).

You should refer to the license agreement accompanying this Software for
additional information regarding your rights and obligations.

SOFTWARE AND DOCUMENTATION ARE PROVIDED ?AS IS? WITHOUT WARRANTY OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE.
IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER
CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR
OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR
CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF
SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
(INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.

Author                                     Date                                                Revision History / Comments
------------------------------------------------------------------------------------------------------------------------
Harsha Jagadish              1 - Jan - 2011                                                    Initial Version v1.0
*******************************************************************************/
#ifndef __sdadc_EnergyMeter
#define __sdadc_EnergyMeter

#include "HardwareProfile_EnergyMeter.h"
#include "Config_EnergyMeter.h"
#include "signalProcess_EnergyMeter.h"
#include "RegDef_EnergyMeter.h"
#include "system_EnergyMeter.h"


//Read SPI data from Sigma Delta ADC by sending 0x00
#define READ_DATA_FROM_SDADC_SPI                    0x00

//Initiate data buffer read from Sigma Delta ADC by sending 0x01
#define INITIATE_DATA_BUFFER_READ                   0x01


//Sample per cycle @ 60Hz
#define SAMPLE_PER_CYCLE        65.1

//Timer1 Prescaler as set in EnergyMeter Initialization
#define TIMER1_PRESCALE         4

//Timer 1 count between two successive samples - sample period in timer 1 count = ((1/60)/65.1)/0.4x10^-6 = 640
// Sample per cycle @ 60Hz = 65.1
//Timer1 prescale - 1:4
#define COUNTS_PER_SAMPLE      (((1/60)/SAMPLE_PER_CYCLE)*GetPeripheralClock())/(TIMER1_PRESCALE)

#define COUNTS_PER_SAMPLE_BY_2  (COUNTS_PER_SAMPLE/2)
#define COUNTS_PER_SAMPLE_BY_4  (COUNTS_PER_SAMPLE/4)
#define COUNTS_PER_SAMPLE_BY_3_4  (COUNTS_PER_SAMPLE*3/4)

/****************************************************************************
  Macro:   SaveBankSelectRegister(SaveVariable)

 Description:   Macro saves the Bank Select Register of RAM to a temporary location to save the compiler BSR tracking state,
                         when in line assembly is used.

  Precondition:  None

  Parameters:   SaveVariable - Variable to which the BSR will be stored
  
  Returns:   None
  
  Remarks: * It is necessary to save the BSR register content to maintain the compiler BSR tracking state, when IN-LINE ASSEMBLY
                     is used, where BSR values are changed based on the variable being accessed
                  * This macro must always use "movff" instruction for data movement to save BSR into variable to avoid BSR value change
  ***************************************************************************/
#define SaveBankSelectRegister(SaveVariable)           _asm  movff  BSR,SaveVariable  _endasm

/****************************************************************************
  Macro:   RestoreBankSelectRegister(SaveVariable)

 Description:   Macro restores the Bank Select Register of RAM from a known temporary location to restore BSR status before "C" code
                         to transfer control back to assembly.

  Precondition:  This macro must be called only if "SaveBankSelectRegister" is executed prior to this. Else garbage may get loaded to BSR

  Parameters:   SaveVariable - Variable from which the BSR register will be restored
  
  Returns:   None
  
  Remarks: * This macro must be called only if "SaveBankSelectRegister" is executed prior to this. Else garbage may get loaded to BSR
                  * This macro must always use "movff" instruction for data movement to restore BSR from variable to avoid BSR value change
  ***************************************************************************/
#define RestoreBankSelectRegister(SaveVariable)      _asm  movff  SaveVariable,BSR  _endasm


/****************************************************************************
  Macro:   LoadBSR()

 Description:   Macro Loads the BSR register with bank from which variable need to be accessed

  Precondition:  It is preferebale to have BSR content stored, before its value is altered

  Parameters:   bank -  Bank from which variables stored need to be accessed
  
  Returns:   None
  
  Remarks: * It is necessary to save the BSR register content to maintain the compiler BSR tracking state, when IN-LINE ASSEMBLY
                     is used, where BSR values are changed based on the variable being accessed
                  * This macro must always use "movlb" instruction for bank selection
  ***************************************************************************/
#define LoadBSR(bank)                _asm  movlb	bank  _endasm


/****************************************************************************
  Macro:   SelectSDADC()

 Description:   Macro selects the SD-ADC chip select to enable data transfer

  Precondition:  Chip Select pin must be made output port

  Parameters:   None
  
  Returns:   None
  
  Remarks: None
  ***************************************************************************/
#define SelectSDADC()                _asm  bcf		EM_ASM_SDADC_CS,0  _endasm


/****************************************************************************
  Macro:   EnableSDADC()

 Description:   Macro selects the SD-ADC chip select to enable data transfer

  Precondition:  Chip Select pin must be made output port

  Parameters:   None
  
  Returns:   None
  
  Remarks: None
  ***************************************************************************/
#define EnableSDADC()                EM_SDADC_CS = 0

/****************************************************************************
  Macro:   DisableSDADC()

 Description:   Macro deselects the SD-ADC chip select to enable data transfer

  Precondition:  Chip Select pin must be made output port

  Parameters:   None
  
  Returns:   None
  
  Remarks: None
  ***************************************************************************/
#define DisableSDADC()                EM_SDADC_CS = 1


#if defined(REACTIVE_POWER_ENABLE_COMPUTATION) ||\
	defined(REACTIVE_ENERGY_ENABLE_COMPUTATION)
/****************************************************************************
  Macro:   DetectZeroCrossingForFrequencyCalc(signal_sample)

 Description:   Macro tracks the Voltage line for zero crossing. It calculates the time between successive zero crossings using
                         TIMER1. Timer counts is used in determining the period of the line signal, thus helping in determining the frequency.
                         This function is crucial to determine frequency of line, which is later used in Gain & Phase compensation of reactive
                          component calculation.

  Precondition:  TIMER1 must be configured with 1:4 prescale mode with 16bit read/write mode @ 10MIPS operation
                           Variables used in this macro must be in BANK 1 

  Parameters:   signal_sample - integer sample value using which, signal's zero crsssing is detected
  
  Returns:   Updates the "Line_Freq" variable with Timer counts between two successive zero crossing
  
  Remarks:  Zero crossing is detected using present & previous sample by checking the value transition from negative to positive sign
  ***************************************************************************/
#define DetectZeroCrossingForFrequencyCalc(signal_sample)    _asm  movf	signal_sample+1,0,1         \
                                                      xorwf	 signal_sample##_Delayed_Freq+1,0,1     \
                                                      btfss	 WREG,7,0                               \
                                                      bra		Cycle_Prog                          \
                                                      btfss	 signal_sample##_Delayed_Freq+1,7,1     \
                                                      bra		Cycle_Prog                          \
                                                      movf	TMR1L,0,0                               \
                                                      movwf	Line_Freq,1                             \
                                                      movf	TMR1H,0,0                               \
                                                      movwf	Line_Freq+1,1                           \
                                                      clrf	TMR1H,0                                 \
                                                      clrf	TMR1L,0                                 \
                                                      movf   signal_sample##_Delayed_Freq,0,1       \
                                                      subwf  signal_sample,0,1                      \
                                                      movwf  SampleDiff,1                           \
                                                      movf   signal_sample##_Delayed_Freq+1,0,1     \
                                                      subwfb signal_sample+1,0,1                    \
                                                      movwf  SampleDiff+1,1                         \
                                                      clrf   SampleDiffToZero,1                     \
                                                      clrf   SampleDiffToZero+1,1                   \
                                                      movf   signal_sample##_Delayed_Freq,0,1       \
                                                      subwf  SampleDiffToZero,1,1                   \
                                                      movf   signal_sample##_Delayed_Freq+1,0,1     \
                                                      subwfb SampleDiffToZero+1,1,1                 \
                                                      bcf   STATUS,0,0                              \
                                                      rrcf  SampleDiff+1,0,1                        \
                                                      cpfsgt SampleDiffToZero+1,1                   \
                                                      bra   more_than_half                          \
                                                      bcf   STATUS,0,0                              \
                                                      rrcf   WREG,0,0                               \
                                                      cpfsgt SampleDiffToZero+1,1                   \
                                                      bra   more_than_quarter                       \
                                                      bra   between_25_50                           \
                                                    more_than_quarter:                              \
                                                      bra   between_0_25                            \
                                                    more_than_half:                                 \
                                                      bcf   STATUS,0,0                              \
                                                      rrcf  SampleDiff+1,1,1                        \
                                                      addwf SampleDiff+1,0,1                        \
                                                      cpfsgt SampleDiffToZero+1,1                   \
                                                      bra   between_50_75                           \
                                                      bra   between_75_100                          \
                                                    between_0_25:                                   \
                                                      movlw 0x80                                    \
                                                      subwf Line_Freq,1,1                           \
                                                      movlw 0x02                                    \
                                                      subwfb Line_Freq+1,1,1                        \
                                                      movf  TimerRemainder,0,1                      \
                                                      addwf Line_Freq,1,1                           \
                                                      movf  TimerRemainder+1,0,1                    \
                                                      addwf Line_Freq+1,1,1                         \
                                                      movlw 0xE0                                    \
                                                      movwf TimerRemainder,1                        \
                                                      movlw 0x01                                    \
                                                      movwf TimerRemainder+1,1                      \
                                                      bra   Cycle_Prog                              \
                                                    between_25_50:                                  \
                                                      movlw 0xE0                                    \
                                                      subwf Line_Freq,1,1                           \
                                                      movlw 0x01                                    \
                                                      subwfb Line_Freq+1,1,1                        \
                                                      movf  TimerRemainder,0,1                      \
                                                      addwf Line_Freq,1,1                           \
                                                      movf  TimerRemainder+1,0,1                    \
                                                      addwf Line_Freq+1,1,1                         \
                                                      movlw 0x40                                    \
                                                      movwf TimerRemainder,1                        \
                                                      movlw 0x01                                    \
                                                      movwf TimerRemainder+1,1                      \
                                                      bra   Cycle_Prog                              \
                                                    between_50_75:                                  \
                                                      movlw 0x40                                    \
                                                      subwf Line_Freq,1,1                           \
                                                      movlw 0x01                                    \
                                                      subwfb Line_Freq+1,1,1                        \
                                                      movf  TimerRemainder,0,1                      \
                                                      addwf Line_Freq,1,1                           \
                                                      movf  TimerRemainder+1,0,1                    \
                                                      addwf Line_Freq+1,1,1                         \
                                                      movlw 0xA0                                    \
                                                      movwf TimerRemainder,1                        \
                                                      movlw 0x00                                    \
                                                      movwf TimerRemainder+1,1                      \
                                                      bra   Cycle_Prog                              \
                                                    between_75_100:                                 \
                                                      movlw 0xA0                                    \
                                                      subwf Line_Freq,1,1                           \
                                                      movlw 0x00                                    \
                                                      subwfb Line_Freq+1,1,1                        \
                                                      movf  TimerRemainder,0,1                      \
                                                      addwf Line_Freq,1,1                           \
                                                      movf  TimerRemainder+1,0,1                    \
                                                      addwf Line_Freq+1,1,1                         \
                                                      movlw 0x00                                    \
                                                      movwf TimerRemainder,1                        \
                                                      movlw 0x00                                    \
                                                      movwf TimerRemainder+1,1                      \
                                                      bra   Cycle_Prog                              \
                                                Cycle_Prog:                                         \
                                                      movff	signal_sample+1,signal_sample##_Delayed_Freq+1	\
                                                _endasm
#else
#define DetectZeroCrossingForFrequencyCalc(x)  
#endif


/****************************************************************************
  Macro:   ReadDataFromSDADC(Channel1,Channel0)

 Description:   Macro Reads the 4 bytes data from Sigma Delta ADC through SPI interms of 4 SPI reads. Macro reads CH1 Current
                         data (MSB followed by LSB) first and then CH0 Voltage data in same order.

  Precondition:  SD-ADC must be initialized for sequential 4 byte read with SD-ADC Chip select selected before this operation is carried
                           Variables used in this macro must be in BANK 1 

  Parameters:   Channel1 - Current channel input of SDADC
                         Channel0 - Voltage channel input of SDADC
  
  Returns:   Updates "VoltageSample" & "CurrentSample" variables with latest Voltage & Current readings
  
  Remarks: Necessary to maintain required number of NOPS between successive reads to complete SPI read process
  ***************************************************************************/	
#define  ReadDataFromSDADC(Channel1,Channel0)	_asm 						\
										movlw	INITIATE_DATA_BUFFER_READ	\
										movwf	SSPBUF,0					\
										nop									\
										nop									\
										nop									\
										nop									\
										nop									\
										nop									\
										nop									\
										nop									\
										nop									\
										nop									\
							            movlw	READ_DATA_FROM_SDADC_SPI	\
										movwf	SSPBUF,0					\
										nop									\
										nop									\
										nop									\
										nop									\
										nop									\
										nop									\
										nop									\
										nop									\
										nop									\
										nop									\
										movff	SSPBUF,Channel1+1		    \
										movwf	SSPBUF,0					\
										nop									\
										nop									\
										nop									\
										nop									\
										nop									\
										nop									\
										nop									\
										nop									\
										nop									\
										nop									\
										movff	SSPBUF,Channel1     		\
										movwf	SSPBUF,0					\
										nop									\
										nop									\
										nop									\
										nop									\
										nop									\
										nop									\
										nop									\
										nop									\
										nop									\
										nop									\
										movff	SSPBUF,Channel0+1    		\
										bcf 	PIR1,3,0					\
										movwf	SSPBUF,0					\
										nop									\
										nop									\
										nop									\
									SPI_Wait:								\
										btfss	PIR1,3,0					\
										bra		SPI_Wait					\
										movff	SSPBUF,Channel0     		\
									_endasm												

									
/****************************************************************************
  Macro:   UpdateOneSampleDelayedRegs()

 Description:   Macro updates the one sample delayed Voltage & Current variables. Delayed samples are used in the phase correction between
                         Voltage & Current

  Precondition:  This macro must be executed only after Present Voltage & Current Samples from SD-ADC are read.
                           Variables used in this macro must be in BANK 1 

  Parameters:   None
  
  Returns:   Updates the "CurrentSample_Delayed" & "VoltageSample_Delayed" variable 
  
  Remarks: None
  ***************************************************************************/
#define  UpdateOneSampleDelayedRegs()		_asm							\
											movff	VoltageSample+1,VoltageSample_Delayed+1 \
											movff	VoltageSample,VoltageSample_Delayed     \
											movff	CurrentSample+1,CurrentSample_Delayed+1 \
											movff	CurrentSample,CurrentSample_Delayed     \
											_endasm


#if defined(ACTIVE_ENERGY_ENABLE_COMPUTATION)
/****************************************************************************
  Macro:   UpdateActiveEnergy_CFPulse()

 Description:   Macro updates Active energy accumulated over last signal cycle and checks if it is more than 
                         energy = 1 / METER CONSTANT. If it is more than that, then the CF Pulse is turned on definite time.
                         Macro also updates the Active Energy accumulation register by value equal to 1 / METER CONSTANT.

  Precondition:  Incremental energy comes with the gain correction from main routine, where summation of active instanteneous power
                           is converted to energy with period multiplication.
                           Variables used in this macro must be in BANK 1 

  Parameters:   None
  
  Returns:   Updates the "Energy_Act_Per_Meter_Constant" & "ENERGY_ACT" registers
  
  Remarks: The Active incremental energy over last signal cycle is multiplied with period. Followed by it is the
                   Gain correction and updated to "Incremental_CF_Energy_Act" (with division by 128 so that when added over
                   signal cycles, incremental values is approximated to original value). This incremental energy is accumulated, which
                  on over flow represents the energy equivalent to (1/METER CONSTANT) kWh, which turns on CF Pusle for definite period
  ***************************************************************************/
#define  UpdateActiveEnergy_CFPulse()	   		_asm										\
												movf	Incremental_CF_Energy_Act,0,1		\
												addwf	Energy_Act_Per_Meter_Constant,1,1	\
												movf	Incremental_CF_Energy_Act+1,0,1		\
												addwfc	Energy_Act_Per_Meter_Constant+1,1,1	\
												movf	Incremental_CF_Energy_Act+2,0,1		\
												addwfc	Energy_Act_Per_Meter_Constant+2,1,1	\
												movf	Incremental_CF_Energy_Act+3,0,1		\
												addwfc	Energy_Act_Per_Meter_Constant+3,1,1	\
												btfss	Energy_Act_Per_Meter_Constant+3,7,1	\
												bra		LPI_CF_chk_carry					\
												bcf		EM_ASM_CF_PULSE_ACT,0				\
												bra		LPI_CF_end							\
											LPI_CF_chk_carry:								\
												bnc		LPI_CF_no_carry						\
												movf	CF_Pulse_Width_W,0,1				\
												btfss	STATUS,2,0							\
												bsf		EM_ASM_CF_PULSE_ACT,0				\
												movwf	CF_HIGH_Time_Rem,1					\
												incf	Energy_Act_Per_CF_Pulse,1,1			\
												bsf		Blink_Energy,0,1					\
												clrf	CF_HIGH_Time_Rem_Fine,1				\
												bsf		CF_HIGH_Time_Rem_Fine,3,1			\
												bra		LPI_CF_end							\
											LPI_CF_no_carry:								\
												decfsz	CF_HIGH_Time_Rem_Fine,1,1			\
												bra		LPI_CF_end							\
												bsf		CF_HIGH_Time_Rem_Fine,3,1			\
												dcfsnz	CF_HIGH_Time_Rem,1,1				\
												bcf		EM_ASM_CF_PULSE_ACT,0				\
											LPI_CF_end:										\
												movf	METER_CONSTANT_BY_100,0,1			\
												cpfseq	Energy_Act_Per_CF_Pulse,1			\
												bra		eng_acc								\
												clrf	Energy_Act_Per_CF_Pulse,1			\
												movlb	3									\
												bcf		STATUS,0,0							\
												clrf	WREG,0								\
												incf	ENERGY_ACT,1,1						\
												addwfc	ENERGY_ACT+1,1,1					\
												addwfc	ENERGY_ACT+2,1,1					\
												addwfc	ENERGY_ACT+3,1,1					\
										eng_acc:											\
												movlb	1										\
											_endasm	
											
#else
#define UpdateActiveEnergy_CFPulse()  
#endif


#if defined(REACTIVE_ENERGY_ENABLE_COMPUTATION)
/****************************************************************************
  Macro:   UpdateReactiveEnergy_CFPulse()

 Description:   Macro updates Reactive energy accumulated over last signal cycle and checks if it is more than 
                         energy = 1 / METER CONSTANT. If it is more than that, then the CF Pulse is turned on definite time.
                         Macro also updates the Reactive Energy accumulation register by value equal to 1 / METER CONSTANT.

  Precondition:  Incremental energy comes with the gain correction from main routine, where summation of Reactive instanteneous power
                           is converted to energy with period multiplication.
                           Variables used in this macro must be in BANK 1 

  Parameters:   None
  
  Returns:   Updates the "Energy_React_Per_Meter_Constant" & "ENERGY_REACT" registers
  
  Remarks: The Reactive incremental energy over last signal cycle is multiplied with period. Followed by it is the
                   Gain correction and updated to "Incremental_CF_Energy_React" (with division by 128 so that when added over
                   signal cycles, incremental values is approximated to original value). This incremental energy is accumulated, which
                  on over flow represents the energy equivalent to (1/METER CONSTANT) kVARh, which turns on CF Pusle for definite period
  ***************************************************************************/
#define  UpdateReactiveEnergy_CFPulse()		_asm											\
											movf	Incremental_CF_Energy_React,0,1			\
											addwf	Energy_React_Per_Meter_Constant,1,1		\
											movf	Incremental_CF_Energy_React+1,0,1		\
											addwfc	Energy_React_Per_Meter_Constant+1,1,1	\
											movf	Incremental_CF_Energy_React+2,0,1		\
											addwfc	Energy_React_Per_Meter_Constant+2,1,1	\
											movf	Incremental_CF_Energy_React+3,0,1		\
											addwfc	Energy_React_Per_Meter_Constant+3,1,1	\
											btfss	Energy_React_Per_Meter_Constant+3,7,1	\
											bra		LPI_CF_chk_carry_Rx						\
											bcf		EM_ASM_CF_PULSE_REACT,0					\
											bra		LPI_CF_end_Rx							\
									LPI_CF_chk_carry_Rx:									\
											bnc		LPI_CF_no_carry_Rx						\
											movf	CF_Pulse_Width_W_Rx,0,1					\
											btfss	STATUS,2,0								\
											bsf		EM_ASM_CF_PULSE_REACT,0					\
											movwf	CF_HIGH_Time_Rem_Rx,1					\
											incf	Energy_React_Per_CF_Pulse,1,1			\
											bsf		Blink_Energy,1,1						\
											clrf	CF_HIGH_Time_Rem_Fine_Rx,1				\
											bsf		CF_HIGH_Time_Rem_Fine_Rx,3,1			\
											bra		LPI_CF_end_Rx							\
									LPI_CF_no_carry_Rx:										\
											decfsz	CF_HIGH_Time_Rem_Fine_Rx,1,1			\
											bra		LPI_CF_end_Rx							\
											bsf		CF_HIGH_Time_Rem_Fine_Rx,3,1			\
											dcfsnz	CF_HIGH_Time_Rem_Rx,1,1					\
											bcf		EM_ASM_CF_PULSE_REACT,0					\
									LPI_CF_end_Rx:											\
											movf	METER_CONSTANT_BY_100,0,1				\
											cpfseq	Energy_React_Per_CF_Pulse,1				\
											bra		eng_acc_r								\
											clrf	Energy_React_Per_CF_Pulse,1				\
											movlb	3										\
											bcf		STATUS,0,0								\
											clrf	WREG,0									\
											incf	ENERGY_REACT,1,1						\
											addwfc	ENERGY_REACT+1,1,1						\
											addwfc	ENERGY_REACT+2,1,1						\
											addwfc	ENERGY_REACT+3,1,1						\
									eng_acc_r:												\
											movlb	1										\
									_endasm
#else
#define UpdateReactiveEnergy_CFPulse()  
#endif



/****************************************************************************
  Macro:   CheckSignalLimitingCondition(SignalInput)

 Description:   Macro Checks the signal sample in variable "VoltageSample" or "CurrentSample" for its limiting condition values
                         on positive and negative ends.

  Precondition:  Variables used in this macro must be in BANK 1 
                           
  Parameters:   SignalInput -  Signal whose limiting condition need to be checked. Signal includes Voltage & Current samples
  
  Returns:   Updates SignalInput sample to its limiting values in case of overflow
  
  Remarks: None
  ***************************************************************************/
#define  CheckSignalLimitingCondition(SignalInput)			_asm						\
															movlw	0x80				\
															xorwf	SignalInput+1,0,1	\
															iorwf	SignalInput,0,1		\
															btfsc	STATUS,2,0			\
															incf	SignalInput,1,1		\
															_endasm


/****************************************************************************
  Macro:   GetAbsMaximumVoltageAmp()

 Description:   Macro Checks the voltage signal sample for its Absolute Maximum Amplitude

  Precondition:  Variables used in this macro must be in BANK 1.
                           The macro operates on "VoltageSample" & "VoltageABSMaxAmp" variable in BANK1
                           
  Parameters:   None
  
  Returns:   Updates "VoltageABSMaxAmp" variable
  
  Remarks: None
  ***************************************************************************/
#define  GetAbsMaximumVoltageAmp()						_asm						\
														movf	VoltageSample+1,0,1	\
														btfsc	STATUS,4,0			\
														comf	VoltageSample+1,0,1	\
														cpfsgt	VoltageABSMaxAmp_ISR,1	\
														movwf	VoltageABSMaxAmp_ISR,1	\
														_endasm


/****************************************************************************
  Macro:   GetAbsMaximumCurrentAmp()

 Description:   Macro Checks the Current signal sample for its Absolute Maximum Amplitude

  Precondition:  Variables used in this macro must be in BANK 1.
                           The macro operates on "CurrentSample" & "CurrentABSMaxAmp" variable in BANK1
                           
  Parameters:   None
  
  Returns:   Updates "CurrentABSMaxAmp" variable
  
  Remarks: None
  ***************************************************************************/
#define  GetAbsMaximumCurrentAmp()						_asm						\
														movf	CurrentSample+1,0,1	\
														btfsc	STATUS,4,0			\
														comf	CurrentSample+1,0,1	\
														cpfsgt	CurrentABSMaxAmp_ISR,1	\
														movwf	CurrentABSMaxAmp_ISR,1	\
														_endasm
 

/****************************************************************************
  Macro:   CompleteReadSDADC()

 Description:   Macro completes the Read cycle of SD-ADC over SPI by deselecting & reselecting Chip Select

  Precondition:  necessary to maintain the required NOPs for the Chip select reset to be effective
                           
  Parameters:   None
  
  Returns:   None
  
  Remarks: None
  ***************************************************************************/
#define  CompleteReadSDADC()						_asm						\
													movwf	SSPBUF,0			\
													bsf		EM_ASM_SDADC_CS,0	\
													nop							\
													nop							\
													nop							\
													nop							\
													nop							\
													nop							\
													nop							\
													nop							\
													nop							\
													bcf		EM_ASM_SDADC_CS,0	\
													_endasm


/****************************************************************************
  Macro:   PhaseCompensate_VI()

 Description:   Macro implemets Phase Compensation by the method of interpolation between Voltage & Current.
                         The Phase Compensation angle magnitude and sign is obtained by calibration process. The Phase Compensation
                         is implemented as:
                           1. Xp = X[n] - X[n-1]
                           2. Xp = Xp * Phase Compensation Magnitude
                           3. Xp = X[n-1] + Xp
                           Depending on sign of phase correction angle, 
			if positive, present sample of current & phase compensated voltage samples are used for power calculation
			if negative, previous sample of current & phase compensated voltage are used for power calculation

						   
  Precondition:  Variables used in this macro must be in BANK 1 
                           Gets phase correction angle sign from register Flags1
                           
  Parameters:   None
  
  Returns:   Updates "VoltageSample_PhaseComp" & "CurrentSample_PhaseComp" variables that are used in power computation
                    after phase compensation
  
  Remarks: None
  ***************************************************************************/
#define  PhaseCompensate_VI()			_asm									\
										movf	VoltageSample_Delayed,0,1		\
										subwf	VoltageSample,0,1				\
										movwf	VoltageSample_PhaseComp,1		\
										movf	VoltageSample_Delayed+1,0,1		\
										subwfb	VoltageSample+1,0,1				\
										movwf	LPI_temp1,1						\
																				\
										movf	PhaseComp_Amplitude,0,1			\
																				\
										mulwf	VoltageSample_PhaseComp,1		\
										movff	PRODH,VoltageSample_PhaseComp	\
										mulwf	LPI_temp1,1						\
										movf	PRODL,0,0						\
										addwf	VoltageSample_PhaseComp,1,1		\
										movff	PRODH,VoltageSample_PhaseComp+1	\
										btfsc	STATUS,0,0						\
										incf	VoltageSample_PhaseComp+1,1,1	\
										btfss	LPI_temp1,7,1					\
										bra		LPI_PhA_Diff_Done				\
										movf	PhaseComp_Amplitude,0,1			\
										subwf	VoltageSample_PhaseComp+1,1,1	\
									LPI_PhA_Diff_Done:							\
										movf	VoltageSample_Delayed,0,1		\
										addwf	VoltageSample_PhaseComp,1,1		\
										movf	VoltageSample_Delayed+1,0,1		\
										addwfc	VoltageSample_PhaseComp+1,1,1	\
																				\
										btfsc	Flags1,2,1						\
										bra		LPI_PhA_PC_Neg					\
										movff	CurrentSample_Delayed,CurrentSample_PhaseComp	\
										movff	CurrentSample_Delayed+1,CurrentSample_PhaseComp+1	\
										bra		LPI_PhA_PC_Done					\
									LPI_PhA_PC_Neg:								\
										movff	CurrentSample,CurrentSample_PhaseComp	\
										movff	CurrentSample+1,CurrentSample_PhaseComp+1	\
									LPI_PhA_PC_Done:							\
									_endasm



#if defined(ACTIVE_POWER_ENABLE_COMPUTATION) ||\
	defined(ACTIVE_ENERGY_ENABLE_COMPUTATION) ||\
	defined(REACTIVE_POWER_ENABLE_COMPUTATION) ||\
	defined(REACTIVE_ENERGY_ENABLE_COMPUTATION)
/****************************************************************************
  Macro:   SummateInstanteneousPower(Volt,Curr,Power,type)

 Description:   Macro implemets instanteneous power calculation and summates over 128 samples. 
                         This summation is later used in energy & power computation

						   
  Precondition:  Variables used in this macro must be in BANK 1 
                           Voltage & Current phase correction must have need done before this is executed.
                           In case of reactive power computation, Voltage samples must be processed with phase shifter
                           Requires temporary variables LPI_Res0 to LPI_Res3 to be defined as UINT8 in BANK1
                           
  Parameters:   Volt - Voltage Sample from SD-ADC
                         Curr - Current Sample from SD-ADC
                         Power - Power being calculated using input Voltage & Current samples & accumulate ins power over 128 samples
                         type - Type of the power being calculated
  
  Returns:   Updates "VoltageSample_PhaseComp" & "CurrentSample_PhaseComp" variables that are used in power computation
                    after phase compensation
  
  Remarks: Macro implements the below math functionality
//****************************** SIGNED MULTIPLICATION ALGORITHM *******************************
//**************** (LPI_Res = VoltageSample_PhaseComp * CurrentSample_PhaseComp) Phase power calculation *********************************
//LPI_Res	=			[VoltageSample_PhaseComp+1] [VoltageSample_PhaseComp]		*	[CurrentSample_PhaseComp+1] [CurrentSample_PhaseComp]	
//----------------------------------------------------------------------------------------------------------------------------------------------------
//										[CurrentSample_PhaseComp+1] [CurrentSample_PhaseComp]		[VoltageSample_PhaseComp] [CurrentSample_PhaseComp]
//										------------ + ----------------			
//					[VoltageSample_PhaseComp+1] [ CurrentSample_PhaseComp+1]	[VoltageSample_PhaseComp] [CurrentSample_PhaseComp+1]
//	              Carry_of_Add      
//----------------------------------------------------------------------------------------------------------------------------------------------------
//LPI_Res =    LPI_Res3		LPI_Res2				LPI_Res1				LPI_Res0
//----------------------------------------------------------------------------------------------------------------------------------------------------
//		    - [CurrentSample_PhaseComp+1]		- [CurrentSample_PhaseComp]						NOTE:*"If and only if CurrentSample_PhaseComp is negetive "*
//		    - [VoltageSample_PhaseComp+1]		- [VoltageSample_PhaseComp]					NOTE:*"If and only if VoltageSample_PhaseComp is negetive"*
//----------------------------------------------------------------------------------------------------------------------------------------------------
//LPI_Res =    LPI_Res3		LPI_Res2				LPI_Res1				LPI_Res0
//----------------------------------------------------------------------------------------------------------------------------------------------------		
  ***************************************************************************/
#define  SummateInstanteneousPower(Volt,Curr,Power,type)								\
																_asm					\
																movf	Volt,0,1		\
																mulwf	Curr,1			\
																movff	PRODH,LPI_Res1	\
																movff	PRODL,LPI_Res0	\
																movf	Volt+1,0,1		\
																mulwf	Curr+1,1		\
																movff	PRODH,LPI_Res3	\
																movff	PRODL,LPI_Res2	\
																movf	Volt,0,1		\
																mulwf	Curr+1,1		\
																movf	PRODL,0,0		\
																addwf	LPI_Res1,1,1	\
																movf	PRODH,0,0		\
																addwfc	LPI_Res2,1,1	\
																clrf	WREG,0			\
																addwfc	LPI_Res3,1,1	\
																movf	Volt+1,0,1		\
																mulwf	Curr,1			\
																movf	PRODL,0,0		\
																addwf	LPI_Res1,1,1	\
																movf	PRODH,0,0		\
																addwfc	LPI_Res2,1,1	\
																clrf	WREG,0			\
																addwfc	LPI_Res3,1,1	\
																btfss	Curr+1,7,1		\
																bra		LPI_PhA_P_ChkNegV_##type	\
																movf	Volt,0,1		\
																subwf	LPI_Res2,1,1	\
																movf	Volt+1,0,1		\
																subwfb	LPI_Res3,1,1	\
															LPI_PhA_P_ChkNegV_##type:	\
																btfss	Volt+1,7,1		\
																bra		LPI_PhA_P_SqrDone_##type	\
																movf	Curr,0,1		\
																subwf	LPI_Res2,1,1	\
																movf	Curr+1,0,1		\
																subwfb	LPI_Res3,1,1	\
																						\
															LPI_PhA_P_SqrDone_##type:	\
																movf	LPI_Res0,0,1	\
																addwf	Power,1,1		\
																movf	LPI_Res1,0,1	\
																addwfc	Power+1,1,1		\
																movf	LPI_Res2,0,1	\
																addwfc	Power+2,1,1		\
																movf	LPI_Res3,0,1	\
																addwfc	Power+3,1,1		\
																clrf	WREG,0			\
																btfsc	LPI_Res3,7,1	\
																comf	WREG,1,0		\
																addwfc	Power+4,1,1		\
																_endasm
#else
#define SummateInstanteneousPower(Volt,Curr,Power,type)  
#endif
  
  
#if defined(RMS_CURRENT_ENABLE_COMPUTATION) ||\
	defined(RMS_VOLTAGE_ENABLE_COMPUTATION)
/****************************************************************************
  Macro:   SummateInstanteneousSquareRMS(Signal,Accumulator,type)

 Description:   Macro implemets instanteneous squaring of signal samples and accumulates over cycle to
                        compute RMS value of Voltage & Current

						   
  Precondition:  Variables used in this macro must be in BANK 1 
                           
  Parameters:   Signal - Signal sample from SD-ADC whose RMS value ned to be computed
                         Accumulator - Summation of Square value accumulator, which inturn is used in RMS calculation
                         type - Type of the RMS signal being calculated
  
  Returns:   Updates "Acc_RMS_Square_Voltage_ISR" & "Acc_RMS_Square_Current_ISR" variables that are used in RMS computation
  
  Remarks: Macro implements the below math functionality
//****************************** UNSIGNED MULTIPLICATION ALGORITHM *******************************
//**************** [Accumulator = Accumulator + (Signal * Signal) ];  Square current accumulation**************************
//Accumulator	=			[Signal+1] [Signal]		*	[Signal+1] [Signal]	
//-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
//													[Signal+1] [Signal]		[Signal] [Signal]
//													------------ + ----------------			
//								[Signal+1] [ Signal+1]	[Signal] [Signal+1]
//	                     Carry_of_Add    Carry_of_Add      
//-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
//Accumulator =  Accumulator+4  Accumulator+3		Accumulator+2		Accumulator+1				Accumulator
//-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
  ***************************************************************************/
#define  SummateInstanteneousSquareRMS(Signal,Accumulator,type)			_asm							\
																	btfss	Signal+1,7,1			\
																	bra		LPI_PhA_I_Pos_##type	\
																	comf	Signal+1,1,1			\
																	negf	Signal,1				\
																	btfsc	STATUS,0,0				\
																	incf	Signal+1,1,1			\
																									\
																LPI_PhA_I_Pos_##type:				\
																	movf	Signal,0,1				\
																	mulwf	Signal,1				\
																	movf	PRODL,0,0				\
																	addwf	Accumulator,1,1			\
																	movf	PRODH,0,0				\
																	addwfc	Accumulator+1,1,1		\
																	clrf	WREG,0					\
																	addwfc	Accumulator+2,1,1		\
																	addwfc	Accumulator+3,1,1		\
																	addwfc	Accumulator+4,1,1		\
																	movf	Signal+1,0,1			\
																	mulwf	Signal+1,1				\
																	movf	PRODL,0,0				\
																	addwf	Accumulator+2,1,1		\
																	movf	PRODH,0,0				\
																	addwfc	Accumulator+3,1,1		\
																	clrf	WREG,0					\
																	addwfc	Accumulator+4,1,1		\
																	rlncf	Signal+1,0,1			\
																	mulwf	Signal,1				\
																	movf	PRODL,0,0				\
																	addwf	Accumulator+1,1,1		\
																	movf	PRODH,0,0				\
																	addwfc	Accumulator+2,1,1		\
																	clrf	WREG,0					\
																	addwfc	Accumulator+3,1,1		\
																	addwfc	Accumulator+4,1,1		\
																	_endasm
#else
#define SummateInstanteneousSquareRMS(Signal,Accumulator,type)  
#endif


/****************************************************************************
  Macro:   CheckSDADCConfig(SDADC_Config,Config_Status)

 Description:   Macro periodically checks the SD-ADC Configuration status, to check for any corruption
                        in configuration and hence operation. This macro will be executed to check config status every
                        128 Signal cycle time.

  Precondition:  Variables used in this macro must be in BANK 1 
                           "MCP_CONFIG_CHK" register type must be defined in BANK1. This contains flags for config status indication
                           "SDMCP_CON" register type contains the registers containing the SD-ADC configuration values for cross reference in BANK1
                           "spi_mcp_chk" 8 bit temporary variable must be defined in BANK1
                           
  Parameters:   SDADC_Config - register of type "SDMCP_CON" containing SD-ADC config values
                         Config_Status - register of type "MCP_CONFIG_CHK" containing flags for config status indication
  
  Returns:   Updates "MCP_CONFIG_CHK" type register
  
  Remarks: None
  ***************************************************************************/
#define  CheckSDADCConfig(SDADC_Config,Config_Status)		_asm						\
															btfss	Config_Status,0,1	\
															bra		end_mcp_chk			\
															bcf		EM_ASM_SDADC_CS,0	\
															bcf		Config_Status,0,1	\
															movlw	0x15				\
															movwf	SSPBUF,0			\
															nop							\
															nop							\
															nop							\
															nop							\
															nop							\
															nop							\
															nop							\
															nop							\
															nop							\
															nop							\
															movlw	0x00				\
															movwf	SSPBUF,0			\
															nop							\
															nop							\
															nop							\
															nop							\
															nop							\
															nop							\
															nop							\
															nop							\
															movf	SDADC_Config,0,1		\
															nop							\
															movff	SSPBUF,spi_mcp_chk	\
															xorwf	spi_mcp_chk,0,1		\
															bnz		mcp_con_error		\
															movlw	0x00				\
															movwf	SSPBUF,0			\
															nop							\
															nop							\
															nop							\
															nop							\
															nop							\
															nop							\
															nop							\
															nop							\
															movf	SDADC_Config+1,0,1	\
															nop							\
															movff	SSPBUF,spi_mcp_chk	\
															xorwf	spi_mcp_chk,0,1		\
															bnz		mcp_con_error		\
															bsf		EM_ASM_SDADC_CS,0	\
															movlw	0x11				\
															nop							\
															nop							\
															bcf		EM_ASM_SDADC_CS,0	\
															movwf	SSPBUF,0			\
															nop							\
															nop							\
															nop							\
															nop							\
															nop							\
															nop							\
															nop							\
															nop							\
															nop							\
															nop							\
															movlw	0x00				\
															movwf	SSPBUF,0			\
															nop							\
															nop							\
															nop							\
															nop							\
															nop							\
															nop							\
															nop							\
															nop							\
															movf	SDADC_Config+2,0,1	\
															nop							\
															movff	SSPBUF,spi_mcp_chk	\
															xorwf	spi_mcp_chk,0,1		\
															bnz		mcp_con_error		\
															movlw	0x00				\
															movwf	SSPBUF,0			\
															nop							\
															nop							\
															nop							\
															nop							\
															nop							\
															nop							\
															nop							\
															nop							\
															movlw	0xFC				\
															andwf	SDADC_Config+3,0,1	\
															nop							\
															movff	SSPBUF,spi_mcp_chk	\
															xorwf	spi_mcp_chk,0,1		\
															bnz		mcp_con_error		\
															bra		end_mcp_chk			\
													mcp_con_error:						\
															bsf		Config_Status,2,1	\
													end_mcp_chk:						\
															bsf		EM_ASM_SDADC_CS,0	\
															_endasm


/****************************************************************************
  Macro:   ReadSignalPeriod(period)

 Description:   Macro reads the period of 128 samples of signal using TIMER3

  Precondition:  Variables used in this macro must be in BANK 1 
                           TIMER3 must be initialized with 1:8 prescaler
                           
  Parameters:   period - unsigned integer period for 128 samples of signal
  
  Returns:   Updates period register
  
  Remarks: None
  ***************************************************************************/
#define  ReadSignalPeriod(period)					_asm							\
														movff	TMR3L,period		\
														movff	TMR3H,period+1	\
														clrf	TMR3H,0				\
														clrf	TMR3L,0				\
													_endasm	
 

/****************************************************************************
  Macro:   SetSignalCycleComplete()

 Description:   Macro sets flag to indicate the completion of signal cycle with 128 sample accumulation

  Precondition:  Flags1 register must be defined
                           
  Parameters:   None
  
  Returns:   Updates Flags1.CycleComplete bit in register
  
  Remarks: None
  ***************************************************************************/
 #define  SetSignalCycleComplete()			Flags1.bits.CycleComplete = 1
  
#endif	//__sdadc_EnergyMeter

