/******************************************************************************

File Name:           signalProcess_EnergyMeter.h
Dependencies:    Microchip Peripheral Library
Processor:           PIC18F65J90
Company:             Microchip Technology, Inc.

Copyright 2010 Microchip Technology Inc.  All rights reserved.

Microchip licenses to you the right to use, modify, copy and distribute
Software only when embedded on a Microchip microcontroller or digital signal
controller that is integrated into your product or third party product
(pursuant to the sublicense terms in the accompanying license agreement).

You should refer to the license agreement accompanying this Software for
additional information regarding your rights and obligations.

SOFTWARE AND DOCUMENTATION ARE PROVIDED ?AS IS? WITHOUT WARRANTY OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE.
IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER
CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR
OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR
CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF
SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
(INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.

Author                                     Date                                                Revision History / Comments
------------------------------------------------------------------------------------------------------------------------
Harsha Jagadish              1 - Jan - 2011                                                    Initial Version v1.0
*******************************************************************************/
#ifndef __signalProcess_EnergyMeter
#define __signalProcess_EnergyMeter

#include "HardwareProfile_EnergyMeter.h"
#include "Config_EnergyMeter.h"

/****************************************************************************
  Macro:   ProcessSignalForOffsetRemoval()

 Description:   Macro implements a HPF Filter for offset removal in input signal,

  Precondition:   High Pass Filter must be designed with the signal frequency range of interest from 45Hz - 66Hz
                            Variables "HPF_Intermediate_Result_1" & "HPF_Intermediate_Result_1" must be defined as INT32.
                             and "HPF_Voltage_Output_Delayed" & "HPF_Current_Output_Delayed" must be defined as INT32.
                             "HPF_Voltage_Input_Delayed" and "HPF_Current_Input_Delayed" must be defined as INT16

  Parameters:   Input_Sample - Input Signal (16 bit signed) to be processed through HPF
		   HPF_Input_Delayed - Input Signal (16 bit signed) delayed by one sample time to be processed through HPF
		   HPF_Output_Delayed - HPF output (32 bit signed) delayed by one sample time
  
  Returns:   input signal is processed with offset removal and loaded back to same signal variable

  Remarks: 
//***************** The HPF for all the two channels of Current & Voltage ***************************************************************
//	* HPF of First orer is added with cutoff 4.14Hz to filter out DC component in channel input so as 
//          to enable measurement of only AC component
//	* HPF Filter equation: 	Y[n] = (1-(1/A0)) ( Y[n-1] + X[n] - X[n-1] ) - (Y[n-1]/A0) 		
//           where Y[n] is present output & Y[n-1] is previous output and
//	    X[n] is present input & X[n-1] is previous input
//	* Filter Coefficient A0 = e ^ (-2 x pi x fc / F) = 2^12			where fc-> cutoff frequency & F-> sampling rate 3.609ksps
//*******************************************************************************************************************************
  ***************************************************************************/
#define ProcessSignalForOffsetRemoval(Input_Sample,HPF_Input_Delayed,HPF_Output_Delayed)						\
											SaveBankSelectRegister(SaveBSRStatus);								\
											LoadBSR(SDADC_ISR_VARIABLE_BANK);									\
											_asm																\
											movff	Input_Sample,HPF_Intermediate_Result_1						\
											movff	Input_Sample+1,HPF_Intermediate_Result_1+1					\
											movlw	0x00														\
											btfsc	Input_Sample+1,7,1											\
											movlw	0xFF														\
											movwf	HPF_Intermediate_Result_1+2,1								\
											movwf	HPF_Intermediate_Result_1+3,1								\
											movf	HPF_Input_Delayed,0,1										\
											subwf	HPF_Intermediate_Result_1,1,1								\
											movf	HPF_Input_Delayed+1,0,1										\
											subwfb	HPF_Intermediate_Result_1+1,1,1								\
											movlw	0x00														\
											btfsc	HPF_Input_Delayed+1,7,1											\
											movlw	0xFF														\
											subwfb	HPF_Intermediate_Result_1+2,1,1								\
											movlw	0x00														\
											btfsc	HPF_Input_Delayed+1,7,1											\
											movlw	0xFF														\
											subwfb	HPF_Intermediate_Result_1+3,1,1								\
											movff	HPF_Intermediate_Result_1+2,HPF_Intermediate_Result_1+3		\
											movff	HPF_Intermediate_Result_1+1,HPF_Intermediate_Result_1+2		\
											movff	HPF_Intermediate_Result_1,HPF_Intermediate_Result_1+1		\
											movlw	0															\
											movwf	HPF_Intermediate_Result_1,1									\
											_endasm																\
											RestoreBankSelectRegister(SaveBSRStatus);						\
											HPF_Intermediate_Result_1 = HPF_Intermediate_Result_1 + HPF_Output_Delayed;	\
											HPF_Intermediate_Result_2 = HPF_Intermediate_Result_1 + HPF_Output_Delayed;	\
											SaveBankSelectRegister(SaveBSRStatus);							\
											LoadBSR(SDADC_ISR_VARIABLE_BANK);									\
											_asm																\
											bcf		STATUS,0,0													\
											btfsc	HPF_Intermediate_Result_2+3,7,1								\
											bsf		STATUS,0,0													\
											rrcf	HPF_Intermediate_Result_2+3,1,1								\
											rrcf	HPF_Intermediate_Result_2+2,1,1								\
											rrcf	HPF_Intermediate_Result_2+1,1,1								\
											rrcf	HPF_Intermediate_Result_2,1,1								\
											movlw	0x00														\
											btfsc	HPF_Intermediate_Result_2+3,7,1								\
											movlw	0xFF														\
											movff	HPF_Intermediate_Result_2+1,HPF_Intermediate_Result_2		\
											movff	HPF_Intermediate_Result_2+2,HPF_Intermediate_Result_2+1		\
											movff	HPF_Intermediate_Result_2+3,HPF_Intermediate_Result_2+2		\
											movwf	HPF_Intermediate_Result_2+3,1								\
											_endasm																\
											RestoreBankSelectRegister(SaveBSRStatus);						\
											HPF_Output_Delayed = HPF_Intermediate_Result_1 - HPF_Intermediate_Result_2;	\
											HPF_Intermediate_Result_2=HPF_Output_Delayed;						\
											HPF_Input_Delayed = Input_Sample;									\
											SaveBankSelectRegister(SaveBSRStatus);							\
											LoadBSR(SDADC_ISR_VARIABLE_BANK);									\
											_asm																\
											movlw	0x00														\
											btfsc	HPF_Intermediate_Result_2+3,7,1								\
											movlw	0xFF														\
											movff	HPF_Intermediate_Result_2+1,HPF_Intermediate_Result_2		\
											movff	HPF_Intermediate_Result_2+2,HPF_Intermediate_Result_2+1		\
											movff	HPF_Intermediate_Result_2+3,HPF_Intermediate_Result_2+2		\
											movwf	HPF_Intermediate_Result_2+3,1								\
											_endasm																\
											RestoreBankSelectRegister(SaveBSRStatus);						\
											Input_Sample=(INT16)HPF_Intermediate_Result_2



#if defined(REACTIVE_POWER_ENABLE_COMPUTATION) ||\
	defined(REACTIVE_ENERGY_ENABLE_COMPUTATION)
/****************************************************************************
  Macro:   ProcessVoltageFor90PhaseShift()

 Description:   Macro Phase shifts the input signal by a angle of 90 degree lag. The macro implements LPF operated in cutoff band
                         shifts the input signal by 90 degree with attenuation. This signal is amplified to restore the signal level and get 90
                         degree phase shift. LPF implemented has cutoff frequency of ~4Hz. 
		    Followed by LPF processing, the signal is processed for phase compensation by small value, to get exact 90 degree lag

  Precondition:   The macro operates on "VoltageSample" to get phase shifted output, which is used in reactive component calculation
                            Low Pass Filter must be designed with the signal frequency range of interest from 45Hz - 66Hz to operate in cutoff band
                            Variables "LPFAcc"  must be defined as INT32.
                             and "LPFYD"  must be defined as INT32.
                             "LPF_X_D1"must be defined as INT16
                             "PhaseComp_Amplitude_90Lag" 8 bit phase compensation magnitude must be defined

  Parameters:   None
  
  Returns:   input signal is processed for phase shift is loaded to "VoltageSample_90Lag"

  Remarks: 
//***** LOW PASS FILTER IMPLEMENTATION FOR VOLTAGE SAMPLES PHASE SHIFTING BY 90 DEGREE ***************
//	*	To use LPF as phase shifter to obtain 90 degree lag, configure LPF for its opeartion in Stop Band
//	*	LPF Filter Equation:
//	*	Filter Coefficients for first oredr LPF that is configured with a cut-off frequency of ~4.14 Hz
//	*	Floating point numbers are converted to Q15 format	
//	*	F - Sampling frequency = 3906ksps
//	*	Filter equation: Y[n] = Y[n-1] + (1/2^11)*(X[n]+X[n-1]-2*Y[n-1])
//	*	The output of the LPF will be attenuated & 90 degree phase shifted output for voltage samples
//	*	Amplify the output samples by multiplying it with definite Gain
//	*	Introduce small Phase corrections to LPF output to get the exact 90 degree phase shift output
//	*	Little phase correction would be necessary due to practicality of LPF in its stop band opeartion and the order being first order
//****************************************************************************************************************
  ***************************************************************************/
#define  ProcessVoltageFor90PhaseShift()		_asm										\
											movlw	0									\
											movwf	LPFAcc,1							\
											movwf	LPFAcc+1,1							\
											movff	VoltageSample_PhaseComp,LPFAcc+2	\
											movff	VoltageSample_PhaseComp+1,LPFAcc+3	\
											movlw	0									\
											btfsc	VoltageSample_PhaseComp+1,7,1		\
											movlw	0xFF								\
											movwf	LPFAcc+4,1							\
											movf	LPFYD,0,1							\
											subwf	LPFAcc,1,1							\
											movf	LPFYD+1,0,1							\
											subwfb	LPFAcc+1,1,1						\
											movf	LPFYD+2,0,1							\
											subwfb	LPFAcc+2,1,1						\
											movf	LPFYD+3,0,1							\
											subwfb	LPFAcc+3,1,1						\
											movlw	0									\
											btfsc	LPFYD+3,7,1							\
											movlw	0xFF								\
											subwfb	LPFAcc+4,1,1						\
																						\
											movf	LPF_X_D1,0,1						\
											addwf	LPFAcc+2,1,1						\
											movf	LPF_X_D1+1,0,1						\
											addwfc	LPFAcc+3,1,1						\
											movlw	0									\
											btfsc	LPF_X_D1+1,7,1						\
											movlw	0xFF								\
											addwfc	LPFAcc+4,1,1						\
																						\
											movf	LPFYD,0,1							\
											subwf	LPFAcc,1,1							\
											movf	LPFYD+1,0,1							\
											subwfb	LPFAcc+1,1,1						\
											movf	LPFYD+2,0,1							\
											subwfb	LPFAcc+2,1,1						\
											movf	LPFYD+3,0,1							\
											subwfb	LPFAcc+3,1,1						\
											movlw	0									\
																						\
											btfsc	LPFYD+3,7,1							\
											movlw	0xFF								\
											subwfb	LPFAcc+4,1,1						\
																						\
											movlw	0x0B								\
											movwf	Counter,1							\
										LPF_Shift_Right_N:								\
											bcf		STATUS,0,0							\
											btfsc	LPFAcc+4,7,1						\
																						\
											bsf		STATUS,0,0							\
											rrcf 	LPFAcc+4,1,1						\
											rrcf 	LPFAcc+3,1,1						\
											rrcf 	LPFAcc+2,1,1						\
											rrcf 	LPFAcc+1,1,1						\
											rrcf 	LPFAcc,1,1							\
											decf	Counter,1,1							\
											bnz		LPF_Shift_Right_N					\
																						\
											movf	LPFYD,0,1							\
											addwf	LPFAcc,1,1							\
											movf	LPFYD+1,0,1							\
											addwfc	LPFAcc+1,1,1						\
											movf	LPFYD+2,0,1							\
											addwfc	LPFAcc+2,1,1						\
											movf	LPFYD+3,0,1							\
											addwfc	LPFAcc+3,1,1						\
																						\
											movff	LPFAcc,LPFYD						\
											movff	LPFAcc+1,LPFYD+1					\
											movff	LPFAcc+2,LPFYD+2					\
											movff	LPFAcc+3,LPFYD+3					\
																						\
											movff	VoltageSample_PhaseComp,LPF_X_D1	\
											movff	VoltageSample_PhaseComp+1,LPF_X_D1+1	\
																						\
											movlw	0x07								\
											movwf	Counter,1							\
										LPF_Shift_Leftt_N:								\
											bcf		STATUS,0,0							\
											rlcf LPFAcc,1,1								\
											rlcf LPFAcc+1,1,1							\
											rlcf LPFAcc+2,1,1							\
											rlcf LPFAcc+3,1,1							\
											decf	Counter,1,1							\
											bnz		LPF_Shift_Leftt_N					\
																						\
											movff	LPFAcc+2,VoltageSample_90Lag		\
											movff	LPFAcc+3,VoltageSample_90Lag+1		\
																						\
											movf	Phase_V_90_D1,0,1						\
											subwf	VoltageSample_90Lag,0,1				\
											movwf	Phase_V90,1							\
											movf	Phase_V_90_D1+1,0,1					\
											subwfb	VoltageSample_90Lag+1,0,1			\
											movwf	LPI_temp1,1							\
																						\
											movf	PhaseComp_Amplitude_90Lag,0,1		\
																						\
											mulwf	Phase_V90,1							\
											movff	PRODH,Phase_V90						\
											mulwf	LPI_temp1,1							\
											movf	PRODL,0,0							\
											addwf	Phase_V90,1,1							\
											movff	PRODH,Phase_V90+1						\
											btfsc	STATUS,0,0							\
											incf	Phase_V90+1,1,1						\
											btfss	LPI_temp1,7,1						\
											bra		LPI_PhA_Diff_Done_r					\
																						\
											movf	PhaseComp_Amplitude_90Lag,0,1		\
											subwf	Phase_V90+1,1,1						\
										LPI_PhA_Diff_Done_r:							\
											movf	Phase_V_90_D1,0,1						\
											addwf	Phase_V90,1,1							\
											movf	Phase_V_90_D1+1,0,1					\
											addwfc	Phase_V90+1,1,1						\
																						\
											movff	VoltageSample_90Lag,Phase_V_90_D1		\
											movff	VoltageSample_90Lag+1,Phase_V_90_D1+1 \
											movff	Phase_V90,VoltageSample_90Lag    		\
											movff	Phase_V90+1,VoltageSample_90Lag+1 	\
											_endasm
#else
#define ProcessVoltageFor90PhaseShift()  
#endif

#endif	//__signalProcess_EnergyMeter

