/******************************************************************************

File Name:           sysinit_EnergyMeter.c
Dependencies:    Microchip Peripheral Library
Processor:           PIC18F65J90
Company:             Microchip Technology, Inc.

Copyright 2010 Microchip Technology Inc.  All rights reserved.

Microchip licenses to you the right to use, modify, copy and distribute
Software only when embedded on a Microchip microcontroller or digital signal
controller that is integrated into your product or third party product
(pursuant to the sublicense terms in the accompanying license agreement).

You should refer to the license agreement accompanying this Software for
additional information regarding your rights and obligations.

SOFTWARE AND DOCUMENTATION ARE PROVIDED ?AS IS? WITHOUT WARRANTY OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE.
IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER
CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR
OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR
CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF
SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
(INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.

Author                                     Date                                                Revision History / Comments
------------------------------------------------------------------------------------------------------------------------
Harsha Jagadish              1 - Jan - 2011                                                    Initial Version v1.0
*******************************************************************************/
#include "HardwareProfile_EnergyMeter.h"
#include "RegDef_EnergyMeter.h"
#include "eeprom_EnergyMeter.h"


/************************************************************************************
 Function    : void Init_EnergyMeter(void)

 Include     : HardwareProfile_EnergyMeter.h
 
 Description : The function initializes all the peripherals that are used in the energymeter 
		   application. The function also calls function for the initialization of "Sigma Delta ADC".
		   It also initializes the calibration varaiables to thier calibrated values or 
		   to their default values depending upon previous calibration history. The peripherals
		   that are initialized include:
			*	Timers3 for period calculation
			*	Timer0 for EEPROM energy storage timing, No Load timer calculation and other SW clock
			*	PWM as clock source for "Sigma Delta ADC"
			*	"Sigma Delta ADC" external sigma  - delta ADC
			*	SPI for communication with "Sigma Delta ADC" & EEPROM
			*	LCD for dsiaply of energy
			*	USART for communication with GUI
			*	Interrupts for periodic ADC data read
			*	Ports initialization as input & output ports
			*	LVD for power down detect
	
 Arguments   : None
 
 Remarks     : 	This function is called from "main" function at the begining for hardware & variable
				initailization
				Code uses Peripheral Library support of PIC18 available with MCC18
**************************************************************************************/
void Init_EnergyMeter(void)
{

	EnableSystemPLL();			//Enable PLL to operate processor at 4 * FOSC Frequency
	
	ClearWDT();
	
	SaveEnergyStatus = SAVE_ENERGY_TO_EEPROM_DISABLED;
	
//********************** IO PORTS Configuration ********************************************************
	//Set all analog multiplexed ports as Digital
	ConfigPortsForDigitalMode();
	
	//"Sigma Delta ADC" Chip select port in configured as output port & disabled
	SDADCChipSelectConfigure();
	
	//SD-ADC reset line is set to output & disabled
	SDADCResetPinConfigure();
	
	//EEPROM Chip select line is set to disable chip select of EEPROM
	EEPROMChipSelectConfigure();
	
	//configure Active CF Pulse pins to output & clear CF Pulse signal
	ACTIVECFPulseConfigure();

	//configure Reactive CF Pulse pins to output & clear CF Pulse signal
	REACTIVECFPulseConfigure();
		
	//Initialize all unused port pins to output and drive low
	//This would avoid any noise interference and reduce the power consumption
	UnusedPortsConfigure();
//******************************************************************************************************
	//Disable all Golabl Interrupts
	DisableAllGlobalInterrupts();

	//Function to initialize LCD and clear all data registers
	Init_LCD();
	
//************************* Calibration registers & Energy accumulator initialization ****************************	
	//Function to Set Calibration registers to the default value
	ConfigCalibrationRegsToDefault();
	

	EEPROM_Load_Error = FALSE;
	/**********NOTE: Below function will efectively load calibration parameters from EEPROM only if the device is previously calibrated
	else the calibration registers will retain the default value that were loaded from previously called function ************************/
	EEPROM_Load_Error = LoadFromEEPROM(ADDRESS_OF_BYTES_CAL_REGS_STORED_IN_EEPROM,
	                                   (UINT8*)&PHASE_COMPENSATION,
	                                   NUMBER_OF_BYTES_CAL_REGS_STORED_IN_EEPROM);		//Load Calibration values from EEPROM to RAM
	EEPROM_Load_Error |= LoadFromEEPROM(ADDRESS_OF_BYTES_ENERGY_STORED_IN_EEPROM,
	                                   (UINT8*)&ENERGY_ACT,
	                                   NUMBER_OF_BYTES_ENERGY_STORED_IN_EEPROM);		//Load Calibration values from EEPROM to RAM
	if(EEPROM_Load_Error == TRUE)
	{
		ConfigCalibrationRegsToDefault();
	}	
	else
	{
		SaveEnergyStatus = SAVE_ENERGY_TO_EEPROM_ENABLED;
	}

    
	//Function to configure Operational registers
	ConfigOpeartionalRegs();    
//********************************************************************************************************
	/* Make CCP1 an output
               Brief pulse clears the PLL lock detect when no signal is present
               Again configure back the CCP1 pin to output state
           */
	CCPPWMPinConfig();
	
//************** TIMER0 Configuration for timer 0 to emulatet the RTCC************************************
//		* TIMER0 Interrupt OFF
//		* TIMER0 in 16 bit  mode
//		* 1:256 prescaler
//		* Interrupt enabled
//********************************************************************************************************
	OpenTimer0(TIMER_INT_ON| T0_16BIT | T0_PS_1_256 | T0_SOURCE_INT );
	ConfigSWTimerInterruptToLowPriority();
	//Timer 0 interrupt cleared to avoid any previous stray interrupt 
	ClearInterruptStatusSWTimer();
	//Initialize the Timer0 counter to get 1.0s time interrupt
	WriteTimer0(SET_TIMER0_COUNTER_1SEC);
	
//************** TIMER3 Configuration for period between two succcesive sample reading to calculate ***************************	
//		* TIMER3 Interrupt On
//		* TIMER3 in 16 bit mode
//		* 1:8 prescaler
//**************************************************************
	OpenTimer3(TIMER_INT_OFF | T3_16BIT_RW | T3_PS_1_8 );
	WriteTimer3(RESET_TIMER_COUNT);			//Load Timer3 with 0x0000
//************** TIMER1 Configuration for Frequency calculation ***************************	
//		* TIMER1 Interrupt Off
//		* TIMER1 in 16 read-write bit mode
//		* 1:4 prescaler
//NOTE: Set the macro "FREQUENCY_TIMER_PRESCALE" prescale value as per configuration
//**************************************************************
	OpenTimer1(TIMER_INT_OFF | T1_16BIT_RW | T1_PS_1_4 );
	WriteTimer1(RESET_TIMER_COUNT);			//Load Timer1 with 0x0000

//*************** CCP1 Configuration *****************************
// 		* Configure CCP1 in PWM mode to generate clock with 50% duty cycle with a frequency of
//		   1.96MHz for 60Hz input
//**************************************************************
	OpenPWM1(PWM_COUNT);
	SetDCPWM1(PWM_DUTY_CYCLE);

//************************ USART Initialization for Communication with Calibration GUI *************************
	Init_USART();					//Function to Initialize USART
	ClearWDT();
//************ Interrupt 0 Configuration for DR pulse reception from "Sigma Delta ADC" for data ready interrupt **************
//		* Interrupt on Falling edge
//		* Interrupt priority high
//**************************************************************
	OpenRB0INT( FALLING_EDGE_INT | PORTB_INT_PRIO_HIGH | PORTB_CHANGE_INT_OFF );
	EnableInterruptPriorityMode();				// Enable high/low priority for interrupts	
	//Disable INT0 momentarily till complete initialization of all peripherals and "Sigma Delta ADC"
	ClearInterruptStatusSDADC();
	DisableInterruptSDADC();
//************************* "Sigma Delta ADC" Initialization **********************************************************
	while(Init_MCP_SD_ADC(NOMINAL_CURR_CHANNEL_GAIN));				// Call function to initialize the "Sigma Delta ADC"

//********************************************************************************************************
// Make LVD interrupt priority high and enable the interrupt.
	PowerDownDetectConfig();

	ClearWDT();
	
	LCD_Update();		//Update LCD
//******************** Enable the interrupts ******************************************************************
	EnableAllGlobalInterrupts();

}


/************************************************************************************
 Function    : void ConfigCalibrationRegsToDefault(void)

 Description : The function initializes all "CALIBRATION REGISTERS" to default values
	
 Arguments   : None
 
 Remarks     : This function is called from "main" function at the begining.
**************************************************************************************/
void ConfigCalibrationRegsToDefault(void)
{

	MODE1_DEF = 0x000A;
	
//***************** Calibration registers Initialization to default values **************************
	GAIN_I_RMS = 0x62A8;
	GAIN_V_RMS = 0x14A0;
	GAIN_POWER_ACT = 0x4F5B;
	GAIN_POWER_REACT = 0x7A7D;
	
	GAIN_ENERGY_ACT	= 0x7FFF;
	GAIN_ENERGY_APP	= 0x7FFF;
	GAIN_ENERGY_REACT	= 0x7FFF;

	OFFSET_POWER_ACT = 0;
	OFFSET_POWER_REACT = 0;


	GAIN_NUMR_ENERGY_ACT = 0xC814;	
	GAIN_DENR_ENERGY_ACT = 0x00;
	
	GAIN_NUMR_ENERGY_REACT = 0x9A64;
	GAIN_DENR_ENERGY_REACT = 0x01;
//********************************************************************************************

	METER_CONSTANT = 3200;	
	PHASE_COMPENSATION = 0x02;					//After conversion to 4MHz Operation, this is the ideal phase delay
//********************************************************************************************
	LINE_CYC		= 0x07;
	CF_PULSE_WIDTH = CF_PULSE_WIDTH_VAL;			// CF pulse width is approximately 90 ms (81ms at 60Hz, 98 ms at 50 Hz)
	
	CREEP_THRESHOLD_MINUTE = 2;
	CREEP_THRESHOLD_SECOND = 10;
}


/************************************************************************************
 Function    : void ConfigOpeartionalRegs(void)

 Description : The function initializes all "OPERATIONAL REGISTERS" For meter operation
	
 Arguments   : None
 
 Remarks     : This function is called from "main inititalization" function at the begining.
**************************************************************************************/
void ConfigOpeartionalRegs(void)
{
//***************** Opeartional registers Initialization **************************
//Initialize variables associated with the energy calculation

	METER_CONSTANT_BY_100 = METER_CONSTANT / 100;

	MODE1.byte_val =	(UINT8)(MODE1_DEF&0x00FF);		// Copy over MODE1 default register

	Flags1.bits.CycleComplete = 0;		//Clear Cycle Complete flag bit that indicates the end of 128 samples of input signal per routine
	CommFlags.byte_val=0;			// Clear CommFlags flag	

	Sw_Clk_Act.time.clk_sec = 0;	//Sw Clk emulated from Timer 0  is used in No load blinking check
	Sw_Clk_Act.time.clk_min = 0;	//Sw Clk emulated from Timer 0  is used in No load blinking check
	Sw_Clk_React.time.clk_sec = 0;	//Sw Clk emulated from Timer 0  is used in No load blinking check
	Sw_Clk_React.time.clk_min = 0;	//Sw Clk emulated from Timer 0  is used in No load blinking check
	
	PHASE_COMPENSATION_90 = PHASE_COMPENSATION_90_FREQ[15];
	PhaseComp_Amplitude_90Lag = PHASE_COMPENSATION_90 * 2;
}



/************************************************************************************
 Function    : void Init_Delay(void)

 Description : The function delays the initialization in order for the voltage to stabilize at 5V for MCP3901
	
 Arguments   : None
 
 Remarks     : Approx 1.0s delay introduced 
**************************************************************************************/
void Init_Delay(void)
{
	Delay10KTCYx(100); 					//Added delay to stabilize MCP3901 voltage
}


/************************************************************************************
 Function    : void LCD_Update(void)

 Description : The function updates the LCD with latest Energy (Active & Rreactive) alternating between
 				Active & Reactive Energy display at the rate of approx 6s. In case of any uncalibrated meter
 				or any error with energy restore, "Out CAL" message is displayed alternating with other energy
 				display
	
 Arguments   : None
 
 Remarks     : Rate of update approx 2.5s and rate of alternation approx 6s
**************************************************************************************/
#define CHANGE_LCD_DSIPLAY_STATE	0xFF
void LCD_Update(void)
{
static UINT8 Display_Alternate=0;

	if(CAL_CONTROL.bits.CAL_MODE == 0)	//If not Calibration mode
	{
		if(EEPROM_Load_Error==FALSE)
		{
			if(Display_Alternate)
			{	
				DisplayDataOnLCD(ENERGY_ACT,LCD_ActiveEnergy); // This below function will converted from Hex to BCD value.
				Display_Alternate ^= CHANGE_LCD_DSIPLAY_STATE;
			}
			else
			{
				DisplayDataOnLCD(ENERGY_REACT,LCD_ReactiveEnergy); // This below function will converted from Hex to BCD value.
				Display_Alternate ^= CHANGE_LCD_DSIPLAY_STATE;
			}
		}
		else
		{
			Display_Error();
		}
	}	
	return;
}

