/******************************************************************************

File Name:           system_EnergyMeter.c
Dependencies:    Microchip Peripheral Library
Processor:           PIC18F65J90
Company:             Microchip Technology, Inc.

Copyright 2010 Microchip Technology Inc.  All rights reserved.

Microchip licenses to you the right to use, modify, copy and distribute
Software only when embedded on a Microchip microcontroller or digital signal
controller that is integrated into your product or third party product
(pursuant to the sublicense terms in the accompanying license agreement).

You should refer to the license agreement accompanying this Software for
additional information regarding your rights and obligations.

SOFTWARE AND DOCUMENTATION ARE PROVIDED ?AS IS? WITHOUT WARRANTY OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE.
IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER
CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR
OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR
CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF
SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
(INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.

Author                                     Date                                                Revision History / Comments
------------------------------------------------------------------------------------------------------------------------
Harsha Jagadish              1 - Jan - 2011                                                    Initial Version v1.0
*******************************************************************************/

#include "system_EnergyMeter.h"

#define TERM_CHAR			'X'

void High_isr_routine(void);
void Low_isr_routine(void);

/**************High Priority Interrupt Vector Table initialization ***************************/
#pragma code High_isr=0x08
void High_ISR(void)
{
 _asm goto High_isr_routine _endasm

}


/**************Low Priority Interrupt Vector Table initialization ***************************/
#pragma code Low_isr=0x18
void Low_ISR(void)
{
_asm goto Low_isr_routine _endasm

}


/***********************************************************************************************************
 Function    :   void High_isr_routine(void)

 Include     :   Energymeter.h
 
 Description : The function serves as High Interrupt Service routine. The Routine is called on interrupt reception from 
                        (in the order of priority of service) : 
                        1. LVD Interrupt on Power Down Condition
                        2. INT0 from "Sigma Delta ADC" on data ready signal. 
	
 Arguments   : None
 
 Remarks     : The ISR is implemented in "IN LINE ASSEMBLY" for the time criticality of the execution time. As ISR will be called
                       every  1 / ( Data Ready Rate ) [ 1 / 3906 samples per second  =  256.4uS], it is quite necessary that ISR is executed with 
                       least of the possible time to reserve the processor bandwidth for other calculations & house keeping operations. 
                       The priority within High ISR depends on the order in which the interrupt flags are checked. In below part, interrupt from
                        LVD (Low Voltage Detect) on Power Down takes higher priority than INT0 data ready interrupt from SD-ADC.
		  
NOTE:          1. Before ISR code is written, it is necessary to tabulate the High Interrupt Vector table
                     2. The Compiler is directed to save only STATUS,W,PC contents. rest of the registers have to be saved manually
                          upon its usage in the in-line assembly code
***********************************************************************************************************/
#pragma interrupt High_isr_routine  nosave=section(".tmpdata"),TBLPTR, TABLAT
void High_isr_routine(void)
{

	//Check if the Interrupt occured becaues of the LVD indicating a power down.
	if(IsInterruptFromPowerDownDatect())
	{
		/* It is a power down condition. Save energy accumulated along with other data before complete drain out of charge*/
		SaveEnergyOnPowerDown();			
	}	

	//Check if the Interrupt occured becaues of the INT0 interrupt from data ready signal of "Sigma Delta ADC" upon data conversion
	else if(IsInterruptFromSDADC())
	{
		//Process SD-ADC data to process for energy calculation
		Process_SDADC_Data();
	}
	return;			//Return from ISR
}


/***********************************************************************************************************
 Function    : void Low_isr_routine(void)

 Include     : Energymeter.h
 
 Description : The function serves as Low Interrupt Service routine. The Routine is called on interrupt reception from:
		   1. TIMER0 that is used to update the time which is used to determine the time event for storage of 
		       Accumulated energy to EEPROM
		   2. Interrupt USART reception. When ever a data is recieved from GUI, the interrupt is generated
		   3. Interrupt on Transmission comletion from USART.
	
 Arguments   : None
  
 Remarks     : 1. Before ISR code is written, it is necessary to tabulate the Low Interrupt Vector table
		2. The Compiler is directed to save only STATUS,W,PC contents. rest of the registers have to be saved manually
		upon its usage in the in-line assembly code
		3. Interupt priority among Low priority interrupt
			1. TIMER0
			2. USART RX
			3. USART TX
***********************************************************************************************************/
#pragma interruptlow Low_isr_routine nosave=section(".tmpdata"),TBLPTR, TABLAT
void Low_isr_routine(void)
{
	//Check for TIMER0 interrupt flag first giving it higher priority
	if(IsInterruptFromSWTimer())
	{
		WriteTimer0(SET_TIMER0_COUNTER_1SEC);				//Re-initialize the Timer0 counter to get 1.0s time interrupt

		Sw_Clk_Act.time.clk_sec++;			//Increment seconds counter that counts time interms of multiples of 1.0 seconds
		Sw_Clk_React.time.clk_sec++;			//Increment seconds counter that counts time interms of multiples of 1.0 seconds
		if(Sw_Clk_Act.time.clk_sec == 60)		//Is 1 minute completed (1.0s x 60 = 60s)
		{
			Sw_Clk_Act.time.clk_sec = 0;		//re-initialize seconds counter	
			Sw_Clk_Act.time.clk_min++;			//Increment minute counter
		}
		if(Sw_Clk_React.time.clk_sec == 60)		//Is 1 minute completed (1.0s x 60 = 60s)
		{
			Sw_Clk_React.time.clk_sec = 0;		//re-initialize seconds counter	
			Sw_Clk_React.time.clk_min++;		//Increment minute counter
		}	
		Tmr_Seconds_since_EEP_Save++;			//increment timer for 1 day counting to save energy		
		
		ClearInterruptStatusSWTimer();	//clear Timer0 interrupt flag for further interrupt reception
	}
	else if(IsInterruptFromGUIRXInterface())		//If the LOW ISR is because of USART RX interrupt
	{
		if(COMM_RECEPTION_BUFFER)					//If so check for a valid non-zero reception value as the protocol between controller & GUI will never have raw zero transmission ever
		{
			if(RCSTAbits.OERR)					//Check if any buffer overflow occured
			{
				CommFlags.bits.RCOverrun=1;		//set the overrun flag if so
				RCSTAbits.CREN = 0;				//clear the USART buffer overrun
				RCSTAbits.CREN = 1;				//clear the USART buffer overrun
			}	
			if(CommFlags.bits.RCDone)			//check if the reception was suppose to be complete
			{
				CommFlags.bits.BufOverrun = 1;	//if so, protocol violation. set processor buffer overrun flag
			}
			if(CommFlags.bits.TXInProgress)		//check if any transmission is in progress to avoid unintended reception
			{
				CommFlags.bits.BufOverrun = 1;	//if so, protocol violation. set processor buffer overrun flag
			}
			if(CommFlags.byte_val==0)			//check for "cyclecomplete" flag. if it is middle of cycle, only then proceed for reception
			{
				*CommsBuffer++ = COMM_RECEPTION_BUFFER;		//load the recieved data from GUI through USART into RAM buffer location pointed by "CommsBuffer"
				if(COMM_RECEPTION_BUFFER==TERM_CHAR)			//check for termination charecter
				{
					CommFlags.bits.RCDone = 1;	//if so set the reception complete flag indicating no more reception untill the recieved data is processed
					CommsBuffer = temp_init;	//re-initialize the RAM buffer pointer to the begining of the buffer
				}
			}
		}
	}
	else //if the LOW INTERRUPT is because of the USART TX which is set on completion of transmission of data loaded into transmission buffer TXREG
	{
		COMM_TRANSMISSION_BUFFER = *CommsBuffer;
		if ((*CommsBuffer++==TERM_CHAR))	//check if the loaded data is the termination char
		{									//this indicates the end of transmission cycle untill next command rcieved from GUI
			DisableTransmissionToGUI();		//disable Trasnmission interrupt till next set of data has to be transmitted to GUI
			CommFlags.bits.TXInProgress= 0;	//CLEAR the transmission in progress flag to allow next reception of command from GUI
			CommsBuffer = temp_init;		//re-initialize the RAM buffer pointer to the begining of the buffer
		}
	}
	return;			//Successfully return from low ISR
}
